/*
 *  This file declares constants, structures, and functions for
 *  initializing and accessing configuration parameters.
 */

#ifndef OBF_CONFIG_H
#define OBF_CONFIG_H

#include <stdbool.h>
#include <stdio.h>

#define VERSION "3.0.0"

#define CASSERT

#define ELF

#define X86
#define MHZ 600
#define ICACHE_SIZE 16384
#define DCACHE_SIZE 17384

#define ALIGN_LOOPS_ALWAYS

// #define KERNEL



typedef struct Parameter {
    short pm_type;	// parameter type
    short pm_char;	// command option character c in -dc or -ec or -xc
    char *pm_name;	// long identifying name
    char **pm_list;	// list of legal choices, if constrained
    char *pm_sval;	// specified string value
    double pm_nval;	// specified numeric value
} Parameter;

// valid parameter types

#define PT_SPACER  0	// spacer for listings
#define PT_SWITCH  1	// miscellaneous boolean value
#define PT_OPTIM   2	// optimization switch
#define PT_DEBUG   3	// debugging switch
#define PT_INTEGER 4	// integer value
#define PT_FLOAT   5	// floating value
#define PT_STRING  6	// arbitrary string
#define PT_CHOICE  7	// constrained string (must match one of a list)



/*
 *  A generic doubly-linked list structure holding one profiling file per node.
 */
typedef struct ProfileFile {
    struct ProfileFile *pf_next;
    struct ProfileFile *pf_prev;
    char *pf_name;
    FILE *pf_file;
} ProfileFile;



void Configure(int argc, char *argv[]);
Parameter *GetParameter(char *name);
bool BooleanParameter(char *name);
long IntegerParameter(char *name);
double FloatParameter(char *name);
char *StringParameter(char *name);
char *MakeFilename(char *pattern, char *basename);



#endif				// CONFIG_H
