/*
 *  obf_global.h -- definitions of global variables and constants used by
 *                  the obfuscator in addition to the one's used in plto.
 *
 *  Also serves as a "master" include file by its inclusion
 *  of other commonly used definition files.
 */
#ifndef OBF_GLOBAL_H
#define OBF_GLOBAL_H

#ifndef OBF_GLOBAL          // if undefined, declare globals without defining
    #define OBF_GLOBAL extern
#endif

#include "global.h"
#include "x86-classes.h"



//#define NUMBER_OF_RESTORE1_COPIES 12
//#define NUMBER_OF_RESTORE2_COPIES 11
//#define NUMBER_OF_RESTORE3_COPIES 10

/*
 *  GLOBAL VARIABLES
 *
 *  Global variables should only be defined once. The hack that we use is
 *  the one used in plto, namely to prefix all of the declarations with
 *  OBF_GLOBAL. If OBF_GLOBAL is not set, then it is set to be extern. The
 *  file that should define all these global variables should set OBF_GLOBAL
 *  to be nothing.  This way, these variables are only defined once.
 */

/*
 * The offset to the base of the table containing the target addresses
 * of branch function calls.
 */
OBF_GLOBAL ADDRESS bf_target_table_offset;



/*
 * The pointer to the first basic block of the branch function called by the
 * former jumps. This function is initially created in InsertBranchFunctions,
 * because every jump converted to branch function call needs a pointer to the
 * caller, but it could not be completed with the perfect hash function until
 * all the addresses are recalculated, which happens only after all
 * modifications to the instruction stream.
 */
OBF_GLOBAL Block *bf_jump_first_bbl;



/*
 * The pointer to the first basic block of the branch function called by the
 * former calls. This function is initially created in InsertBranchFunctions,
 * because every call converted to branch function call needs a pointer to the
 * caller, but it could not be completed with the perfect hash function until
 * all the addresses are recalculated, which happens only after all
 * modifications to the instruction stream.
 */
OBF_GLOBAL Block *bf_call_first_bbl;



/*
 * The offset to the base of the table containing the target addresses
 * of traps.
 */
OBF_GLOBAL ADDRESS trap_target_table_offset;



/*
 * The array of pointers to all basic blocks of my_restore() function. This
 * function is initially created in InstallMySignalHandler(), to reserve the
 * space in the instruction stream, before the final addresses are
 * recalculated. It is created early, because my_handler() needs
 * to have a pointer to it, but it could not be completed with the perfect
 * hash function until all the addresses are recalculated, which happens only
 * after signal based and other (like junk insertion) modifications to the
 * instruction stream.
 */
OBF_GLOBAL Block *my_restore_bbls[6]; //replaced by below, get rid once done
OBF_GLOBAL LinkedList *my_restore_bbls_lists[6];



/*
 *  command-line options
 */
OBF_GLOBAL bool EXTEND_CANDIDATES;

OBF_GLOBAL bool EQUALIZE_BLOCKS;
OBF_GLOBAL bool INVERT_CONDITIONAL_BRANCHES;
OBF_GLOBAL bool INVERT_HOT_BRANCHES;
OBF_GLOBAL bool CUT_OFF_STARTS;
OBF_GLOBAL bool ISOLATE_HOT_REGIONS;


OBF_GLOBAL bool INDIRECT_CONTROL_FLOW;

OBF_GLOBAL bool SPOOF_JUMP_TABLES;
//OBF_GLOBAL bool JT_TARGET_NOTHING;
OBF_GLOBAL bool JT_TARGET_BF;
OBF_GLOBAL bool JT_TARGET_TRAP;

OBF_GLOBAL bool INSERT_BRANCH_FUNCTIONS;
OBF_GLOBAL bool BF_CONVERT_CALLS;

OBF_GLOBAL bool INSTALL_TRAPS;
OBF_GLOBAL bool TRAPS_CONVERT_CALLS;
OBF_GLOBAL bool TRAPS_CONVERT_RETURNS;

OBF_GLOBAL bool INTERLEAVE_HOT_COLD;

OBF_GLOBAL bool INSERT_JUNK;
OBF_GLOBAL bool JUMP_IN_JUNK;
OBF_GLOBAL bool JUNK_IN_JUNK;

//new code - insert 5 types of conditional jumps into junk 03/16/06
//---new code start
OBF_GLOBAL bool JUMP_IN_JUNK_NEAR_0;
OBF_GLOBAL bool JUMP_IN_JUNK_NEAR_1;
OBF_GLOBAL bool JUMP_IN_JUNK_NEAR_2;
OBF_GLOBAL bool JUMP_IN_JUNK_SHORT_0;
OBF_GLOBAL bool JUMP_IN_JUNK_SHORT_1;
OBF_GLOBAL bool JUMP_IN_JUNK_NEAR_2_SHORT_1;
OBF_GLOBAL bool JUMP_IN_JUNK_SKIP_UNCOND;
//---new code end

#endif

