/*
 *  obf_macros.h
 *
 *  This file includes various definitions, macros and shared data
 *  structures used throughout the obfuscator.
 */
#ifndef OBF_MACROS_H
#define OBF_MACROS_H



/*
 *  Evaluates to TRUE if the specified instruction is a non-indirect call,
 *  FALSE otherwise.
 */
#define INSTR_IS_DIRECT_CALL(x) \
    (isCallClass(x) && \
    !INSTR_HAS_FLAG(x, INDIRECT) && \
    !INSTR_HAS_FLAG(x, INSTR_FLIST_FLAG))



/*
 *  Evalues to TRUE if the specified instruction is a non-indirect jump,
 *  FALSE otherwise.
 */
#define INSTR_IS_DIRECT_JUMP(x) \
    (isJumpClass(x) && \
    !isConditionalInstr(x) && \
    !INSTR_HAS_FLAG(x, INDIRECT) && \
    !INSTR_HAS_FLAG(x, INSTR_FLIST_FLAG))



/*
 *  Evaluates to TRUE if the specified instruction is a conditional branch,
 *  FALSE otherwise.
 */
#define INSTR_IS_COND_JUMP(x) \
    (isJumpClass(x) && \
    isConditionalInstr(x))



/*
 *  Returns the pointer to the basic block targeted by the unconditional jump
 *  specified.
 */
#define INSTR_TARGET_BBL(x) (NODE_TO((x)->i_Bblock->b_succ))



/*
 *  Returns the pointer to the first instruction of the basic block targeted by
 *  the unconditional jump specified.
 */
#define INSTR_TARGET(x) (INSTR_TARGET_BBL(x)->b_first)

#endif                          // OBF_MACROS_H

