/*
 *  obf_print.c
 *  
 */
#include "obf_global.h"
#include "obf_print.h"
#include "sysdep.h"
//#include "x86-classes.h"



char *instrClassToString(int class);
void PrintInstructions(int which);
static void PrintObjdumpStyleInstr(Instruction *instr);
static void PrintObjdumpStyleInstrEdges(Instruction *instr);
void PrintBlocks();
void PrintFunctions();


static int *orig_instr_by_class;
static int *obf_instr_by_class;



/*
 *  GetInstrClassDistribution(when) --
 *
 *  ...
 */
void GetInstrClassDistribution(int when) {
    int *instr_by_class;
    Instruction *iiterator;

    instr_by_class = (int *)zalloc((NUMBER_OF_CLASSES + 2) * sizeof(int));

    FOREACH_INSTR(iiterator) {
        instr_by_class[NUMBER_OF_CLASSES + 1]++;
        instr_by_class[iiterator->i_class]++;
    }

    if (when == BEFORE) {
        orig_instr_by_class = instr_by_class;
    } else {
        obf_instr_by_class = instr_by_class;
    }

    return;
}



/*
 *  PrintHotFunctionNames() --
 *
 *  ...
 */
void PrintHotFunctionNames() {
    Function *fiterator;
//    Block *biterator;

    printf("Hot functions\n");

    FOREACH_FUNCTION(fiterator) {
        Block *biterator;
        bool hot = true;
        int num_blocks = 0;
        int num_instr = 0;
        FOREACH_BBL_IN_FUN(fiterator, biterator) {
            Instruction *iiterator;
            if (!BBL_HAS_FLAG(biterator, BBL_HOT_FLAG)) {
                hot = false;
                break;
            }
            num_blocks++;
            FOREACH_INSTR_IN_BBL(biterator, iiterator) {
                num_instr++;
            }
        }

        if (hot) {
            //printf("%s\n", fiterator->f_name);
            printf(" blocks: %2d, instructions: %3d, name: %s\n",
                num_blocks, num_instr, fiterator->f_name);
        }
    }
}



/*
 *  PrintInstrClassDistribution() --
 *
 *  ...
 */
void PrintInstrClassDistribution() {
    int class;

    if (!orig_instr_by_class || !obf_instr_by_class) {
        REPORT(RP_DETAIL, "No distribution collected");
        return;
    }

    for (class = 0; class < NUMBER_OF_CLASSES + 2; class++) {
        if (orig_instr_by_class[class] && obf_instr_by_class[class]) {
            printf("%-7s 0x%03x orig - %6d (%02.2f)  obf - %6d (%02.2f)\n",
                instrClassToString(class), class,
                orig_instr_by_class[class],
                (double)orig_instr_by_class[class] * 100 /
                orig_instr_by_class[NUMBER_OF_CLASSES + 1],
                obf_instr_by_class[class],
                (double)obf_instr_by_class[class] * 100 /
                obf_instr_by_class[NUMBER_OF_CLASSES + 1]);
        }
    }

    return;
}



/*
 *  instrClassToString(class) --
 *
 *  ...
 */
char *instrClassToString(int class) {
    char *result;

    switch (class) {
        case CLASS_NONE:
            result = "NONE";
            break;
        case CLASS_ADD:
            result = "ADD";
            break;
        case CLASS_AND:
            result = "AND";
            break;
        case CLASS_XOR:
            result = "XOR";
            break;
        case CLASS_INC:
            result = "INC";
            break;
        case CLASS_PUSH:
            result = "PUSH";
            break;
        case CLASS_POP:
            result = "POP";
            break;
        case CLASS_JUMP:
            result = "JUMP";
            break;
        case CLASS_TEST:
            result = "TEST";
            break;
        case CLASS_XCHG:
            result = "XCHG";
            break;
        case CLASS_MOV:
            result = "MOV";
            break;
        case CLASS_OR:
            result = "OR";
            break;
        case CLASS_SUB:
            result = "SUB";
            break;
        case CLASS_CMP:
            result = "CMP";
            break;
        case CLASS_CALL:
            result = "CALL";
            break;
        case CLASS_MUL:
            result = "MUL";
            break;
        case CLASS_DIV:
            result = "DIV";
            break;
        case CLASS_NOP:
            result = "NOP";
            break;
        case CLASS_RET:
            result = "RET";
            break;
        case CLASS_LEAVE:
            result = "LEAVE";
            break;
        case CLASS_INT:
            result = "INT";
            break;
        case CLASS_HALT:
            result = "HALT";
            break;
        case CLASS_DEC:
            result = "DEC";
            break;
        case CLASS_NEG:
            result = "NEG";
            break;
        case CLASS_ADDC:
            result = "ADDC";
            break;
        case CLASS_SUBB:
            result = "SUBB";
            break;
        case CLASS_PUSHALL:
            result = "PUSHALL";
            break;
        case CLASS_POPALL:
            result = "POPALL";
            break;
        case CLASS_IMUL:
            result = "IMUL";
            break;
        case CLASS_JA:
            result = "JA";
            break;
        case CLASS_JAE:
            result = "JAE";
            break;
        case CLASS_JB:
            result = "JB";
            break;
        case CLASS_JBE:
            result = "JBE";
            break;
        case CLASS_JCXZ:
            result = "JCXZ";
            break;
        case CLASS_JE:
            result = "JE";
            break;
        case CLASS_JG:
            result = "JG";
            break;
        case CLASS_JGE:
            result = "JGE";
            break;
        case CLASS_JL:
            result = "JL";
            break;
        case CLASS_JLE:
            result = "JLE";
            break;
        case CLASS_JNE:
            result = "JNE";
            break;
        case CLASS_JNO:
            result = "JNO";
            break;
        case CLASS_JNP:
            result = "JNP";
            break;
        case CLASS_JNS:
            result = "JNS";
            break;
        case CLASS_JO:
            result = "JO";
            break;
        case CLASS_JP:
            result = "JP";
            break;
        case CLASS_JS:
            result = "JS";
            break;
        case CLASS_LEA:
            result = "LEA";
            break;
        case CLASS_PUSHF:
            result = "PUSHF";
            break;
        case CLASS_PUSHFD:
            result = "PUSHFD";
            break;
        case CLASS_POPF:
            result = "POPF";
            break;
        case CLASS_POPFD:
            result = "POPFD";
            break;
        case CLASS_CMPS:
            result = "CMPS";
            break;
        case CLASS_STOS:
            result = "STOS";
            break;
        case CLASS_LODS:
            result = "LODS";
            break;
        case CLASS_SCAS:
            result = "SCAS";
            break;
        case CLASS_STD:
            result = "STD";
            break;
        case CLASS_CLD:
            result = "CLD";
            break;
        case CLASS_SETO:
            result = "SETO";
            break;
        case CLASS_SETNO:
            result = "SETNO";
            break;
        case CLASS_SETB:
            result = "SETB";
            break;
        case CLASS_SETAE:
            result = "SETAE";
            break;
        case CLASS_SETE:
            result = "SETE";
            break;
        case CLASS_SETNE:
            result = "SETNE";
            break;
        case CLASS_SETBE:
            result = "SETBE";
            break;
        case CLASS_SETA:
            result = "SETA";
            break;
        case CLASS_SETS:
            result = "SETS";
            break;
        case CLASS_SETNS:
            result = "SETNS";
            break;
        case CLASS_SETP:
            result = "SETP";
            break;
        case CLASS_SETNP:
            result = "SETNP";
            break;
        case CLASS_SETL:
            result = "SETL";
            break;
        case CLASS_SETGE:
            result = "SETGE";
            break;
        case CLASS_SETLE:
            result = "SETLE";
            break;
        case CLASS_SETG:
            result = "SETG";
            break;
        case CLASS_RSM:
            result = "RSM";
            break;
        case CLASS_BTS:
            result = "BTS";
            break;
        case CLASS_SHRD:
            result = "SHRD";
            break;
        case CLASS_CMPXCHG:
            result = "CMPXCHG";
            break;
        case CLASS_BTR:
            result = "BTR";
            break;
        case CLASS_MOVZB:
            result = "MOVZB";
            break;
        case CLASS_MOVZW:
            result = "MOVZW";
            break;
        case CLASS_BSR:
            result = "BSR";
            break;
        case CLASS_BSF:
            result = "BSF";
            break;
        case CLASS_MOVSW:
            result = "MOVSW";
            break;
        case CLASS_MOVSB:
            result = "MOVSB";
            break;
        case CLASS_XADD:
            result = "XADD";
            break;
        case CLASS_BSWAP:
            result = "BSWAP";
            break;
        case CLASS_BTC:
            result = "BTC";
            break;
        case CLASS_ROL:
            result = "ROL";
            break;
        case CLASS_ROR:
            result = "ROR";
            break;
        case CLASS_RCL:
            result = "RCL";
            break;
        case CLASS_RCR:
            result = "RCR";
            break;
        case CLASS_SHL:
            result = "SHL";
            break;
        case CLASS_SHR:
            result = "SHR";
            break;
        case CLASS_SAR:
            result = "SAR";
            break;
        case CLASS_NOT:
            result = "NOT";
            break;
        case CLASS_IDIV:
            result = "IDIV";
            break;
        case CLASS_CSTD:
            result = "CSTD";
            break;
        case CLASS_FADD:
            result = "FADD";
            break;
        case CLASS_FMUL:
            result = "FMUL";
            break;
        case CLASS_CPUID:
            result = "CPUID";
            break;
        case CLASS_INCL:
            result = "INCL";
            break;
        default:
            result = "*OTHER";
            break;
    }
    return result;
}



/*
 *  ObfuscatePrintStats() --
 *
 *  Outputs the statistics that can later be used to estimate the confusion
 *  achieved by obfuscation.
 *  TO DO: replace 1's with some global variables set from the command line.
 */
void ObfuscatePrintStats() {

    Section *seciterator;

    if (1) {
        PrintInstructions(PRINT_ALL);
    }

    if (1) {
        PrintInstructions(PRINT_HOT);
    }

    if (1) {
        PrintInstructions(PRINT_COLD);
    }

    if (1) {
        PrintInstructions(PRINT_ALL_REG);
    }

    if (1) {
        PrintBlocks();
    }

    if (1) {
        PrintFunctions();
    }

    if (1) {
        FOREACH_SECTION(seciterator) {
            if (strcmp(seciterator->s_name, ".text") == 0){
                printf("Text Section: old size %d %X new size %d %X\n",
                    seciterator->s_datasize, seciterator->s_datasize,
                    seciterator->s_newdatasize, seciterator->s_newdatasize);
            }
        }

        FOREACH_SECTION(seciterator) {
            if (strcmp(seciterator->s_name, ".data") == 0){
                printf("Data Section: old size %d %X new size %d %X\n",
                    seciterator->s_datasize, seciterator->s_datasize,
                    seciterator->s_newdatasize, seciterator->s_newdatasize);
            }
        }
    }

    return;
}



/*
 *  PrintInstructions(which) --
 *
 *  ...
 */
void PrintInstructions(int which) {
    Instruction *iiterator;
    int total_instrs = 0;
    int total_bytes = 0;

    if (which == PRINT_ALL) {
        REPORT(RP_DETAIL, "All Instructions-------->>>");
        FOREACH_INSTR(iiterator) {
            PrintObjdumpStyleInstrEdges(iiterator);
            total_instrs++;
            total_bytes += iiterator->i_length;
        }
        REPORT(RP_DETAIL, "End All Instructions------>>>>");
        REPORT(RP_DETAIL, "Total Number of Instructions: %d", total_instrs);
        REPORT(RP_DETAIL, "Total Size of Instructions: %d", total_bytes);
    } else if (which == PRINT_HOT) {
        REPORT(RP_DETAIL, "All Hot Instructions-------->>>");
        FOREACH_INSTR(iiterator) {
            if (BBL_HAS_FLAG(iiterator->i_Bblock, BBL_HOT_FLAG)) {
            //if (strcmp(iiterator->i_Bblock->b_fun->f_name,
            //    "ReadParse") == 0) {
                PrintObjdumpStyleInstrEdges(iiterator);
                total_instrs++;
                total_bytes += iiterator->i_length;
            }
        }
        REPORT(RP_DETAIL, "End All Hot Instructions------>>>>");
        REPORT(RP_DETAIL, "Total Number of Hot Instructions: %d", total_instrs);
        REPORT(RP_DETAIL, "Total Size of Hot Instructions: %d", total_bytes);
    } else if (which == PRINT_COLD) {
        REPORT(RP_DETAIL, "All Cold Instructions-------->>>");
        FOREACH_INSTR(iiterator) {
            if (!BBL_HAS_FLAG(iiterator->i_Bblock, BBL_HOT_FLAG)) {
            //if (strcmp(iiterator->i_Bblock->b_fun->f_name,
            //    "ReadParse") != 0) {
                PrintObjdumpStyleInstrEdges(iiterator);
                total_instrs++;
                total_bytes += iiterator->i_length;
            } 
        }
        REPORT(RP_DETAIL, "End All Cold Instructions------>>>>");
        REPORT(RP_DETAIL, "Total Number of Cold Instructions: %d",
            total_instrs);
        REPORT(RP_DETAIL, "Total Size of Cold Instructions: %d", total_bytes);
    } else {
        REPORT(RP_DETAIL, "All Distr Instructions-------->>>");
        FOREACH_INSTR(iiterator) {
            PrintObjdumpStyleInstr(iiterator);
            total_instrs++;
            total_bytes += iiterator->i_length;
        }
        REPORT(RP_DETAIL, "End All Distr Instructions------>>>>");
        REPORT(RP_DETAIL, "Total Number of Instructions: %d", total_instrs);
        REPORT(RP_DETAIL, "Total Size of Instructions: %d", total_bytes);
    }

    return;
}



/*
 *  PrintObjdumpStyleInstrEdges(instr) --
 *
 *  ...
 */
void PrintObjdumpStyleInstrEdges(Instruction *instr) {

    int i = 0;

    printf("%8x:", instr->i_final_address);
/*
    for (i = 0; i < INSTR_LENGTH(instr); i++) {
        printf( " %.2x", SECTION_NEWDATA(SECTION_REF_CODE)[INSTR_FINAL_ADDRESS(instr) - SECTION_NEWADDRESS(SECTION_REF_CODE) + i]);
    }
*/
    for (i = 0; i < instr->i_length; i++) {
        //smth wrong here, look later
        printf( " %.2x", (SECTION_REF_CODE->s_newdata)[instr->i_final_address
            - SECTION_REF_CODE->s_newaddress + i]);
    }

    i = 3 * i + 9;
    for (; i < 32; i++) {
        printf(" ");
    }

    if (instr->i_length > 7) {
        printf("   ");
    }

    if (INSTR_HAS_FLAG(instr, INSTR_LITERAL_BYTES_FLAG)) {
        if (INSTR_HAS_FLAG(instr, INSTR_SIGILL_FLAG)) {
            printf("--Illegal Instruction--\n");
        } else {
            printf("--Literal Bytes--\n");
        }
    } else if (INSTR_HAS_FLAG(instr, INSTR_JUNK_FLAG)) {
        printf("--Junk Instruction--\n");
    } else if ((instr->i_Bblock->b_last == instr) &&
        ((instr->i_Bblock->b_type == BT_UNCOND) ||
        (instr->i_Bblock->b_type == BT_SWITCH))) {
        printf("--Original Jump--\n");
    } else if ((instr->i_Bblock->b_last == instr) &&
        (instr->i_Bblock->b_type == BT_BRANCH)) {
        printf("--Original Branch--\n");
    } else if (INSTR_HAS_FLAG(instr, INSTR_CLASS_RETURN)) {
        printf("--Original Return--\n");
    } else if (INSTR_HAS_FLAG(instr, INSTR_CLASS_CALL)) {
        printf("--Original Call--\n");
    } else if (INSTR_HAS_FLAG(instr, INSTR_CLASS_SYSCALL)) {
        printf("--Original Syscall--\n");
    } else {
        printf("%-19s og(%x)\n", PrintInstruction(instr), instr->i_address);
    }

    return;
}



/*
 *  PrintObjdumpStyleInstr(instr) --
 *
 *  ...
 */
void PrintObjdumpStyleInstr(Instruction *instr) {

    int i = 0;

    printf("%8x:", instr->i_final_address);
/*
    for (i = 0; i < INSTR_LENGTH(instr); i++) {
        printf( " %.2x", SECTION_NEWDATA(SECTION_REF_CODE)[INSTR_FINAL_ADDRESS(instr) - SECTION_NEWADDRESS(SECTION_REF_CODE) + i]);
    }
*/
    for (i = 0; i < instr->i_length && i < 7 ; i++) {
        //smth wrong here, look later
        printf( " %.2x", (SECTION_REF_CODE->s_newdata)[instr->i_final_address
            - SECTION_REF_CODE->s_newaddress + i]);
    }

    i = 3 * i + 9;
    for (; i < 32; i++) {
        printf(" ");
    }

    if (instr->i_length > 7) {
//        printf("   ");
    }

    if (INSTR_HAS_FLAG(instr, INSTR_LITERAL_BYTES_FLAG)) {
        if (INSTR_HAS_FLAG(instr, INSTR_SIGILL_FLAG)) {
            printf("illeg Instruction--\n");
        } else {
            printf("--Literal Bytes--\n");
        }
    } else {
        printf("%-19s og(%x)\n", PrintInstruction(instr), instr->i_address);
    }

    return;
}



/*
 *  PrintBlocks() --
 *
 *  For every basic block prints its boundaries - the address of its first
 *  instruction and the address of its last instruction.
 */
void PrintBlocks() {
    int bbls = 0;
    int hot_bbls = 0;
    int cold_bbls = 0;
    int zero_bbls = 0;
    Block *biterator;
    Instruction *first;
    Instruction *last;

    FOREACH_BBL(biterator) {

        if (BBL_HAS_FLAG(biterator, BBL_HOT_FLAG)) {
            hot_bbls++;
        } else if (BBL_HAS_FLAG(biterator, BBL_COLD_FLAG)) {
            cold_bbls++;
        } else if (BBL_HAS_FLAG(biterator, BBL_ZERO_FLAG)) {
            zero_bbls++;
        }

        bbls++;

        first = biterator->b_first;
        last = biterator->b_last;

        if (first && last) {
            printf("[BBL| FirstInstr: 0x%x, LastInstr: 0x%x]\n",
                first->i_final_address, last->i_final_address);
        }
    }

    REPORT(RP_DETAIL, "End All Basic Blocks------>>>>");
    REPORT(RP_DETAIL, "Total Hot Basic Blocks:       %d", hot_bbls);
    REPORT(RP_DETAIL, "Total Cold Basic Blocks:      %d", cold_bbls);
    REPORT(RP_DETAIL, "Total Zero Basic Blocks:      %d", zero_bbls);
    REPORT(RP_DETAIL, "Total Number of Basic Blocks: %d", bbls);

    return;
}



/*
 *  PrintFunctions() --
 *
 *  For every function prints its boundaries - the address of its first
 *  instruction and the address of its last instruction.
 */
void PrintFunctions() {
    Function *fiterator;

    FOREACH_FUN(fiterator) {
        printf( "[FUNCTION| FirstInstr: 0x%x, LastInstr: 0x%x]\n",
            fiterator->f_first->b_first->i_final_address,
            fiterator->f_last->b_last->i_final_address);
    }

    return;
}



/*
 *  PrintControlTransferEdges() --
 *
 *  Prints out the number of control transfer instructions by type and the
 *  total number of the control transfer edges.
 */
void PrintControlTransferEdges() {
    Block *biterator;
    int num_jumps = 0;
    int num_branches = 0;
    int num_returns = 0;
    int num_calls = 0;
    int num_syscalls = 0;
    int num_control_transfer_edges;

    FOREACH_BBL(biterator) {
        //if (INSTR_HAS_FLAG(biterator->b_last, INSTR_CLASS_JUMP)) {
        if ((biterator->b_type == BT_UNCOND) ||
            (biterator->b_type == BT_SWITCH)) {
            num_jumps++;
        //} //else if (INSTR_HAS_FLAG(biterator->b_last, INSTR_CLASS_BRANCH)) {
        } else if (biterator->b_type == BT_BRANCH) {
            num_branches++;
        } else if (INSTR_HAS_FLAG(biterator->b_last, INSTR_CLASS_RETURN)) {
            num_returns++;
        } else if (INSTR_HAS_FLAG(biterator->b_last, INSTR_CLASS_CALL)) {
            num_calls++;
        } else if (INSTR_HAS_FLAG(biterator->b_last, INSTR_CLASS_SYSCALL)) {
            num_syscalls++;
        }
    }

    num_control_transfer_edges = num_jumps + 2 * num_branches +
        //num_returns +
        2 * num_calls + 2 * num_syscalls;

    REPORT(RP_DETAIL, "Edge statistics:");
    REPORT(RP_DETAIL, "%10d jumps\n", num_jumps);
    REPORT(RP_DETAIL, "%10d branches\n", num_branches);
    REPORT(RP_DETAIL, "%10d returns\n", num_returns);
    REPORT(RP_DETAIL, "%10d calls\n", num_calls);
    REPORT(RP_DETAIL, "%10d syscalls\n", num_syscalls);
    REPORT(RP_DETAIL, "%10d total control transfer edges\n",
        num_control_transfer_edges);

    return;
}

