#ifndef ELF_CONSTANTS_H
#define ELF_CONSTANTS_H

#include "config.h"

/*
 *  The constant for the ELF section type corresponding to an intrusion
 *  detection system inserted by PLTO.
 */
#define SHT_IDS 20

/*
 *  The the offset from an address to the PC value used for its relocation.
 */
#define PC32_OFFSET 4

/*
 *  The page size for ELF files.
 */
#define MAXPAGESIZE 0x1000

/*
 *  The padding value to be used between sections or functions.
 */
#define PAD 0x00

/*
 *  These sections are sections of particular interest to PLTO.
 *  If this is being ported to another architecture these may
 *  need to be changed.
 */
#define TEXT_SECTION_NAME ".text"
#define CODE_SECTION_NAME ".text"
#define DATA_SECTION_NAME ".data"
#define RODATA_SECTION_NAME ".rodata"
#define BSS_SECTION_NAME ".bss"
#define GOT_SECTION_NAME ".got"
#define SYM_SECTION_NAME ".symtab"

#define GOT_SECTION_SIZE 0xc
#define RELOCATION_SECTION_PREFIX ".rel"
#define DEBUGGING_SECTION_PREFIX ".debug"

#define ENTRY_SYMBOL_NAME "_start"
#define EXIT_SYMBOL_NAME "_exit"

#define GOT_NAME "_GLOBAL_OFFSET_TABLE_"
#define SECTION_HEADER_STRING_TABLE_NAME ".shstrtab"
#define SECTION_SYMBOL_TABLE_NAME ".symtab"
#define SECTION_SYMBOL_STRING_TABLE_NAME ".strtab"

#define CIDENT_SECTION_START_PREFIX "__start_"
#define CIDENT_SECTION_STOP_PREFIX "__stop_"

/* These were symbols defined in the kernel
 * could be encountered anywhere in the application
 * too - SOMU. 
 */
#define END_OF_TEXT_SYMBOL_NAME "_etext"
#define END_OF_DATA_SYMBOL_NAME "_edata"
#define END_OF_BSS_SYMBOL_NAME  "_end"

#endif
