/*
 *  file.h -- structures and macros for the basic
 *  reading/parsing of Elf32 relocatable binaries. Much of this already is OK
 *  for or could easily be generalized to also read executable binaries or even
 *  dynamically linked binaries, but for our purposes this should be sufficient.
 */
#ifndef ELF_READ_H
#define ELF_READ_H

#include <elf.h>
#include "global.h"

/*
 *  This structure is a basic representation of an ELF binary. It is mostly
 *  just a wrapper of structs around the raw bytes of an ELF file contained
 *  in file_data.
 */
typedef struct Elf32File {
    Elf32_Ehdr *ef_hdr;		  // Elf header
    Elf32_Shdr *ef_shdr_tbl;	  // section header table
    Elf32_Shdr *ef_sym_hdr;	  // symbol table section header
    Elf32_Sym *ef_symt;		  // symbol table
    char *ef_strn_tbl;		  // section header string table
    char *ef_sym_strn_tbl;	  // symbol table string table
    UINT8 *ef_file_data;	  // raw file data
    int ef_file_size;		  // size of the raw data in bytes
    BINARY_TYPE ef_type;	  // type of the input binary
    Section **ef_sect_map;	  // table mapping section numbers to structs
} Elf32File;

/*
 *  A macro to access the name of a given symbol (Elf32_Sym *) in an Elf32File.
 */
#define ELF32_SYMBOL_NAME(elf_file, sym) \
    (&(elf_file)->ef_sym_strn_tbl[sym->st_name])

/*
 *  A macro for iterating over each section header (Elf32_Shdr *) in an
 *  Elf32File.
 */
#define FOREACH_ELF32_SECTION(elf_file, iterator) \
    int indexval; \
    for (indexval = 1, iterator = &(elf_file)->ef_shdr_tbl[indexval]; \
	indexval < (elf_file)->ef_hdr->e_shnum; \
	indexval++, iterator = &(elf_file)->ef_shdr_tbl[indexval])

/*
 *  A macro for iterating over each symbol (Elf32_Sym *) in an Elf32File.
 */
#define FOREACH_ELF32_SYMBOL(elf_file, iterator) \
    int indexval; \
    int indexlimit = (elf_file)->ef_sym_hdr->sh_size / \
    (elf_file)->ef_sym_hdr->sh_entsize; \
    for (indexval = 1, iterator = &(elf_file)->ef_symt[indexval]; \
	indexval < indexlimit; \
	indexval++, iterator = &(elf_file)->ef_symt[indexval])

#endif
