/*
 *  print.c
 */
#include "protos.h"



/*
 *  PrintSectionHeader(shdr,name) --
 */
void PrintSectionHeader(Elf32_Shdr *shdr, char *name) {
    printf("%s {\n", name);
    printf("  sh_type:      0x%x\n", shdr->sh_type);
    printf("  sh_flags:     0x%x\n", shdr->sh_flags);
    printf("  sh_addr:      0x%x\n", shdr->sh_addr);
    printf("  sh_offset:    0x%x\n", shdr->sh_offset);
    printf("  sh_size:      0x%x\n", shdr->sh_size);
    printf("  sh_link:      0x%x\n", shdr->sh_link);
    printf("  sh_info:      0x%x\n", shdr->sh_info);
    printf("  sh_addralign: 0x%x\n", shdr->sh_addralign);
    printf("  sh_entsize:   0x%x\n", shdr->sh_entsize);
    printf("}\n");
}



/*
 *  PrintSection(index,shdr,strn_tbl,tab) --
 */
void PrintSection(int index, Elf32_Shdr *shdr, char *strn_tbl, char *tab) {
    printf("%s%3d %-25s %8x %8x %8x\n",
	tab,
	index,
	&strn_tbl[shdr->sh_name],
	shdr->sh_offset, shdr->sh_size, shdr->sh_offset + shdr->sh_size - 1);
}



/*
 *  PrintSections(elf_file) --
 */
void PrintSections(Elf32File *elf_file) {
    int i;

    printf("%3s %-25s %-8s %-8s %-8s\n", "Idx", "Name", "File Off", "Size",
	"End Off");

    // print the regular sections
    for (i = 0; i < elf_file->ef_hdr->e_shnum; i++) {
	if (elf_file->ef_shdr_tbl[i].sh_type == SHT_REL
	    || elf_file->ef_shdr_tbl[i].sh_type == SHT_NULL) {
	    continue;
	} else if (strcmp(".symtab",
		&elf_file->ef_strn_tbl[elf_file->ef_shdr_tbl[i].sh_name]) ==
	    0) {
	    continue;
	} else if (strcmp(".strtab",
		&elf_file->ef_strn_tbl[elf_file->ef_shdr_tbl[i].sh_name]) ==
	    0) {
	    continue;
	}

	PrintSection(i, &elf_file->ef_shdr_tbl[i], elf_file->ef_strn_tbl, "");
    }

    // print the relocation sections
    for (i = 0; i < elf_file->ef_hdr->e_shnum; i++) {
	if (elf_file->ef_shdr_tbl[i].sh_type == SHT_REL) {
	    PrintSection(i,&elf_file->ef_shdr_tbl[i],elf_file->ef_strn_tbl,"");
	}
    }

    // print the symbol table and symbol string table
    for (i = 0; i < elf_file->ef_hdr->e_shnum; i++) {
	if (strcmp(".symtab",
		&elf_file->ef_strn_tbl[elf_file->ef_shdr_tbl[i].sh_name]) ==
	    0) {
	    PrintSection(i, &elf_file->ef_shdr_tbl[i], elf_file->ef_strn_tbl,
		"");
	} else if (strcmp(".strtab",
		&elf_file->ef_strn_tbl[elf_file->ef_shdr_tbl[i].sh_name]) ==
	    0) {
	    PrintSection(i, &elf_file->ef_shdr_tbl[i], elf_file->ef_strn_tbl,
		"");
	}
    }
}



/*
 *  PrintElfSymbol(elf_file,symbol) --
 */
void PrintElfSymbol(Elf32File *elf_file, Elf32_Sym *symbol) {
    printf("%8x l ", symbol->st_value);

    switch (ELF32_ST_TYPE(symbol->st_info)) {
    case STT_FUNC:	printf("%2s", "F");	break;
    case STT_NOTYPE:	printf("%2s", "");	break;
    case STT_OBJECT:	printf("%2s", "O");	break;
    case STT_SECTION:	printf("%2s", "d");	break;
    case STT_FILE:	printf("%2s", "df");	break;
    default:		printf("%2s", "Wh");	break;
    }

    printf(" ");

    switch (symbol->st_shndx) {
    case SHN_LORESERVE:		// also SHN_LOPROC
	printf("LORESERVE/LOPROC");
	break;
    case SHN_HIPROC:
	printf("HIPROC");
	break;
    case SHN_ABS:
	printf("*ABS*");
	break;
    case SHN_HIRESERVE:
	printf("HIRESERVE");
	break;
    case SHN_UNDEF:
	printf("*UND*");
	break;
    case SHN_COMMON:
	printf("*COM*");
	break;
    default:
	printf("%s",
	    &elf_file->ef_strn_tbl[elf_file->ef_shdr_tbl[symbol->st_shndx].
		sh_name]);
    }

    printf(" %8x %s\n", symbol->st_size,
	&elf_file->ef_sym_strn_tbl[symbol->st_name]);
}



/*
 *  PrintElfSymbols(elf_file) --
 */
void PrintElfSymbols(Elf32File *elf_file) {
    Elf32_Sym *cur_sym;

    FOREACH_ELF32_SYMBOL(elf_file, cur_sym) {
	PrintElfSymbol(elf_file, cur_sym);
    }
}



/*
 *  PrintRelocation(elf_file,shdr,reloc,pad) --
 */
void PrintRelocation(Elf32File *elf_file, Elf32_Shdr *shdr, Elf32_Rel *reloc,
    char *pad) {
    Elf32_Sym *symbol;

    symbol = &elf_file->ef_symt[ELF32_R_SYM(reloc->r_info)];

    printf("%-s", pad);

    if (ELF32_ST_TYPE(symbol->st_info) == STT_SECTION) {
	printf("%-25s ",
	    &elf_file->ef_strn_tbl[elf_file->ef_shdr_tbl[symbol->st_shndx].
		sh_name]);
    } else {
	printf("%-25s ", &elf_file->ef_sym_strn_tbl[symbol->st_name]);
    }

    printf("%8x %8x %8x %4x\n",
	reloc->r_offset,
	elf_file->ef_file_data[shdr->sh_offset + reloc->r_offset],
	ELF32_R_SYM(reloc->r_info), ELF32_R_TYPE(reloc->r_info));
}



/*
 *  PrintRelocations(elf_file) --
 */
void PrintRelocations(Elf32File *elf_file) {
    Elf32_Shdr *cur_shdr;
    int num_ents, i;
    Elf32_Rel *cur_reloc, *relocs;

    FOREACH_ELF32_SECTION(elf_file, cur_shdr) {
	if (cur_shdr->sh_type == SHT_REL) {
	    PrintSection(indexval, cur_shdr, elf_file->ef_strn_tbl, "Section ");
	    printf("  %-25s %-8s %-8s %-8s %s\n", "Symbol", "Offset", "Addend",
		"SymIndx", "Type");
	    num_ents = cur_shdr->sh_size / cur_shdr->sh_entsize;
	    relocs = (Elf32_Rel *) &elf_file->ef_file_data[cur_shdr->sh_offset];

	    for (i = 0; i < num_ents; i++) {
		cur_reloc = &relocs[i];
		PrintRelocation(elf_file,
		    &elf_file->ef_shdr_tbl[cur_shdr->sh_info], cur_reloc, "  ");
	    }
	}
    }
}



/*
 *  PrintElfHeader(elf_hdr) --
 */
void PrintElfHeader(Elf32_Ehdr *elf_hdr) {
    printf("{\n\t e_ident      %s \n", elf_hdr->e_ident);
    printf("\t e_type         0x%x \n", elf_hdr->e_type);
    printf("\t e_machine      0x%x \n", elf_hdr->e_machine);
    printf("\t e_version      0x%x \n", elf_hdr->e_entry);
    printf("\t e_entry        0x%x \n", elf_hdr->e_entry);
    printf("\t e_phoff        0x%x \n", elf_hdr->e_phoff);
    printf("\t e_shoff        0x%x \n", elf_hdr->e_shoff);
    printf("\t e_flags        0x%x \n", elf_hdr->e_flags);
    printf("\t e_ehsize       0x%x \n", elf_hdr->e_ehsize);
    printf("\t e_phentsize    0x%x \n", elf_hdr->e_phentsize);
    printf("\t e_phnum        0x%x \n", elf_hdr->e_phnum);
    printf("\t e_shentsize    0x%x \n", elf_hdr->e_shentsize);
    printf("\t e_shnum        0x%x \n", elf_hdr->e_shnum);
    printf("\t e_shstrndx     0x%x \n }\n", elf_hdr->e_shstrndx);
}
