/*
 *  protos.h -- prototypes for ELF handling functions
 */

#ifndef PROTOS_H
#define PROTOS_H

#include <elf.h>
#include "config.h"
#include "global.h"
#include "elfio.h"
#include "upm.h"

#include "file.h"



/*  convert.c  */

Section *GetPltoSections(Elf32File *elf_file);
void SetupPltoDataStructures();
void FillElf32_ShdrFromPltoFlags(Elf32_Shdr *shdr, UINT32 plto_flags);
UINT32 GetPltoFlagsFromElf32_Shdr(Elf32_Shdr *elf_shdr);
void FillElf32_SymFromPltoFlags(Elf32_Sym *sym, UINT32 plto_flags);
UINT16 BinaryTypeToElfType(BINARY_TYPE type);
UINT32 GetPltoFlagsFromElf32_Rel(Elf32_Rel *elf_rel);



/*  read.c  */

Elf32File *GetInitializedBinary(char *infile_name);



/*  relocate.c  */

void ApplyElfRelocations(Relocation *relocations);
void ApplyRelocation_32(Relocation *relocation);
void ApplyRelocation_PC32(Relocation *rel);
void ApplyRelocation_GOT32(Relocation *rel);
void ApplyRelocation_PLT32(Relocation *rel);
void ApplyRelocation_COPY(Relocation *rel);
void ApplyRelocation_GLOB_DAT(Relocation *rel);
void ApplyRelocation_JMP_SLOT(Relocation *rel);
void ApplyRelocation_RELATIVE(Relocation *rel);
void ApplyRelocation_GOTOFF(Relocation *rel);
void ApplyRelocation_GOTPC(Relocation *rel);
void UpdateBSSForSymbols();
void UpdateGOTForRelocations();
void InitializeUPMs();
void GetFakeRelocationInfoForUPM(Patch *upm, UINT32 *type, ADDRESS *offset,
    Section **section);



/*  print.c */

void PrintSectionHeader(Elf32_Shdr *shdr, char *name);
void PrintElfHeader(Elf32_Ehdr *elf_hdr);
void PrintSection(int index, Elf32_Shdr *shdr, char *strn_tbl, char *tab);
void PrintSections(Elf32File *elf_file);
void PrintElfSymbol(Elf32File *elf_file, Elf32_Sym *symbol);
void PrintElfSymbols(Elf32File *elf_file);
void PrintRelocation(
    Elf32File *elf_file, Elf32_Shdr *shdr, Elf32_Rel *reloc, char *pad);
void PrintRelocations(Elf32File *elf_file);



/*  utils.c  */

Section *CreateGOTSection();
Symbol *GetElfSymbol(Symbol *symbol_list, char *symbol_name);
void SetupSectionNewData(Section *section_list);
bool IsIgnoredElfSection(Elf32File *elf_file, int shdr_index);
ADDRESS AddGotEntry(ADDRESS address);
void WriteFileData(char *filename, UINT8 *data, int data_size);
char *RelocationTypeToString(int type);
void SortSectionsByWriteOrder();
int GetSectionOrderScore(Section *section);
bool IsDebuggingInfoSection(Elf32File *elf_file, int shdr_index);
bool IsValidCIdentifier(char *name);



/*  write.c  */

ADDRESS ComputeFirstSectionAddress();



#endif				// PROTOS_H
