#ifndef _NDG_H_
#define _NDG_H_

typedef unsigned long long WINT;

typedef struct _elem_list {
    void *data;
    int idx;   /* elem idx */
    struct _elem_list *next;
    struct _elem_list *prev;
}NDGElem;

typedef struct _node_list {
    struct _elem_list *elems;
    int idx;
    struct _node_list *next;
}NDGNode;

typedef struct _edge_list {
    int id;
    struct _node_list * v1;
    struct _node_list * v2;
    WINT weight;
    struct _edge_list *next;
}NDGEdge;

/* non-directed graph */
typedef struct _ndg {
    struct _node_list *nodes;
    int num_nodes;
    struct _node_list **nodes_index; /* used for quick indexing node */

    struct _edge_list *edges; /* list of weighted edges */
    int num_edges;
    WINT **Weight;     /* used for doing sorting */
    WINT **OrigWeight; /* keep the original edge weights in the graph */
    int table_size;
    
    int (*data_cmp) (void *d1, void *d2);
    void (*print_data) (void *data);
}NDG;

NDG *NDGCreate(void);
void NDGAddEdge(NDG *graph, void *data1, void *data2, WINT weight);
void NDGInitWeight(NDG *graph);
void NDGPrintGraph(NDG *graph);
void NDGSort(NDG *graph);
NDGNode *FindNode(NDG *graph, void *data);

#endif
