/*
 *  cp.h
 *
 *  Constant Propagator
 */
#ifndef CP_H
#define CP_H

#include "global.h"
#include "sysdep.h"

/*
 *  Value
 *  A value slot models a slot on the stack or the contents of a register.
 */
typedef struct Value {
    UINT8 v_type;
    ADDRESS v_value;
    bool v_live;
    UINT16 v_flags;
    INT32 v_stackOffset;
} Value;

#define SLOT_ADDRESS_FLAG       (1 << 0)	// slot contains an address
#define SLOT_EAX_FLAG           (1 << 1)	// slot aliases register %eax
#define SLOT_EBX_FLAG           (1 << 2)	// slot aliases register %ebx
#define SLOT_ECX_FLAG           (1 << 3)	// slot aliases register %ecx
#define SLOT_EDX_FLAG           (1 << 4)	// slot aliases register %edx
#define SLOT_ESI_FLAG           (1 << 5)	// slot aliases register %esi
#define SLOT_EDI_FLAG           (1 << 6)	// slot aliases register %edi
#define SLOT_EBP_FLAG           (1 << 7)	// slot aliases register %ebp
#define SLOT_ESP_FLAG           (1 << 8)	// slot aliases register %esp
#define SLOT_STACK_FLAG         (1 << 9)	// slot aliases a stack locn

#define SLOT_SET_FLAG(slot, flg)     (((slot)->v_flags) |= (flg))
#define SLOT_HAS_FLAG(slot, flg)     (bool)(((slot)->v_flags) & (flg))
#define SLOT_REMOVE_FLAG(slot, flg)  (((slot)->v_flags) &= ~(flg))
#define SLOT_SET_FLAG_DIRECT(slot, flg)     (((slot).v_flags) |= (flg))
#define SLOT_HAS_FLAG_DIRECT(slot, flg)     (((slot).v_flags) & (flg))
#define SLOT_REMOVE_FLAG_DIRECT(slot, flg)  (((slot).v_flags) &= ~(flg))

#define REGISTER_INDEX_EAX           0
#define REGISTER_INDEX_EBX           1
#define REGISTER_INDEX_ECX           2
#define REGISTER_INDEX_EDX           3
#define REGISTER_INDEX_ESP           4
#define REGISTER_INDEX_EBP           5
#define REGISTER_INDEX_EDI           6
#define REGISTER_INDEX_ESI           7

#define SLOT_TYPE_ERROR              0
#define SLOT_TYPE_UNDEFINED          1
#define SLOT_TYPE_UNKNOWN            2
#define SLOT_TYPE_CONSTANT           3


/*
 *  We don't want to model functions with activation records that are too large.
 *  MAXIMUM_AR_SIZE is the largest we will attempt to model
 */
#define MAXIMUM_AR_SIZE              10000

/* If we can't determine the size of the stack, we'll set it to this size */
#define UNKNOWN_STACK_SIZE             400

/* Set to 1 if addresses should be propagated, 0 if they shouldn't */
#define PROPAGATE_ADDRESSES            1

/*
 *  PropagateConstants() will move constant values through the registers and
 *  the stack to try and pre-evaluate instructions, and simplify instructions
 */
void PropagateConstants();

/*
 *  Allocates stack and register snapshots for basic blocks, and sets their
 *  flags if allocation was successful
 */
void AllocateSnapshots();

/* Allocates a snapshot structure for the stackframe of this function */
void BBLAllocateStackshot(Block *bbl);

/* Allocates a snapshot structure for the general purpose registers */
void BBLAllocateRegshot(Block *bbl);

/* Returns true if 'instr' defines no memory or registers */
bool InstrDefinesNothing(Instruction *instr);

/* Returns the register (see DEFINES above) given index into the bitvector */
int IndexToRegister(int idx);

/* Returns the index into a register snapshot of the register 'aReg' */
int RegisterToIndex(int aReg);


#endif				// CP_H
