/*
 *  This file contains stuff specific to the Edge and
 *  EdgeNode data structures. These data structures are used to
 *  represent control flow relationships between basic blocks.
 */
#ifndef EDGE_H
#define EDGE_H

#include "plto_types.h"
#include "bbl.h"

/*
 *  STRUCTURES
 *
 *  A generic doubly-linked list structure holding one edge per node.
 */
typedef struct EdgeNode {
    struct EdgeNode *en_next;	   // next node in the list
    struct EdgeNode *en_prev;	   // previous node in the list
    struct Edge *en_edge;	   // shared edge associated with this node
    UINT32 en_id;		   // ID number for this node
} EdgeNode;

/* Macros for accessing fields in the nodes */

/* Macros to make using edges easier. */
#define NODE_TO(node)            ((node)->en_edge->e_to)
#define NODE_FROM(node)          ((node)->en_edge->e_from)
#define NODE_TYPE(node)          ((node)->en_edge->e_type)
#define NODE_JT_SLOT(node)       ((node)->en_edge->e_jumpTableSlot)
#define NODE_WEIGHT(node)        ((node)->en_edge->e_edgeWeight)

/*
 *  The representation of an edge, with source and destination pointers.
 */
typedef struct Edge {
    struct Edge *e_next;	   // next edge; used when sorting
    struct Edge *e_prev;	   // previous edge; used when sorting
    Block *e_to;		   // basic block pointed to by this edge
    Block *e_from;		   // bbl where this edge originates
    UINT16 e_flags;		   // flags
    ETYPE e_type;		   // type
    UINT32 e_id;		   // ID number for this edge
    EDGE_WEIGHT_TYPE e_edgeWeight; // weight of the edge
    int e_jumpTableSlot;	   // original jumptable slot (for ET_SWITCH)
} Edge;

/* Macros for accessing fields in the edges */
#define EDGE_HAS_FLAG(edge,z)    ((bool)(((edge)->e_flags) & z))
#define EDGE_SET_FLAG(edge,z)    (((edge)->e_flags) |= z)
#define EDGE_REMOVE_FLAG(edge,z) (((edge)->e_flags) &= (~z))

/*
 * Structure for edge profiling.
 */
typedef struct ProfConfig {
    ADDRESS pc_initStart;
    ADDRESS pc_counterStart;
    ADDRESS pc_currentStart;
    int pc_bblsProf;
    int pc_edgsProf;
    int pc_callsProf;
    int pc_jmpsProf;
    int pc_valsProf;
} ProfConfig;

#endif
