/*
 *  flags.h
 *
 *  Bit flags for various structures
 */
#ifndef FLAGS_H
#define FLAGS_H

/*
 *  flags.h
 *
 *  Flags operands, instruction, symbols, sections, edges, functions and bbls
 *
 *  Operand Flags
 */
#define OPERAND_BASE_FLAG    (1 << 0)  // operand has base register
#define OPERAND_DISP_R_FLAG  (1 << 1)  // operand has relative displacement
#define OPERAND_INDEX_FLAG   (1 << 2)  // operand has index register
#define OPERAND_SCALE_FLAG   (1 << 3)  // operand has immediate scale value
#define OPERAND_IMM_FLAG     (1 << 4)  // operand has immed non-scale value
#define OPERAND_DISP_A_FLAG  (1 << 5)  // operand has absolute displacement
#define OPERAND_VMODE_FLAG   (1 << 6)  // operand is in v_mode
#define OPERAND_BMODE_FLAG   (1 << 7)  // operand is in b_mode
#define OPERAND_WMODE_FLAG   (1 << 8)  // operand is in w_mode
#define OPERAND_DMODE_FLAG   (1 << 9)  // operand is in d_mode

/*
 *  The following flags represent the functions for
 *  handling operand types/combinations
 */
#define OPERAND_OPG_FLAG               (1 << 11)
#define OPERAND_OPREG_FLAG             (1 << 12)
#define OPERAND_REGISTER_INDIRECT      (1 << 13)
#define OPERAND_SIGN_EXTEND_FLAG       (1 << 14)
#define OPERAND_SIDI_FLAG              (1 << 15)
#define OPERAND_REGISTER_MODRM         (1 << 16)
#define OPERAND_INDIRECT_FLAG          (1 << 17)
#define OPERAND_RELOCATABLE_PC_FLAG    (1 << 18)
#define OPERAND_RELOCATABLE_ABS_FLAG   (1 << 19)
/* Set if a relocation was put onto this operand. */
#define OPERAND_RELOCATED              (1 << 20)
#define OPERAND_RELOCATABLE_GOTPC_FLAG (1 << 21)

/* The operand represents a CONTROL Register CR0-CR4 */
#define OPERAND_CONTROL_REG_FLAG       (1 << 23)

/* The operand represents a Segment Register (ES, CS, SS, DS, FS, GS) */
#define OPERAND_SEGMENT_REG_FLAG       (1 << 24)
#define OPERAND_DEBUG_REG_FLAG	       (1 << 25)

#define OPERAND_AUTOLENGTH_RELOCATED   (1 <<26)


#define OPERAND_NOCLONE_FLAGS          0

/*
 *  Instruction Flags
 */
/* If the instruction is indirect, e.g jmp *0xf00 */
#define INDIRECT                      (1LL << 0)
/*
 *  If the instruction begins a basic block.  Note that some basic block
 *  should have its firstInstr field pointing to this instruction
 */
#define BBL_START                     (1LL << 1)
/*
 *  If the instruction ends a basic block. Note that some basic block
 *  should have its lastInstr field pointing to this instruction
 */
#define BBL_END                       (1LL << 2)
/* If this instruction starts a function */
#define FUN_START                     (1LL << 3)
/* If this is the last instruction in a function */
#define FUN_END                       (1LL << 4)
/* If the MODR/M byte is used as an extension to the opcode */
#define EXTENDED_OPCODE               (1LL << 5)
/* If the operand has a source register */
#define SOURCE1_FLAG                  (1LL << 6)
/* If the operand has a second source register */
#define SOURCE2_FLAG                  (1LL << 7)
/* If the operand has a destination register */
#define DEST_FLAG                     (1LL << 8)
/*
 *  If, during inlining we find an instruction that uses the contents of FP,
 *  we surround it with an instruction to restore the original FP of the
 *  callee, and one to restore our new FP in the caller.  This instruction
 *  that is surrounded gets the INSTR_USES_FP_BUT_COMPENSATED_FLAG flag set.
 */
#define INSTR_USES_FP_BUT_COMPENSATED_FLAG (1LL << 9)
/* If the condition needs to be switched on the instruction */
#define INSTR_SWITCH_CONDITION_FLAG   (1LL << 10)
/* If a part of the instruction needs to be patched */
#define INSTR_RELOCATION_FLAG         (1LL << 11)
/*
 *  If the value is an ok value
 *  to use in place of calling InstrBPSPDiff()
 */
#define INSTR_BPSP_DIFF_STORED_FLAG   (1LL << 12)

/*
 *  Flags for instruction classes and types
 */
#define INSTR_CLASS_PC_CHANGING       (1LL << 13)
#define INSTR_CLASS_JUMP              (1LL << 14)
#define INSTR_CLASS_RETURN            (1LL << 15)
#define INSTR_CLASS_CALL              (1LL << 16)
#define INSTR_CLASS_BRANCH            (1LL << 17)
#define INSTR_CLASS_LOOP              (1LL << 18)
#define INSTR_CLASS_FLOAT             (1LL << 19)
#define INSTR_CLASS_NOP               (1LL << 20)
#define INSTR_CLASS_HALT              (1LL << 21)
#define INSTR_CLASS_LEAVE             (1LL << 22)
#define INSTR_CLASS_LEA               (1LL << 23)
#define INSTR_CLASS_ADD               (1LL << 24)
#define INSTR_CLASS_SUB               (1LL << 25)
#define INSTR_CLASS_OPDIR             (1LL << 26)
#define INSTR_CLASS_OPOFF             (1LL << 27)
#define INSTR_CLASS_MOV               (1LL << 28)
#define INSTR_CLASS_SYSCALL           (1LL << 29)
#define INSTR_HAS_CLASS_FLAGS         (1LL << 30)
#define INSTR_RELOCATABLE_FLAG        (1LL << 31)

/* Set for instructions that if we knew SOURCE1, we could evaluate */
#define INSTR_EVAL_IF_SOURCE1         (1LL << 32)
/* Set for instructions that if we knew SOURCE2, we could evaluate */
#define INSTR_EVAL_IF_SOURCE2         (1LL << 33)
/* Set for instructions that if we knew DEST, we could evaluate */
#define INSTR_EVAL_IF_DEST            (1LL << 34)
/*
 *  If the instruction defines %ebp, but not maliciously.
 *  This is set for "pop %ebp" instructions seen on function exit.
 */
#define INSTR_DEFINES_FP_NICELY       (1LL << 35)
/* Temporary flag */
#define INSTR_DONT_COMPENSATE_FOR_FP_USE_FLAG (1LL << 36)
/* Temp flag */
#define INSTR_TEMP_FLAG               (1LL << 37)
/* Flist flag */
#define INSTR_FLIST_FLAG              (1LL << 39)
/* Set if the instruction is eliminatable */
#define INSTR_ELIMINATABLE_FLAG       (1LL << 39)
/* Set to indicate that a instruction MAY have a JT behind it. */
#define INSTR_MAY_HAVE_JT             (1LL << 40)
#define INSTR_CLONED                  (1LL << 41)
#define INSTR_LITERAL_BYTES_FLAG      (1LL << 42)
#define INSTR_CLASS_POP               (1LL << 43)
#define INSTR_CLASS_PUSH              (1LL << 44)
/* New instr introduced to tolerate data in the text section */
#define INSTR_DATA_BYTE               (1LL << 45)
/* Set to mark a section boundary when re-splitting merged code sections */
#define INSTR_STARTS_SECTION          (1LL << 46)
/* Extension to allow each instruction to potentially specify its alignment */
#define INSTR_NEEDS_ALIGNMENT         (1LL << 47)
/* Indicates instructions added for padding : Can potentially be removed ?? */
#define INSTR_PADDING_BYTE 	      (1LL << 48)
/* Set for jumps converted to jumps through the spoofed jump tables (binobf) */
#define INSTR_SPOOF_JT_FLAG           (1LL << 49)
/* Set for jumps converted to branch function calls (binobf) */
#define INSTR_JBF_CALL_FLAG           (1LL << 50)
/* Set for calls converted to branch function calls (binobf) */
#define INSTR_CBF_CALL_FLAG           (1LL << 51)
/* Executing the instruction will raise the below signal (binobf) */
#define INSTR_SIGILL_FLAG             (1LL << 52)
#define INSTR_SIGFPE_FLAG             (1LL << 53)
#define INSTR_SIGSEGV_FLAG            (1LL << 54)
/* Set for junk instructions (binobf) */
#define INSTR_JUNK_FLAG               (1LL << 55)



#define INSTR_NOCLONE_FLAG \
    (INSTR_RELOCATABLE_FLAG | FUN_START | FUN_END | INSTR_BPSP_DIFF_STORED_FLAG)

/*
 *  Symbol Flags
 */
#define SYMBOL_FUNCTION_FLAG	(1 << 0)
#define SYMBOL_SECTION_FLAG	(1 << 1)
#define SYMBOL_OBJECT_FLAG	(1 << 2)
#define SYMBOL_WEAK_FLAG	(1 << 3)
#define SYMBOL_LOCAL_FLAG	(1 << 4)
#define SYMBOL_GLOBAL_FLAG	(1 << 5)
#define SYMBOL_UNDEFINED_FLAG	(1 << 6)
#define SYMBOL_ABSOLUTE_FLAG	(1 << 7)
#define SYMBOL_COMMON_FLAG	(1 << 8)
#define SYMBOL_NOTYPE_FLAG	(1 << 9)
#define SYMBOL_OBJECT_END_FLAG	(1 << 10)

/*
 *  Section Flags
 *
 *  The section is marked?? What does this mean?
 */
#define SECTION_MARKED_FLAG (1 << 0)

/*
 *  The section will be dropped (removed) before the binary is written out.
 */
#define SECTION_DROPPED_FLAG (1 << 1)

/*
 *  The section should be writable at runtime.
 */
#define SECTION_WRITABLE_FLAG (1 << 2)

/*
 *  The section contains executable data.
 */
#define SECTION_EXECUTABLE_FLAG (1 << 3)

/*
 *  The section occupies space in the file.
 */
#define SECTION_OCCUPIES_FILESPACE_FLAG (1 << 4)

/*
 *  The section occupies virtual memory space at runtime.
 */
#define SECTION_OCCUPIES_VM_FLAG (1 << 5)

/*
 *  The section holds information about the binary.
 */
#define SECTION_NOTE_FLAG (1 << 6)

/*
 *  The section contains a string table.
 */
#define SECTION_STRING_TABLE_FLAG (1 << 7)

/*
 *  The section contains a symbol table.
 */
#define SECTION_SYMBOL_TABLE_FLAG (1 << 8)

/*
 *  The section contains relocations.
 */
#define SECTION_RELOCATIONS_FLAG (1 << 9)

/*
 *  The section contains intrusion detection information.
 */
#define SECTION_IDS_FLAG (1 << 10)

/*
 *  The section is/has been merged.
 */
#define SECTION_MERGE_FLAG (1 << 11)

/*
 *  The section contains debugging information.
 */
#define SECTION_DEBUGGING_FLAG (1 << 12)

/*
 *  Function Flags
 */
/*  Set when a function has been deemed analyzable (see dominators.h) */
#define FUNCTION_CAN_BE_INLINED_FLAG        (1 << 0)
/*  Set for functions that have been cloned */
#define FUNCTION_TEMP_FLAG                  (1 << 1)
/*  Set for well-behaved functions (that leave stack same as found it) */
#define FUNCTION_WELL_BEHAVED_FLAG          (1 << 2)
/*  Set for functions that alias the stack pointer */
#define FUNCTION_ALIASES_STACK_POINTER_FLAG (1 << 3)
/*  Set for functions where we can determine the difference between
 *  ESP and EBP at every point */
#define FUNCTION_BALANCED_STACK_FLAG        (1 << 4)
/*  Set for functions with a LEAVE/RET combo */
#define FUNCTION_LEAVE_RET_COMBO            (1 << 5)
/*  Set for functions which have the standard gcc prologue of
 *  "push %ebp, mov %esp->%ebp" */
#define FUNCTION_GCC_PROLOGUE_FLAG          (1 << 6)
/*  Set for functions in which constant propagation should be performed */
#define FUNCTION_CP_FLAG                    (1 << 7)
/*  Set for functions in which runtime stack and register snapshots
 *  have been allocated */
#define FUNCTION_SNAPSHOT_FLAG              (1 << 8)
/*  Set for functions who re-define the frame pointer
 *  outside of epilogue/prologue */
#define FUNCTION_REDEFINES_FP_FLAG          (1 << 9)
/*
 *  Set for functions who-re-define the frame pointer
 *  outside of epilogue/prologue, where the re-definition
 *  was not introduced as a result of inlining. The redefinitions that
 *  inlining uses are not malicious, as they are repaired almost instantly.
 */
#define FUNCTION_MALICIOUSLY_REDEFINES_FP_FLAG  (1 << 10)
/*  The function has an exit combo (not necessarily a leave/ret) */
#define FUNCTION_EXIT_COMBO_FLAG            (1 << 11)
/* This function has  a data instruction */
#define FUNCTION_DATA_INSTRUCTION_FLAG      (1 << 12)

/* Apply layout on this function */
#define FUNCTION_LAYOUT_FLAG                (1 << 13)

/*  Function to be excluded from the signal-based obfuscation. Functions
 *  called while installing signal handlers as well as handlers themself
 *  are marked with this flag. (binobf) */
#define FUNCTION_DO_NOT_OBFUSCATE_SIGNAL    (1 << 14)

#define FUNCTION_NOCLONE_FLAGS\
    (FUNCTION_SNAPSHOT_FLAG | \
	FUNCTION_BALANCED_STACK_FLAG | \
	FUNCTION_CP_FLAG | \
	FUNCTION_TEMP_FLAG)

/*
 *  Basic Block Flags
 */
/* Set when a basic block is deemed reachable */
#define BBL_LIVE_FLAG                 (1 << 0)
/* Set for basic blocks that are profile-counters */
#define BBL_PROFILED_FLAG             (1 << 1)
/* Set when a basic block is a loop entry point */
#define BBL_LOOP_HEADER_FLAG          (1 << 2)
/* Set when a basic block contains alignment padding (NOPs and whatnot) */
#define BBL_ALIGN_PAD_FLAG            (1 << 3)
/*
 *  Set for basic blocks that should be aligned on 16-byte boundaries
 *  when within 8 bytes of that boundary
 */
#define BBL_LOOP_ALIGN_ME_FLAG        (1 << 4)
/* Set for basic blocks that are sufficiently hot */
#define BBL_HOT_FLAG                  (1 << 5)
/* Set for basic blocks that are not executed
 * according to the profile information */
#define BBL_ZERO_FLAG                 (1 << 6)
/* Set for basic blocks that are neither sufficiently hot nor in the zero set */
#define BBL_COLD_FLAG                 (1 << 7)
/* Set for basic blocks that are sufficiently hot call sites */
#define BBL_HOT_CALL_SITE_FLAG        (1 << 8)
/* A general purpose temporary flag */
#define BBL_TEMP_FLAG                 (1 << 9)
/* Set for BBLs that have allocated stack snapshots */
#define BBL_STACKSHOT_FLAG            (1 << 10)
/* Set for BBLs that have allocated register snapshots */
#define BBL_REGSHOT_FLAG              (1 << 11)
/* Set in BBLs that need to have their liveness recomputed */
#define BBL_REGLIVENESS_FLAG          (1 << 12)
/* Set in BBLs that need their stack liveness recomputed */
#define BBL_STACKLIVENESS_FLAG        (1 << 13)
/* Set for marking bbls */
#define BBL_MARKED_FLAG               (1 << 14)
/* Set for cloned basic blocks in which switch edges have been resolved */
#define BBL_CLONE_SWITCH_RESOLVED_FLAG (1 << 15)
/* Set for blocks who need to be evaluated by the constant propagator */
#define BBL_CP_FLAG                   (1 << 16)
/* Set when the PSW liveness information for the bbl needs to be recomputed */
#define BBL_PSWLIVENESS_FLAG          (1 << 17)
/* Set for blocks ending with jump through the spoofed jump table (binobf) */
#define BBL_JT_FLAG                   (1 << 18)
/* Set for blocks ending with calls to branch function (binobf) */
#define BBL_BF_FLAG                   (1 << 19)
/* Set for blocks ending with a trap instruction (binobf) */
#define BBL_TRAP_FLAG                 (1 << 20)
/* Set for blocks containing junk instructions (binobf) */
#define BBL_JUNK_FLAG                 (1 << 21)

#define BBL_NOCLONE_FLAGS \
    (BBL_TEMP_FLAG | BBL_LIVE_FLAG | BBL_PROFILED_FLAG | BBL_LOOP_HEADER_FLAG \
    | BBL_ALIGN_PAD_FLAG | BBL_LOOP_ALIGN_ME_FLAG | BBL_STACKSHOT_FLAG \
    | BBL_REGSHOT_FLAG | BBL_REGLIVENESS_FLAG | BBL_STACKLIVENESS_FLAG \
    | BBL_CLONE_SWITCH_RESOLVED_FLAG)


/*
 *  Edge Flags
 */
/* Set when an edge is the only edge going into the destination block */
#define EDGE_ONLY_IN_FLAG             (1 << 0)
/* set for an edge when that edge is the only edge leaving the source block */
#define EDGE_ONLY_OUT_FLAG            (1 << 1)

#define EDGE_NO_FLOW_FLAG            (1 << 2)

/*
 *  Relocation Flags
 */
#define RELOCATION_NOT_HANDLED_FLAG   (1 << 0)
#define RELOCATION_PC_RELATIVE_FLAG (1 << 1)
#define RELOCATION_GOTPC_FLAG (1 << 2)

#endif				// FLAGS_H
