/*
 *  This file contains stuff specific to the Function data
 *  structure which represents a function.
 */
#ifndef FUNCTION_H
#define FUNCTION_H

#include "plto_types.h"
#include "bbl.h"

/*
 *  STRUCTURES
 *
 *  The representation used for a function.
 */
typedef struct Function {
    UINT32 f_flags;		   // flags
    struct Function *f_prev;	   // next function
    struct Function *f_next;	   // previous function
    struct Block *f_first;	   // first basic block in this function
    struct Block *f_last;	   // last basic block in this function
    UINT32 f_id;		   // unique identifier for the function
    char *f_name;		   // function name
    Block *f_entry;		   // entry block
    Block *f_exit;		   // exit block
    UINT32 f_stackFrameSize;	   // max size of this func's activation rec
    UINT32 f_callerFrameSize;	   // max size of activ recs of all callers
    INT32 f_trashDepth;		   // amt of caller's frame this func can kill
    INT32 f_useDepth;		   // amt of caller's frame this func may use
    REGISTER_BITVECTOR f_regsDefined; // bits this func or callees may define
    int f_mayuse;                  // registers that bypass this function
    int f_bypass;                  // registers that may be used before defining
    struct Function *f_chainnext;
    struct Function *f_chainprev;
    struct Function *f_chainhead;
    struct Function *f_headnext;
    struct Function *f_headprev;
    struct Function *f_nexttochain;
} Function;

/* Macros for accessing fields in the functions */
#define FUNCTION_FIRST_INSTR(fun) ((fun)->f_first->b_first)
#define FUNCTION_LAST_INSTR(fun)  ((fun)->f_last->b_last)
#define FUNCTION_HAS_FLAG(fun,z)  ((bool)((fun)->f_flags & (z)))
#define FUNCTION_SET_FLAG(fun,z)  ((fun)->f_flags |= (z))
#define FUNCTION_REMOVE_FLAG(fun, aflg)      ((fun)->f_flags &= ~(aflg))
#define FUNCTION_START(fun) (FUNCTION_FIRST_INSTR(fun)->i_final_address)
#define FUNCTION_END(fun) \
    ((FUNCTION_LAST_INSTR(fun)->i_final_address) + \
    (FUNCTION_LAST_INSTR(fun)->i_length))

extern void FunFree(Function *fun);
extern Function *FunCreate();

#endif
