/*
 *  Hash table lookups. Currently supported are:
 *  Relocations (by address)
 *  Instructions (by address)
 */
#ifndef HASH_H
#define HASH_H

#include "global.h"

/*
 *  Functions for instruction hashing
 */
/* Returns the instruction that resides at 'address' in the executable,
 * or NULL if none exists */
extern Instruction *InstrLookupEntry(UINT32 address);

/* Inserts the instruction pointed to by 'entry' into the hash table */
extern void InstrInsertEntry(Instruction *entry);

/* Removes the instruction pointed to by 'entry' from the hash table */
extern void InstrRemoveEntry(Instruction *entry);

/* Removes all entries from the hash table */
extern void InstrClearHashTable();

/* Traverses the instruction list and adds all the instructions
 * to the hash table */
extern void InstrAddAllHashTable();

/* Prints, to standard out, statistics about the density of the instruction
 * hash table */
extern void ReportInstrDensityInHashTable();

/*
 *  Functions for relocation hashing
 */
/* Returns the relocation at 'address', or NULL if none exists */
extern Relocation *RelocLookupEntry(UINT32 address);

/* Inserts the relocation pointed to by 'entry' into the hash table */
extern void RelocInsertEntry(Relocation *entry);

/* Removes the relocation pointed to by 'entry' from the hash table */
extern void RelocRemoveEntry(Relocation *entry);

extern void BuildRelocationHashTable();

#endif
