/*
 *  Functions for maintaining the linkage between various structures.
 */
#ifndef LINKAGE_H
#define LINKAGE_H

extern void InstrRelinkBasedOnBBL();
extern void BBLRelinkBasedOnFun();

/* Functions for disconnecting funs/bbls/instrs from the CFG */
extern Instruction *InstrDisconnect(Instruction *instr);
extern void FunDisconnect(Function *fun);
extern void BBLDisconnect(Block *block);
extern void RelocationDisconnect(Relocation *reloc);

/* Functions for inserting instructions into the CFG */
extern void InsertInstrBeforeInstr(Instruction *newinstr, Instruction *instr);
extern void InsertInstrAfterInstr(Instruction *newinstr, Instruction *instr);

/* Functions for inserting instructions into basic blocks */
extern void BBLPrependInstr(Block *ibbl, Instruction *iins);

/* Functions for inserting bbls into the CFG */
extern void InsertBBLAfterBBL(Block *newbbl, Block *block);
extern void InsertBBLBeforeBBL(Block *newbbl, Block *block);
extern void InsertRelocationBeforeRelocation(Relocation *new_relocation,
    Relocation *relocation);
extern void InsertRelocationAfterRelocation(Relocation *new_relocation,
    Relocation *relocation);
extern void InsertSymbolAfterSymbol(Symbol *new_symbol, Symbol *symbol);
extern void InsertSymbolBeforeSymbol(Symbol *new_symbol, Symbol *symbol);
extern void InsertSectionAfterSection(Section *new_section, Section *section);
extern void InsertSectionLast(Section *new_section, Section *list);
extern void InsertSectionAfterSectionNamed(Section *section, char *name,
    Section *list);
extern void RemoveSectionFromList(Section *to_remove, Section **list);
extern void RemoveSymbol(Symbol *sym);
extern Section *RemoveDroppedSectionsFromList(Section *list);
extern void InsertFunctionAfterFunction(Function *new_fun, Function *fun);

#endif				// LINKAGE_H
