/*
 *  This file includes various definitions, macros and shared data
 *  structures used throughout.
 */
#ifndef MACROS_H
#define MACROS_H



/*
 *  GENERAL USAGE MACROS
 */
#define ARRAYSIZE(x) (sizeof(x)/sizeof(x[0]))



/*
 *  ITERATIVE MACROS:
 *  for iterating over the entire lists of instructions, blocks and functions.
 */

#define FOREACH_BBL(iterator) \
    for (iterator = bbl_list; iterator != NULL; iterator = iterator->b_next)

#define FOREACH_INSTR(iterator) \
    for (iterator=i_list; iterator!=NULL; iterator=INSTR_NEXT_NONNULL(iterator))

#define FOREACH_INSTR_IN_SECTION(sec, iterator) \
    for (iterator = ((sec)->s_instr) ? ((sec)->s_instr) : i_list; \
	iterator != NULL && \
	    (! INSTR_HAS_FLAG(iterator, INSTR_STARTS_SECTION) \
	    || iterator == (sec)->s_instr); \
	iterator = INSTR_NEXT_NONNULL(iterator))

#define FOREACH_FUN(iterator) \
    for (iterator = f_list; iterator != NULL; iterator = iterator->f_next)

#define FOREACH_FUNCTION(iterator) \
    for (iterator = f_list; iterator != NULL; iterator = iterator->f_next)

#define FOREACH_INSTR_IN_BBL(bbl, iterator) \
    for (iterator = (bbl)->b_first; \
	iterator != NULL && iterator->i_Bblock == (bbl); \
	iterator = iterator->i_next)

#define FOREACH_BBL_IN_FUN(fun, iterator) \
    for (iterator = (fun)->f_first; \
	iterator != NULL && iterator->b_fun == (fun); \
	iterator = iterator->b_next)

#define FOREACH_SYMBOL(iterator) \
    for (iterator = s_list; iterator != NULL; iterator = iterator->sy_next)

#define FOREACH_SYMBOL_IN_LIST(list, iterator) \
    for (iterator = list; iterator != NULL; iterator = iterator->sy_next)

#define FOREACH_SECTION(iterator) \
    for (iterator = sec_list; iterator != NULL; iterator = iterator->s_next)

#define FOREACH_SECTION_IN_LIST(list, iterator) \
    for (iterator = list; iterator != NULL; iterator = iterator->s_next)

#define FOREACH_PRED_IN_BBL(bbl, iterator) \
    for (iterator=(bbl)->b_pred; iterator != NULL; iterator = iterator->en_next)

#define FOREACH_SUCC_IN_BBL(bbl, iterator) \
    for (iterator=(bbl)->b_succ; iterator != NULL; iterator = iterator->en_next)

#define FOREACH_RELOCATION(iterator) \
    for (iterator = r_list; iterator != NULL; iterator = iterator->r_next)

#define FOREACH_RELOCATION_IN_LIST(list, itr) \
    for (itr = list; itr != NULL; itr = itr->r_next)

#define FOREACH_FLIST(iterator) \
    for (iterator=dontTouchFuns; iterator != NULL; iterator = iterator->fl_next)

#define ABSOLUTE_RELOCATION_ADDRESS(reloc) \
    ((reloc)->r_section->s_address + (reloc)->r_offset)



/*
 *  DEBUGGING MACROS
 */

#ifdef CASSERT
    #undef CASSERT
    #define CASSERT(cond,mess)  if (!(cond)) { MALFUNCTION(mess) ;}
#else
    #define CASSERT(cond,mess)
#endif



#endif				// MACROS_H
