/*
 *  This file contains stuff specific to the Operand data structure
 *  which represents an instruction's operand.
 */
#ifndef OPERAND_H
#define OPERAND_H

#include "plto_types.h"
#include "relocation.h"



/*
 *  STRUCTURES
 *
 *  The representation used for an operand.
 */
typedef struct Operand {
    UINT32 a_flags;			// operand flags
    INT32 a_displacement;		// displacement
    struct Relocation *a_relocation;	// relocation
    UINT16 a_base_reg;			// base register
    UINT16 a_index_reg;			// index register
    UINT8 a_scale;			// scale factor
} Operand;



/* Macros for setting bit flags in the operands */
/* WARNING: UNSAFE  (be careful to use {braces} if controlled by if or while) */

#define OP_BASE(op,base) \
    (op)->a_base_reg = (base);  OP_SET_FLAG((op), OPERAND_BASE_FLAG)
#define OP_DISP_R(op,disp) \
    (op)->a_displacement = (disp);  OP_SET_FLAG((op), OPERAND_DISP_R_FLAG)
#define OP_DISP_A(op,disp) \
    (op)->a_displacement = (disp);  OP_SET_FLAG((op), OPERAND_DISP_A_FLAG)
#define OP_INDEX(op,index) \
    (op)->a_index_reg = (index);  OP_SET_FLAG((op), OPERAND_INDEX_FLAG)
#define OP_SCALE(op,s) \
    (op)->a_scale = (s);  OP_SET_FLAG((op), OPERAND_SCALE_FLAG)

/* set operand to immediate value */
#define OP_IMM(op,imm) \
    (op)->a_displacement = (imm);  OP_SET_FLAG((op), OPERAND_IMM_FLAG); \
    OP_SET_FLAG((op), OPERAND_DISP_A_FLAG)

#define OP_TYPE(op,flag)		((op)->a_flags & (flag))
#define OPERAND_HAS_FLAG(op,flag)	((bool)((op)->a_flags & (flag)))
#define OP_SET_FLAG(op, flag)		((op)->a_flags |= (flag))
#define OPERAND_SET_FLAG(op, flag)	((op)->a_flags |= (flag))
#define OPERAND_REMOVE_FLAG(op, flag)	((op)->a_flags &= (~(flag)))

#endif
