/*
 *  opt.layout.h
 *
 *  Functions for changing the basic block layout
 */
#ifndef OPT_LAYOUT_H
#define OPT_LAYOUT_H

/* Termination conditions for chain building */
#define DELTA_CHAIN     30	// When the chains change by no more than 30
#define DELTA_PERCENT   16	// When the chains change by no more than 16%
#define MAXIMUM_ROUNDS  10	// When 10 rounds have elapsed

/* Optimizes layout of basic blocks based on a layout scheme (FOM/PH/ALTO) */
void BBLOptimizeLayout();

/* removes all ET_ENTRY, ET_EXIT, ET_RETURN, and ET_FUNCALL edges */
void RemoveEntryExitReturnFuncallEdges();

/* Removes all BT_ENTRY and BT_EXIT abstract blocks from the program */
void RemoveEntryExitBlocks();

/* Gives some edges preference over others. Everything in here is a hack*/
void BBLMarkEdgesToBeAdjusted();

/* Assigns weights to edges based on their basic block weights */
void EdgeComputeWeights();

/* Assigns weights to basic blocks based on their edge weights */
void BBLComputeWeights();

/* 
 * Assigns temperature (hot, cold, zero) to basic blocks based on their weights
 * and the hot code threshold.
 */
void BBLAssignTemperature();

/* Here temporarily. Calls BBLComputeWeights() and BBLAssignTemperature() */
void BBLAssignWeights();

/* Updates the weight field in the supplied basic block. */
void BBLUpdateWeight(Block *block);

/* Used by qsort to compare two basic blocks by weight */
int BBLCompare(const void *first, const void *second);

/* Returns true if the function contains a hot basic block */
bool FunctionHasHotBBL(Function *fun);

/*
 *  Inserts/deletes jumps to guarantee correct control flow. For instance, if
 *  there is a normal edge between two blocks and they aren't adjacent, then
 *  an uncond. jump is inserted
 */
void InstrUpdateEdges();

#define BBL_LAYOUT_OFF             0x0
#define BBL_LAYOUT_GREEDY          0x1
#define BBL_LAYOUT_ALTO            0x2
#define BBL_LAYOUT_PH              0x3
#define BBL_LAYOUT_FUNS_ONLY       0x4
#define BBL_LAYOUT_PH_WITH_SPLIT   0x5

#endif				// OPT_LAYOUT_H
