/*
 *  relocate.h
 *
 *  Functions to handle relocations and patching of instructions and sections
 */
#ifndef RELOCATE_H
#define RELOCATE_H

extern int UpdateInstrAddresses(Section *section);
extern bool UpdatePCRelativeJumps(Instruction *iterator);
extern void OperandClearBytemodeFlags(Operand *op);
extern void UpdateSectionSizes(bool useNewInfo);

/*
 *  Four functions used to iterate over sections.  GetFirstSection returns
 *  the section with the smallest vma, GetLastSection returns the section with
 *  the largest vma.  GetNextSection returns the section with the smallest vma
 *  that is unmarked.  It marks the section right before returning it.
 *  UnmarkAllSections should be used to clear all marked flags on the sections
 *  if they need to be iterated over again
 */
extern Section *GetNextSection();
extern Section *GetFirstSection();
extern Section *GetLastSection();
extern void UnmarkAllSections();


/* Aligns all functions and schedule instructions */
extern bool Align();

/*
 *  Removes the relocation from the relocation list associated with the given
 *  address.  Returns true if the relocation was successfully removed.
 */
extern bool RemoveRelocationByAddress(UINT32 address);

/* Removes the supplied relocation */
extern void RemoveRelocation(Relocation *rel);

extern FList *AddFlist(ADDRESS addr);
extern FList *GetFlistFunAtAddress(ADDRESS addr);
extern FList *GetFlistFunAtNewAddress(ADDRESS addr);
extern FList *NameToFlist(char *name);
extern void RecalculateAddresses();
extern Instruction *GetPICCallForGOTInstr(Instruction *instr);
extern Instruction *GetGOTInstrForPICCall(Instruction *instr);
extern bool IsPICCall(Instruction *instr);
extern LinkedList *GetSortedSymbolList();


#endif
