/*
 *  This file contains stuff specific to the Relocation data
 *  structure.  This data structure is used to represent a binary
 *  relocation.
 */
#ifndef RELOCATION_H
#define RELOCATION_H

#include "plto_types.h"
#include "symbol.h"
#include "section.h"

/*
 *  STRUCTURES
 *
 *  This struct represents a native relocation.
 */
typedef struct Relocation {
    struct Relocation *r_next;	// next relocation in the global list of relocs
    struct Relocation *r_prev;	// previous relocation in the global list
    UINT32 r_offset;		// offset in section to relocation point
    struct Section *r_section;	// section in which to apply relocation
    struct Symbol *r_symbol;	// symbol the relocation references
    struct Relocation *r_hashNext; // next relocation in the global hash chain
    UINT32 r_flags;		// flags
    UINT8 r_type;		// native type of the relocation
    INT32 r_addend;		// arbitrary amount to add to the relocation
} Relocation;

/* Macros for accessing fields in the relocation entries */
#define RELOCATION_HAS_FLAG(reloc,z)      ((bool)((reloc)->r_flags & (z)))
#define RELOCATION_SET_FLAG(reloc,z)      ((reloc)->r_flags |= (z))
#define RELOCATION_REMOVE_FLAG(reloc,z)   ((reloc)->r_flags &= ~(z))
#define RELOCATION_ADDRESS(reloc) \
    ((reloc)->r_section->s_address + (reloc)->r_offset)

#endif
