/*
 *  This file contains stuff specific to the Section data structure
 *  which represents a section of data/code in a binary.
 */
#ifndef SECTION_H
#define SECTION_H

#include "plto_types.h"

/*
 *  STRUCTURES
 *
 *  The representation used for a section.
 */
typedef struct Section {
    short s_number;		// original ordinal in section header table
    struct Section *s_next;	// next section
    struct Section *s_prev;	// previous section
    struct Section *s_mergeto;	// other section this was merged into, if any
    struct Section *s_relsect;	// associated relocation section
    UINT8 *s_data;		// original data image of the section
    UINT8 *s_newdata;		// current image of section contents
    int s_datasize;		// original size of the section
    int s_newdatasize;		// current size after updating
    int s_delta;		// change in addresses due to section merging
    ADDRESS s_address;		// original starting address of the section
    ADDRESS s_newaddress;	// current starting address of the section
    char *s_name;		// name of the section (e.g. ".text")
    struct Symbol *s_label;	// first symbol in the section
    struct Instruction *s_instr;// first instruction in the section
    UINT32 s_alignment;		// section alignment
    UINT32 s_info;		// miscellaneous information field
    UINT32 s_flags;		// properties of the section
    UINT32 s_growth;		// growth in bytes of the data of the section
    UINT32 s_section_id;	// unique identifier for the section
    int s_name_offset;		// name offset
    int s_file_offset;		// file offset
    int s_lsend;		// local symbol end
    int s_misc;			// multi uses; was quadword.doubleword.d1
} Section;

/* Macros for accessing fields in the section structure */
#define SECTION_CONTAINS(sec,address) \
    (address < (UINT32) (((sec)->s_address) + (int)((sec)->s_datasize))) && \
    (address >= (UINT32) (sec->s_address))
#define SECTION_SET_FLAG(sec,fl) ((sec)->s_flags |= (fl))
#define SECTION_HAS_FLAG(sec,fl) ((bool)((sec)->s_flags & (fl)))
#define SECTION_REMOVE_FLAG(sec,fl) ((sec)->s_flags &= ~(fl))
#define SECTION_WAS_MERGED(sec) \
    ((sec) == SECTION_REF_CODE || (sec)->s_mergeto == SECTION_REF_CODE)

#endif
