/*
 *  This file contains structures related to symbols.
 *
 *  The Symbol helps to identify the types of things stored at
 *  certain locations in a binary.  The DataSymbol helps to
 *  identify regions that should not be disassembled and are
 *  converted to data instructions by the disassembler.
 */
#ifndef SYMBOL_H
#define SYMBOL_H

#include "plto_types.h"
#include "section.h"

/*
 *  A symbol from the symbol table.
 */
typedef struct Symbol {
    struct Symbol *sy_next;	// next entry in the global symbol list
    struct Symbol *sy_prev;	// previous entry in the global symbol list
    char *sy_name;		// name of the symbol
    int sy_value;		// value (address) of the symbol
    int sy_size;		// size of the object named by the symbol
    Section *sy_section;	// pointer to the section it is contained in
    UINT32 sy_flags;		// flags. e.g. function, export, import
    UINT32 sy_alignment;	// alignment constraints (some symbols)
    void *sy_temp;		// misc pointer for various auxiliary info
} Symbol;

/*
 *  Symbols that identify regions that should not be disassembled.
 */
typedef struct DataSymbol {
    char *ds_name;                      // symbol name
    unsigned int ds_datasize;           // region size
    unsigned int ds_alignment;          // alignment boundary
} DataSymbol;



#endif
