/*
 *  sysdep.h
 *
 *  System dependency handler. Everything that is architecture,
 *  system or executable dependent should go in the handler.  It
 *  will re-route the functions to the appropriate machine-dependent
 *  handlers based on what architecture is defined.
 */
#ifndef SYSDEP_H
#define SYSDEP_H

#include "valprof.h"

extern int CalculateInstrLength(Instruction *instr);
extern void UpdateInstrLength(Instruction *instr);
extern bool isValidInstr(Instruction *instr);
extern void InsertDumpCode(char *filename);
extern int DisassembleInstr(ADDRESS pc, UINT8 *data, Instruction *instr);
extern int InstrPushBytes(Instruction *instr);
extern int InstrPopBytes(Instruction *instr);
extern Instruction *InstrCreateNOP();
extern Instruction *InstrCreateJMP();
extern Instruction *InstrCreateSave();
extern Instruction *InstrCreateRestore();
extern Instruction *InstrCreate32BitMemINC(ADDRESS addr);
extern Instruction *InstrCreate32BitMemAdd(ADDRESS addr);
extern Instruction *InstrCreate32BitMemAdc(ADDRESS addr);
extern Instruction *InstrCreateSPInc();	// add IMMEDIATE, %esp
extern Instruction *InstrCreateSPDec();	// sub IMMEDIATE, %esp
extern Instruction *InstrCreate2ByteDoNothing();
extern Instruction *InstrCreateCmovRegToReg();
extern Instruction *InstrCreateCmovMemToReg();
extern Instruction *InstrCreateMovImmToReg(int reg, int value);
extern Instruction *InstrCreate32BitMovMemToReg(int reg);
extern Instruction *InstrCreateCmpRegToValue(int reg, int value);
extern Instruction *InstrCreateCmpStackToValue(int offset, int value);
//Igor, next 12
extern Instruction *InstrCreateCmpImm8WithReg(INT8 imm8, int reg);
extern Instruction *InstrCreateCmpImm32WithReg(INT32 imm32, int reg);
extern Instruction *InstrCreateCmpImm8WithMemAbs(INT8 imm8, int offset);
extern Instruction *InstrCreateCmpImm32WithMemAbs(INT32 imm32, int offset);
extern Instruction *InstrCreateCmpImm8WithMemBase(INT8 imm8, int reg);
extern Instruction *InstrCreateCmpImm32WithMemBase(INT32 imm32, int reg);
extern Instruction *InstrCreateCmpImm8WithMemDisp8Base(INT8 imm8,
    INT8 disp8, int reg);
extern Instruction *InstrCreateCmpImm32WithMemDisp8Base(INT32 imm32,
    INT8 disp8, int reg);
extern Instruction *InstrCreateCmpImm8WithMemDisp32Base(INT8 imm8,
    INT32 disp32, int reg);
extern Instruction *InstrCreateCmpImm32WithMemDisp32Base(INT32 imm32,
    INT32 disp32, int reg);
extern Instruction *InstrCreateCmpImm8WithMemBaseIndex(INT8 imm8, int base,
    int index, int scale);
extern Instruction *InstrCreateCmpImm32WithMemBaseIndex(INT32 imm32, int base,
    int index, int scale);
//Igor, prev 12
extern Instruction *InstrCreateJne();
extern Instruction *InstrCreateJe();
extern Instruction *InstrCreatePushAll();
extern Instruction *InstrCreatePopAll();
extern Instruction *InstrCreatePop(int reg);
extern Instruction *InstrCreatePush(int reg);
extern Instruction *InstrCreatePushImm(int value);
extern Instruction *InstrCreatePushMem(int address);
extern Instruction *InstrCreatePushMemBase(int reg);     // pushl (%<reg>)

extern Instruction *InstrCreatePushMemDisp8Base(INT8 disp8, int reg);
extern Instruction *InstrCreatePushMemDisp32Base(INT32 disp32, int reg);

extern bool UpdateInstr(Instruction *instr);
extern UINT32 InstrLocateJTBaseAddress(Instruction *instr);
extern Instruction *InstrLocateJTBoundsCheck(Instruction *instr);
extern Instruction *InstrLocateJTBaseInstr(Instruction *instr);
extern UINT32 InstrLocateJTSize(Instruction *instr);
extern bool OperandUsesRegister(Operand *op);
extern void ReportSuspiciousInstrs();
extern bool RegisterIs32Bit(int aReg);
extern bool RegisterIs16Bit(int aReg);
extern bool RegisterIs8Bit(int aReg);
extern bool isConditionalInstr(Instruction *inst);
extern bool isUnconditionalInstr(Instruction *inst);
extern void ScheduleInstructions();
extern bool AlignLoops();
extern void BuildOpcodeHashTable();
extern bool FunIsInlinable(Function *aFun);
extern void FunInline(Instruction *callInstr);
extern bool FunCanBeInlinedFromCallSite(Function *fun, Instruction *callIns);
extern char *PrintInstruction(Instruction *instr);
extern char *PrintInstructionFinal(Instruction *instr);
extern char *PrintRegister(int rgb);
extern int getIcacheSize();
extern int getDcacheSize();
extern int getMegahertz();
extern int getIcacheMissPenalty();
extern int getCallPenalty();
extern int InstrAssemble(unsigned char *buffer, Instruction *instr);
extern REGISTER_BITVECTOR ComputeRegistersDefined(Instruction *instr);
extern REGISTER_BITVECTOR ComputeRegistersUsed(Instruction *instr);
extern REGISTER_BITVECTOR ComputeRegistersDefinedLiberal(Instruction *instr);
extern PSW_BITVECTOR ComputePSWUsed(Instruction *instr);
extern PSW_BITVECTOR ComputePSWDefined(Instruction *instr);
extern PSW_BITVECTOR ComputePSWDefinedLiberal(Instruction *instr);
extern void ComputeStackUsed(
    Instruction *instr, int *slots, int bp_index, int arsize);
extern void ComputeStackDefined(
    Instruction *instr, int *slots, int bp_index, int arsize);
extern void ComputeStackDefinedLiberal(
    Instruction *instr, int *slots, int bp_index, int arsize);
extern bool InstrDefinesReg(Instruction *iins, int areg);
extern bool InstrUsesReg(Instruction *iins, int areg);
extern bool RegisterIsLiveInBitvector(int aReg, REGISTER_BITVECTOR vec);
extern bool RegisterIsFramePointer(int aReg);
extern bool RegisterIsStackPointer(int aReg);
extern bool OperandIsGPRegister(Operand *op);
extern bool InstrHasImplicitDest(Instruction *instr);
extern bool InstrHasImplicitDest2(Instruction *instr);
extern UINT32 InstrImplicitDest(Instruction *instr);
extern UINT32 InstrImplicitDest2(Instruction *instr);
extern bool InstrHasImplicitSrc(Instruction *instr);
extern bool InstrHasImplicitSrc2(Instruction *instr);
extern UINT32 InstrImplicitSrc(Instruction *instr);
extern UINT32 InstrImplicitSrc2(Instruction *instr);
extern bool InstrUsesDest(Instruction *instr);
extern void GenerateStaticOpcodeHistogram();
extern void GenerateWeightedOpcodeHistogram();
extern void InstrSetClassFlags(Instruction *instr);
extern bool OperandAccessesStackFromFramePointer(Operand *op);
extern bool OperandAccessesStackFromFramePointerWithSIB(Operand *op);
extern bool OperandAccessesStackFromStackPointer(Operand *op);
extern bool OperandIsAbsoluteMemoryLocation(Operand *op);
extern bool OperandIsImmediate(Operand *op);
extern bool OperandIsAddress(Operand *op);
extern bool OperandUsesContentsOfStackPointer(Operand *op);
extern bool OperandUsesContentsOfFramePointer(Operand *op);
extern bool OperandIsMemory(Operand *op);
extern bool OperandWritesToUnknownStackSlot(Operand *op);
extern bool OperandIsStackPointer(Operand *op);
extern bool OperandIsFramePointer(Operand *op);
extern void OperandUseLargestBytemode(Operand *op);
extern bool InstrDefinesPSW(Instruction *instr);
extern int BitRegisterToRegister(int i);
extern bool InstrIsDependent(Instruction *a, Instruction *b);
extern UINT8 InstrDelayBetween(Instruction *a, Instruction *b);
extern bool InstrsPair(Instruction *a, Instruction *b);
extern UINT32 NumInstrsPairable();
extern void ReplaceNonsupportedInstructions();
extern int GetOperandLocation(Instruction *instr, Operand *op);
extern Instruction *PICGetALUForCall(Instruction *call_instr);
extern void AssembleCode(Section *code_section);
extern Instruction *InstrCreateMovMemToReg(int scale,
    UINT32 index_reg, UINT32 base_reg, UINT32 dest_reg);
extern Instruction *InstrCreate32BitMovRegToMem(int reg);
extern Instruction *InstrCreateMovAbsoluteMemToReg(ADDRESS memloc, int reg);
extern Instruction *InstrCreateSubImmFromReg(int imm_value, UINT32 reg);
extern Instruction *InstrCreate32BitMovImmToMem(ADDRESS mem_loc, int imm);
extern Instruction *InstrCreate32BitAddRegToMem(ADDRESS mem_loc, int reg);
extern Instruction *InstrCreate32BitSubRegFromMem(ADDRESS mem_loc, int reg);
extern Instruction *InstrCreate32BitAddImmToMem(ADDRESS mem_loc, int imm);
extern Instruction *InstrCreate32BitAddMemToReg(ADDRESS mem_loc, UINT32 reg);
extern Instruction *InstrCreateSubMemFromReg(ADDRESS mem_loc, UINT32 reg);
extern Instruction *InstrCreateSubImmFromMem(int imm, ADDRESS mem_loc);
extern Instruction *InstrCreate16BitSubRegFromMem(ADDRESS mem_loc, int reg);
extern Instruction *InstrCreate16BitAddRegToMem(ADDRESS mem_loc, int reg);
extern Instruction *InstrCreateLeave();
extern Instruction *InstrCreateReturn();
extern Instruction *InstrCreate32BitAddImmToReg(int imm, int reg);
//Igor, next 25
extern Instruction *InstrCreateShortJb();
extern Instruction *InstrCreateShortJae();
extern Instruction *InstrCreateShortJe();
extern Instruction *InstrCreateShortJne();
extern Instruction *InstrCreateShortJbe();
extern Instruction *InstrCreateShortJa();
extern Instruction *InstrCreateShortJs();
extern Instruction *InstrCreateShortJns();
extern Instruction *InstrCreateShortJl();
extern Instruction *InstrCreateShortJge();
extern Instruction *InstrCreateShortJle();
extern Instruction *InstrCreateShortJng();
extern Instruction *InstrCreateShortJg();
extern Instruction *InstrCreateShortJnle();
extern Instruction *InstrCreateShortJecxz();
extern Instruction *InstrCreateNearJb();
extern Instruction *InstrCreateNearJae();
extern Instruction *InstrCreateNearJbe();
extern Instruction *InstrCreateNearJa();
extern Instruction *InstrCreateNearJs();
extern Instruction *InstrCreateNearJns();
extern Instruction *InstrCreateNearJl();
extern Instruction *InstrCreateNearJge();
extern Instruction *InstrCreateNearJle();
extern Instruction *InstrCreateNearJg();

extern Instruction *InstrCreate32BitShlImm(UINT32 reg, int times);
extern Instruction *InstrCreate32BitShr(int reg, int value);
extern Instruction *InstrCreate32BitShlCL(UINT32 reg);
extern Instruction *InstrCreate32BitShrCL(UINT32 reg);
extern Instruction *InstrCreateSubRegFromReg(UINT32 src_reg, UINT32 dest_reg);
extern Instruction *InstrCreateIMULRegReg(UINT32 src_reg, UINT32 dest_reg);
extern Instruction *InstrCreateInc(UINT32 reg);
extern Instruction *InstrCreateDec(UINT32 reg);
extern Instruction *InstrCreateCmpRegToReg(UINT32 reg1, UINT32 reg2);
extern Instruction *InstrCreateHalt();
extern Instruction *InstrCreateSetCarry();
extern Instruction *InstrCreateClearCarry();
extern Instruction *InstrCreateJumpOnNoCarry();
extern Instruction *InstrCreateCALLDisp();
//Igor, next 1
extern Instruction *InstrCreateCallNearMemBase(int reg);
extern Instruction *InstrCreateCallIndirect();
extern Instruction *InstrCreateJMPIndirectAbsMemAddr();
//Igor, next 8
extern Instruction *InstrCreateAddRegToReg(int src_reg, int dest_reg);
extern Instruction *InstrCreateAddImm8ToReg(INT8 imm8, int dest_reg);
extern Instruction *InstrCreateAddImm32ToReg(INT32 imm32, int dest_reg);

extern Instruction *InstrCreateSubImm8FromReg(INT8 imm8, int dest_reg);
extern Instruction *InstrCreateSubImm32FromReg(INT32 imm32, int dest_reg);

extern Instruction *InstrCreateOrRegWithMemBaseIndex(int src, int base,
    int index, UINT8 scale);
extern Instruction *InstrCreateOrMemBaseIndexWithReg(int base, int index,
    UINT8 scale, int dest);

extern Instruction *InstrCreateAndImmWithReg(int imm, int dest_reg);

extern Instruction *InstrCreateXorRegWithMemDisp8Base(int src,
    INT8 dest_disp, int dest);
extern Instruction *InstrCreateXorRegWithMemDisp32Base(int src,
    INT32 dest_disp, int dest);

//same as InstrCreateMovMemDisp8BaseToReg
extern Instruction *InstrCreate32BitMovMemDispToReg(int s_reg, INT8 off,
    int d_reg);

extern Instruction *InstrCreateMovRegToReg(int srcreg, int destreg);
extern Instruction *InstrCreateMovRegToMemBase(int src, int dest);
extern Instruction *InstrCreateMovMemBaseToReg(int src, int dest);
extern Instruction *InstrCreateMovRegToMemDisp8Base(int src, INT8 dest_disp,
    int dest);
extern Instruction *InstrCreateMovRegToMemDisp32Base(int src, INT32 dest_disp,
    int dest);
extern Instruction *InstrCreateMovMemDisp8BaseToReg(INT8 src_disp, int src,
    int dest);
extern Instruction *InstrCreateMovMemDisp32BaseToReg(INT32 src_disp, int src,
    int dest);
extern Instruction *InstrCreateMovRegToMemBaseIndex(int src, int base,
    int index, UINT8 scale);
extern Instruction *InstrCreateMovMemBaseIndexToReg(int base, int index,
    UINT8 scale, int dest);
extern Instruction *InstrCreateMovImmToMemBase(int imm, int dest_reg);
extern Instruction *InstrCreateMovImmToMemDisp8Base(int imm, INT8 disp,
    int dest);
extern Instruction *InstrCreateMovImmToMemDisp32Base(int imm, INT32 disp,
    int dest);
extern Instruction *InstrCreateMovImmToMemBaseIndex(int imm, int base,
    int index, int scale);
extern Instruction *InstrCreateMovzReg8ToReg32(UINT32 src_reg, UINT32 dest_reg);

extern Instruction *InstrCreate32BitPopToMem();
extern Instruction *InstrCreateDiv(int divisor);
extern Instruction *InstrCreateSyscall();
//Igor, next 16
extern Instruction *InstrCreateTestRegWithReg(UINT32 reg1, UINT32 reg2);
extern Instruction *InstrCreateTestImmWithReg(int imm, UINT32 reg);
extern Instruction *InstrCreateLea(UINT32 src, int offset, UINT32 dest);
extern Instruction *InstrCreateSeteReg(UINT32 reg);
extern Instruction *InstrCreateSetzReg(UINT32 reg);

extern Instruction *InstrCreate8bitDivByReg(int divisor);
extern Instruction *InstrCreate32bitDivByReg(int divisor);
extern Instruction *InstrCreate8bitIDivByReg(int divisor);
extern Instruction *InstrCreate32bitIDivByReg(int divisor);
extern Instruction *InstrCreate8bitMulByReg(int multiplier);
extern Instruction *InstrCreate32bitMulByReg(int multiplier);
extern Instruction *InstrCreate8bitIMulByReg(int multiplier);
extern Instruction *InstrCreate32bitIMulByReg(int multiplier);

extern Instruction *InstrCreateIMulImm8ByReg(INT8 imm8, int reg);
extern Instruction *InstrCreateIMulImm32ByReg(INT32 imm32, int reg);

extern Instruction *InstrCreateJmpNearAbsDisp32Index(INT32 disp32, int index,
    int scale);
//Igor, prev 16
extern void PrintPSWFlag(int flag);
extern void InitializeUPMs();
extern int GetStackPointer();
extern int GetFramePointer();
extern int GetARegister();
extern int GetSyscallNoForSyscall(Instruction *syscall);
extern void RemoveSyscallNoForSyscall(Instruction *syscall);

#endif				// SYSDEP_H
