/*
 *  x86-asm.h
 *
 *  Definitions used in the 386 assembler/binary writer
 */
#ifndef X86_ASM_H
#define X86_ASM_H

#include "x86-table-defs.h"

/*
 *  The representation used for an opcode
 */
typedef struct Opcode {
    bool o_need_modrm;		// true if this instr has a MODRM byte
    bool o_need_secondary;	// true if this instr has a 2 byte opcode
    UINT8 o_reg;		// bits 3, 4 and 5 of the MODRM byte
    UINT8 o_mod;		// bits 6 and 7 of the MODRM byte
    UINT8 o_primary;		// first byte of the opcode
    UINT8 o_secondary;		// second byte of the opcode
    UINT8 o_table;		// opcode table which this came from
    struct OpcodeHash *o_h;	// entry in the opcode hash table
} Opcode;

/* MAcros for accessing the fields of the opcode */

#define OPCODE_TABLE_INVALID        0
#define OPCODE_TABLE_DIS386         1
#define OPCODE_TABLE_DIS386_TWOBYTE 2
#define OPCODE_TABLE_GRPS           3
#define OPCODE_TABLE_FGRPS          4
#define OPCODE_TABLE_FLOAT_MEM      5
#define OPCODE_TABLE_FLOAT_REG      6

#define OPCODE_EXTENSION 0x0f

/*
 *  Assembles the given instruction and writes the contents to 'buffer'.
 *  The length of the assembled instruction is returned
 */
int xInstrAssemble(unsigned char *buffer, Instruction *instr);

/* Returns the number of bytes that the instruction will pop off
 * the runtime stack when executed. */
int xInstrPopBytes(Instruction *instr);

/* Returns the number of bytes that the instruction will push on
 * the runtime stack when executed. */
int xInstrPushBytes(Instruction *instr);

/* Prints out the opcode(s) and the MODR/M byte if present */
void SpitOutOpcodeData(Opcode *opcode, Instruction *instr);

/*
 *  Given an instruction, its class, and the format of the operands the
 *  instruction has (bytemode and op routine), this function looks through
 *  the opcode table (x86-table.h) and finds the match.
 *  It fills in parts of the opcode structure it returns
 */
Opcode *PatternMatchOpcode(UINT16 class, int bytemode1, op_rtn op1,
    int bytemode2, op_rtn op2, int bytemode3, op_rtn op3, Instruction *instr);

/* Returns true if the given table entry is a match for the data */
bool OpcodeEntryMatches(X86op *entry, op_rtn op1, int bytemode1, op_rtn op2,
    int bytemode2, op_rtn op3, int bytemode3, int type, int class);

/* Returns the length in bytes that the instruction will take up when encoded */
int xCalculateInstrLength(Instruction *instr);

/* Updates the length of the instruction by calling CalculateInstrLength */
void xUpdateInstrLength(Instruction *instr);

/* Switches the condition on the instruction */
void InstrSwitchCondition(Instruction *instr);

/* Returns true if the instruction has an implicit destination */
bool xInstrHasImplicitDest(Instruction *instr);

/* Returns true if the instruction has an implicit second destination */
bool xInstrHasImplicitDest2(Instruction *instr);

/* Returns the implicit register that the instruction defines */
UINT32 xInstrImplicitDest(Instruction *instr);

/* Returns the second implicit register that the instruction defines */
UINT32 _InstrImplicitDest2(Instruction *instr);

/* Returns true if the instruction has an implicit source */
bool xInstrHasImplicitSrc(Instruction *instr);

/* Returns true if the instruction has an implicit second source */
bool xInstrHasImplicitSrc2(Instruction *instr);

/* Returns the implicit register that the instruction uses */
UINT32 xInstrImplicitSrc(Instruction *instr);

/* Returns the second implicit register that the instruction uses */
UINT32 _InstrImplicitSrc2(Instruction *instr);

/*
 *  Returns an entry in the opcode tables that represents what the current
 *  'instr' would use if assembled/disassembled
 */
X86op *InstrGetOpcodeEntry(Instruction *instr);

/*
 *  Given an instruction and an operand returns the location of the operand
 *  in the instruction encoding as an offset from the start of the instruction.
 */
int xGetOperandLocation(Instruction *instr, Operand *op);
void xAssembleCode(Section *code_section);

#endif				// X86_ASM_H
