/*
 *  Definitions for instruction classes.
 */
#ifndef X86_CLASSES_H
#define X86_CLASSES_H



/*
 *  Instruction Type        Value
 *  ==================      =======
 */
#define TYPE_ERROR          0xFF        // 255
#define TYPE_NONE           0x00        // 0
#define TYPE_CONDITIONAL    0x02        // 2
#define TYPE_UNCONDITIONAL  0x03        // 3



/*
 *  Instruction Class    Value
 *  ==================   =======
 */
#define CLASS_NONE       0x00	// 0
#define CLASS_ADD        0x01	// 1
#define CLASS_INPUT      0x02	// 2 -- Input from port to string
#define CLASS_AND        0x03	// 3
#define CLASS_XOR        0x04	// 4
#define CLASS_INC        0x05	// 5
#define CLASS_PUSH       0x06	// 6
#define CLASS_POP        0x07	// 7
#define CLASS_JUMP       0x08	// 8
#define CLASS_TEST       0x09	// 9 -- Logical compare
#define CLASS_XCHG       0x0A	// 10
#define CLASS_MOV        0x0B	// 11
#define CLASS_OR         0x0C	// 12
#define CLASS_OUTPUT     0x0D	// 13 -- Output to port
#define CLASS_SUB        0X0E	// 14
#define CLASS_CMP        0x0F	// 15 -- Compare two operands
#define CLASS_CALL       0x10	// 16
#define CLASS_BRANCH     0x11	// 17
#define CLASS_CMOV       0x12	// 18
#define CLASS_AADJUST    0x13	// 19 -- Decimal adjust after addition
#define CLASS_MISC       0x14	// 20 -- Should deprecate itself
#define CLASS_MUL        0x15	// 21
#define CLASS_DIV        0x16	// 22
#define CLASS_LOAD       0x17	// 23
#define CLASS_STORE      0x18	// 24
#define CLASS_NOP        0x19	// 25
#define CLASS_RET        0x1A	// 26
#define CLASS_CONVERT    0x1B	// 27
#define CLASS_ENTER      0x1C	// 28
#define CLASS_LEAVE      0x1D	// 29
#define CLASS_INT        0x1E	// 30
#define CLASS_LOOP       0x1F	// 31
#define CLASS_HALT       0x20	// 32
#define CLASS_FLAG       0x21	// 33
#define CLASS_PREFETCH   0x22	// 34
#define CLASS_3DNOW      0x23	// 35
#define CLASS_MMX        0x24	// 36
#define CLASS_SET        0x25	// 37
#define CLASS_DEC        0x26	// 38
#define CLASS_PACK       0x27	// 39
#define CLASS_UNDEFINED  0x28	// 40
#define CLASS_ROTATE     0x29	// 41
#define CLASS_SHIFT      0x2A	// 42
#define CLASS_NEG        0x2B	// 43
#define CLASS_ADDC       0x2C	// 44 -- Add with carry
#define CLASS_SUBB       0x2D	// 45 -- Subtraction with borrow
#define CLASS_PUSHALL    0X2E	// 46 -- Push all general registers on stack
#define CLASS_POPALL     0x2F	// 47 -- Pop all general registers from stack
#define CLASS_BOUND      0x30	// 48 -- Check array index against bounds
#define CLASS_ARPL       0x31	// 49 -- Adjust RPL field
#define CLASS_SADJUST    0x32	// 50 -- Decimal adjust after subtraction
#define CLASS_AAA        0x33	// 51 -- Ascii Adjust After Addition
#define CLASS_AAS        0x34	// 52 -- Ascii Adjust After Subtraction
#define CLASS_IMUL       0x35	// 53 -- Signed multiply
#define CLASS_JA         0x36	// 54 -- Jump if CF = 0 and ZF = 0
#define CLASS_JAE        0x37	// 55 -- Jump if CF = 0
#define CLASS_JB         0x38	// 56 -- Jump if CF = 1
#define CLASS_JBE        0x39	// 57 -- Jump if CF = 1 or ZF = 1
#define CLASS_JCXZ       0x3B	// 59 -- Jump if %cx = 0
#define CLASS_JE         0x3D	// 61 -- Jump if ZF = 1
#define CLASS_JG         0x3E	// 62 -- Jump if ZF = 0 and SF = OF
#define CLASS_JGE        0x3F	// 63 -- Jump if SF = OF
#define CLASS_JL         0x40	// 64 -- Jump if SF != OF
#define CLASS_JLE        0x41	// 65 -- Jump if ZF = 1 or SF != OF
#define CLASS_JNE        0x47	// 71 -- Jump if ZF = 0
#define CLASS_JNO        0x4C	// 76 -- Jump if OF = 0
#define CLASS_JNP        0x4D	// 77 -- Jump if PF = 0
#define CLASS_JNS        0x4E	// 78 -- Jump if SF = 0
#define CLASS_JO         0x50	// 80 -- Jump if OF = 1
#define CLASS_JP         0x51	// 81 -- Jump if PF = 1
#define CLASS_JS         0x54	// 84 -- Jump if SF = 1
#define CLASS_LEA        0x56	// 86 -- Load effective address
#define CLASS_LCALL      0x57	// 87
#define CLASS_PUSHF      0x58	// 88 -- Push eflags on stack (lower 16 bits)
#define CLASS_PUSHFD     0x59	// 89 -- Push eflags on stack (all)
#define CLASS_POPF       0x5A	// 90 -- Pop stack into eflags (lower 16 bits)
#define CLASS_POPFD      0x5B	// 91 -- Pop stack into eflags (all)
#define CLASS_SAHF       0x5C	// 92 -- Store AH into flags
#define CLASS_LAHF       0x5D	// 93 -- Load AH from flags
#define CLASS_CMPS       0x5E	// 94 -- Compare string
#define CLASS_STOS       0x5F	// 95 -- Store string
#define CLASS_LODS       0x60	// 96 -- Load string
#define CLASS_SCAS       0x61	// 97 -- Scan string
#define CLASS_LDS        0x62	// 98
#define CLASS_LES        0x63	// 99
#define CLASS_LFS        0x64	// 100
#define CLASS_LGS        0x65	// 101
#define CLASS_LSS        0x66	// 102
#define CLASS_SCASB      0x67	// 103
#define CLASS_SCASD      0x68	// 104
#define CLASS_SCASW      0x69	// 105
#define CLASS_LRET       0x6A	// 106
#define CLASS_INT3       0x6B	// 107
#define CLASS_INTO       0x6C	// 108
#define CLASS_IRET       0x6D	// 109
#define CLASS_MADJUST    0x6E	// 110 -- Adjust after multiply
#define CLASS_DADJUST    0x6F	// 111 -- Adjust after divide
#define CLASS_XLAT       0x70	// 112
#define CLASS_XLATB      0x71	// 113
#define CLASS_LOOPNE     0x72	// 114
#define CLASS_LOOPE      0x73	// 115
#define CLASS_LJUMP      0x74	// 116
#define CLASS_CMC        0x75	// 117
#define CLASS_STC        0x76	// 118
#define CLASS_STI        0x77	// 119
#define CLASS_STD        0x78	// 120
#define CLASS_CLC        0x79	// 121
#define CLASS_CLI        0x7A	// 122
#define CLASS_CLD        0x7B	// 123
#define CLASS_LAR        0x7C	// 124
#define CLASS_LSL        0x7D	// 125
#define CLASS_CLTS       0x7E	// 126
#define CLASS_INVD       0x7F	// 127
#define CLASS_WBINVD     0x80	// 128
#define CLASS_FEMMS      0x81	// 129
#define CLASS_UD2        0x82	// 130 -- Undefined instruction
#define CLASS_WRMSR      0x83	// 131
#define CLASS_RDTSC      0x84	// 132
#define CLASS_RDMSR      0x85	// 133
#define CLASS_RDPMC      0x86	// 134
#define CLASS_CMOVO      0x87	// 135
#define CLASS_CMOVNO     0x88	// 136
#define CLASS_CMOVB      0x89	// 137
#define CLASS_CMOVAE     0x8A	// 138
#define CLASS_CMOVE      0x8B	// 139
#define CLASS_CMOVNE     0x8C	// 140
#define CLASS_CMOVBE     0x8D	// 141
#define CLASS_CMOVA      0x8E	// 142
#define CLASS_CMOVS      0x8F	// 143
#define CLASS_CMOVNS     0x90	// 144
#define CLASS_CMOVP      0x91	// 145
#define CLASS_CMOVNP     0x92	// 146
#define CLASS_CMOVL      0x93	// 147
#define CLASS_CMOVGE     0x94	// 148
#define CLASS_CMOVLE     0x95	// 149
#define CLASS_CMOVG      0x96	// 150
#define CLASS_SETO       0x97	// 151
#define CLASS_SETNO      0x98	// 152
#define CLASS_SETB       0x99	// 153
#define CLASS_SETAE      0x9A	// 154
#define CLASS_SETE       0x9B	// 155
#define CLASS_SETNE      0x9C	// 156
#define CLASS_SETBE      0x9D	// 157
#define CLASS_SETA       0x9E	// 158
#define CLASS_SETS       0x9F	// 159
#define CLASS_SETNS      0xA0	// 160
#define CLASS_SETP       0xA1	// 161
#define CLASS_SETNP      0xA2	// 162
#define CLASS_SETL       0xA3	// 163
#define CLASS_SETGE      0xA4	// 164
#define CLASS_SETLE      0xA5	// 165
#define CLASS_SETG       0xA6	// 166
#define CLASS_RSM        0xA7	// 167
#define CLASS_BTS        0xA8	// 168
#define CLASS_SHRD       0xA9	// 169
#define CLASS_CMPXCHG    0xAA	// 170
#define CLASS_BTR        0xAB	// 171
#define CLASS_MOVZB      0xAC	// 172
#define CLASS_MOVZW      0xAD	// 173
#define CLASS_BSR        0xAE	// 174
#define CLASS_BSF        0xAF	// 175
#define CLASS_MOVSW      0xB0	// 176
#define CLASS_MOVSB      0xB1	// 177
#define CLASS_XADD       0xB2	// 178 -- Exchange and add
#define CLASS_BSWAP      0xB3	// 179
#define CLASS_BTC        0xB4	// 180
#define CLASS_ROL        0xB5	// 181
#define CLASS_ROR        0xB6	// 182
#define CLASS_RCL        0xB7	// 183
#define CLASS_RCR        0xB8	// 184
#define CLASS_SHL        0xB9	// 185
#define CLASS_SHR        0xBA	// 186
#define CLASS_SAR        0xBB	// 187
#define CLASS_NOT        0xBC	// 188
#define CLASS_IDIV       0xBD	// 189
#define CLASS_SLDT       0xBE	// 190
#define CLASS_STR        0xBF	// 191
#define CLASS_LLDT       0xC0	// 192
#define CLASS_LTR        0xC1	// 193
#define CLASS_VERR       0xC2	// 194
#define CLASS_VERW       0xC3	// 195
#define CLASS_SGDT       0xC4	// 196
#define CLASS_SIDT       0xC5	// 197
#define CLASS_LGDT       0xC6	// 198
#define CLASS_LIDT       0xC7	// 199
#define CLASS_SMSW       0xC8	// 200
#define CLASS_LMSW       0xC9	// 201
#define CLASS_INVLPG     0xCA	// 202
#define CLASS_BT         0xCB	// 203
#define CLASS_CMPXCHG8B  0xCC	// 204
#define CLASS_PSRLW      0xCD	// 205
#define CLASS_PSRAW      0xCE	// 206
#define CLASS_PSLLW      0xCF	// 207
#define CLASS_PSRLD      0xD0	// 208
#define CLASS_PSRAD      0xD1	// 209
#define CLASS_PSLLD      0xD2	// 210
#define CLASS_PSRLQ      0xD3	// 211
#define CLASS_PSLLQ      0xD4	// 212
#define CLASS_CWTS       0xD5	// 213
#define CLASS_CSTD       0xD6	// 214
#define CLASS_FADD       0xD7	// 215
#define CLASS_FMUL       0xD8	// 216
#define CLASS_FCOM       0xD9	// 217
#define CLASS_FCOMP      0xDA	// 218
#define CLASS_FSUB       0xDB	// 219
#define CLASS_FSUBR      0xDC	// 220
#define CLASS_FDIV       0xDD	// 221
#define CLASS_FDIVR      0xDE	// 222
#define CLASS_FLD        0xDF	// 223
#define CLASS_FXCH       0xE0	// 224
#define CLASS_FCMOVB     0xE1	// 225
#define CLASS_FCMOVE     0xE2	// 226
#define CLASS_FCMOVBE    0xE3	// 227
#define CLASS_FCMOVU     0xE4	// 228
#define CLASS_FUCOMI     0xE5	// 229
#define CLASS_FCOMI      0xE6	// 230
#define CLASS_FCMOVNB    0xE7	// 231
#define CLASS_FCMOVNE    0xE8	// 232
#define CLASS_FCMOVNBE   0xE9	// 233
#define CLASS_FCMOVNU    0xEA	// 234
#define CLASS_FFREE      0xEB	// 235
#define CLASS_FSTL       0xEC	// 236
#define CLASS_FSTPL      0xED	// 237
#define CLASS_FUCOM      0xEE	// 238
#define CLASS_FUCOMP     0xEF	// 239
#define CLASS_FADDP      0xF0	// 240
#define CLASS_FMULP      0xF1	// 241
#define CLASS_FSUBP      0xF2	// 242
#define CLASS_FSUBRP     0xF3	// 243
#define CLASS_FDIVP      0xF4	// 244
#define CLASS_FDIVRP     0xF5	// 245
#define CLASS_FUCOMIP    0xF6	// 246
#define CLASS_FCOMIP     0xF7	// 247
#define CLASS_FLDENV     0xF8	// 248
#define CLASS_FLDCW      0xF9	// 249
#define CLASS_FNSTENV    0xFA	// 250
#define CLASS_FNSTCW     0xFB	// 251
#define CLASS_FIADD      0xFC	// 252
#define CLASS_FIMUL      0xFD	// 253
#define CLASS_FICOM      0xFE	// 254
#define CLASS_FICOMP     0xFF	// 255
#define CLASS_FISUB      0x100	// 256
#define CLASS_FISUBR     0x101	// 257
#define CLASS_FIDIV      0x102	// 258
#define CLASS_FIDIVR     0x103	// 259
#define CLASS_FILD       0x104	// 260
#define CLASS_FIST       0x105	// 261
#define CLASS_FISTP      0x106	// 262
#define CLASS_FLDL       0x107	// 263
#define CLASS_FRSTOR     0x108	// 264
#define CLASS_FNSAVE     0x109	// 265
#define CLASS_FNSTSW     0x10A	// 266
#define CLASS_FBLD       0x10B	// 267
#define CLASS_FBSTP      0x10C	// 268
#define CLASS_FLDT       0x10D	// 269
#define CLASS_FSTPT      0x10E	// 270
#define CLASS_FNOP       0x10F	// 271
#define CLASS_FCHS       0x110	// 272
#define CLASS_FABS       0x111	// 273
#define CLASS_FTST       0x112	// 274
#define CLASS_FXAM       0x113	// 275
#define CLASS_FLD1       0x114	// 276
#define CLASS_FLDL2T     0x115	// 277
#define CLASS_FLDL2E     0x116	// 278
#define CLASS_FLDPI      0x117	// 279
#define CLASS_FLDLG2     0x118	// 280
#define CLASS_FLDLN2     0x119	// 281
#define CLASS_FLDZ       0x11A	// 282
#define CLASS_F2XM1      0x11B	// 283
#define CLASS_FYL2X      0x11C	// 284
#define CLASS_FPTAN      0x11D	// 285
#define CLASS_FPATAN     0x11E	// 286
#define CLASS_FXTRACT    0x11F	// 287
#define CLASS_FPREM1     0x120	// 288
#define CLASS_FDECSTP    0x121	// 289
#define CLASS_FINCSTP    0x122	// 290
#define CLASS_FPREM      0x123	// 291
#define CLASS_FYL2XP1    0x124	// 292
#define CLASS_FSQRT      0x125	// 293
#define CLASS_FSINCOS    0x126	// 294
#define CLASS_FRNDINT    0x127	// 295
#define CLASS_FSCALE     0x128	// 296
#define CLASS_FSIN       0x129	// 297
#define CLASS_FCOS       0x12A	// 298
#define CLASS_FUCOMPP    0x12B	// 299
#define CLASS_FNCLEX     0x12C	// 300
#define CLASS_FNINIT     0x12D	// 301
#define CLASS_FCOMPP     0x12E	// 302
#define CLASS_FADDS      0x12F	// 303
#define CLASS_FMULS      0x130	// 304
#define CLASS_FCOMS      0x131	// 305
#define CLASS_FCOMPS     0x132	// 306
#define CLASS_FSUBS      0x133	// 307
#define CLASS_FSUBRS     0x134	// 308
#define CLASS_FDIVS      0x135	// 309
#define CLASS_FDIVRS     0x136	// 310
#define CLASS_FLDS       0x137	// 311
#define CLASS_FSTS       0x138	// 312
#define CLASS_FSTPS      0x139	// 313
#define CLASS_FILDLL     0x13A	// 314
#define CLASS_FISTPLL    0x13B	// 315
#define CLASS_ERROR      0x13C	// 316
#define CLASS_FIMULL     0x13D	// 317
#define CLASS_FMULL      0x13E	// 318
#define CLASS_GRPS       0x13F	// 319
#define CLASS_FIADDL     0x140	// 320
#define CLASS_FICOML     0x141	// 321
#define CLASS_FICOMPL    0x142	// 322
#define CLASS_FISUBL     0x143	// 323
#define CLASS_FISUBRL    0x144	// 324
#define CLASS_FIDIVL     0x145	// 325
#define CLASS_FIDIVRL    0x146	// 326
#define CLASS_FILDL      0x147	// 327
#define CLASS_FISTL      0x148	// 328
#define CLASS_FISTPL     0x149	// 329
#define CLASS_FADDL      0x14a	// 330
#define CLASS_FCOML      0x14b	// 331
#define CLASS_FCOMPL     0x14c	// 332
#define CLASS_FSUBL      0x14d	// 333
#define CLASS_FSUBRL     0x14e	// 334
#define CLASS_FDIVL      0x14f	// 335
#define CLASS_FDIVRL     0x150	// 336
#define CLASS_FST        0x151	// 337
#define CLASS_FSTP       0x152	// 338
#define CLASS_PUSHES     0x153  // 339
#define CLASS_POPES	 0x154  // 340
#define CLASS_PUSHCS	 0x155  // 341
#define CLASS_PUSHDS	 0x156	// 342
#define CLASS_POPDS	 0x157  // 343
#define CLASS_PUSHSS	 0x158  // 344
#define CLASS_POPSS      0x159  // 345
#define CLASS_CPUID	 0x15a  // 346
#define CLASS_PUSHEFLAGS 0x15b  // 347
#define CLASS_POPEFLAGS  0x15c  // 348
#define CLASS_UD2B	 0x15d  // 349
#define CLASS_PMULLW	 0x15e	// 350
#define CLASS_LJUMP1     0x15f  // 351
#define CLASS_INCL       0x160  // 352
#define CLASS_MOV_SEG    0x161  // 352
#define CLASS_SHLB       0x162  // 352

#define NUMBER_OF_CLASSES              354



/*
 *  Aliases for jump instructions. Since something like JGE is the same
 *  as JNL, and they even have the same opcode, we want to be able to
 *  use both classes to represent the same instruction
 */
#define CLASS_JC         CLASS_JB
#define CLASS_JECXZ      CLASS_JCXZ
#define CLASS_JNA        CLASS_JBE
#define CLASS_JNAE       CLASS_JB
#define CLASS_JNB        CLASS_JAE
#define CLASS_JNBE       CLASS_JA
#define CLASS_JNC        CLASS_JAE
#define CLASS_JNG        CLASS_JLE
#define CLASS_JNGE       CLASS_JL
#define CLASS_JNL        CLASS_JGE
#define CLASS_JNLE       CLASS_JG
#define CLASS_JNZ        CLASS_JNE
#define CLASS_JPE        CLASS_JP
#define CLASS_JPO        CLASS_JNP
#define CLASS_JZ         CLASS_JE

void xInstrSetClassFlags(Instruction *instr);

#endif				// X86_CLASSES_H
