/*
 *  x86-liveness.h
 *
 *  Functions for register liveness analysis on the x86
 */
#ifndef X86_LIVENESS_H
#define X86_LIVENESS_H

/*
 *  Register bitvector fields
 *  The following definitions link a register with a bit in a register
 *  bitvector.
 */
#define BITVECTOR_REGISTER_NONE  (0 << 0)
#define BITVECTOR_REGISTER_EAX   (1 << 0)
#define BITVECTOR_REGISTER_EBX   (1 << 1)
#define BITVECTOR_REGISTER_ECX   (1 << 2)
#define BITVECTOR_REGISTER_EDX   (1 << 3)
#define BITVECTOR_REGISTER_ESP   (1 << 4)
#define BITVECTOR_REGISTER_EBP   (1 << 5)
#define BITVECTOR_REGISTER_ESI   (1 << 6)
#define BITVECTOR_REGISTER_EDI   (1 << 7)
#define BITVECTOR_REGISTER_ST    (1 << 8)
#define BITVECTOR_REGISTER_ES    (1 << 9)
#define BITVECTOR_REGISTER_CS    (1 << 10)
#define BITVECTOR_REGISTER_SS    (1 << 11)
#define BITVECTOR_REGISTER_DS    (1 << 12)
#define BITVECTOR_REGISTER_FS    (1 << 13)
#define BITVECTOR_REGISTER_GS    (1 << 14)

/* Returns true if the register is a 32 bit register */
bool _RegisterIs32Bit(int areg);

/* Returns true if the register is a 16 bit register */
bool _RegisterIs16Bit(int areg);

/* Returns true if the register is an 8 bit register */
bool _RegisterIs8Bit(int areg);

/* Returns a register bitvector with a bit set for each register that is
 * used by 'instr' */
REGISTER_BITVECTOR xComputeRegistersUsed(Instruction *instr);

/* Returns a register bitvector with a bit set for each register that is
 * completely defined by 'instr' */
REGISTER_BITVECTOR xComputeRegistersDefined(Instruction *instr);

/*
 *  Returns a register bitvector with a bit set for each register that is
 *  completely or partially defined by 'instr'
 */
REGISTER_BITVECTOR xComputeRegistersDefinedLiberal(Instruction *instr);

/* Returns a PSW bitvector with a bit set for each PSW flag 'instr' uses */
PSW_BITVECTOR xComputePSWUsed(Instruction *instr);

/* Returns a PSW bitvector with a bit set for each PSW flag that is defined
 * by 'instr' */
PSW_BITVECTOR xComputePSWDefined(Instruction *instr);

/* Returns a PSW bitvector with a bit set for each flag that
 * the instruction may define */
PSW_BITVECTOR xComputePSWDefinedLiberal(Instruction *instr);

/* Fills in the integer array with true or false
 * depending on which stack slots 'instr' uses */
void xComputeStackUsed(Instruction *instr, int *slots, int bp_index,
    int arsize);

/* Fills in the integer array with true or false
 * depending on which stack slots 'instr' defines */
void xComputeStackDefined(Instruction *instr, int *slots, int bp_index,
    int arsize);

/* Fills in the integer array with true or false
 * depending on which stack slots might be defined by 'instr' */
void xComputeStackDefinedLiberal(Instruction *instr, int *slots, int bp_index,
    int arsize);

/* Returns the register that the bitvector bit 'i' corresponds to */
int xBitRegisterToRegister(int i);

/* Returns true if the register is live in the bitvector */
bool xRegisterIsLiveInBitvector(int aReg, REGISTER_BITVECTOR vec);

#endif				// X86_LIVENESS_H
