/*
 *  x86-util.h
 *
 *  x86 general purpose functions.
 */
#ifndef X86_UTIL_H
#define X86_UTIL_H

/*
 *  Updates the bytemode of the instruction's operand 'op'.
 *  false is returned if something terrible happened,
 *  e.g: we couldn't find the instruction in the opcode table
 */
bool UpdateBytemode(Instruction *instr, Operand *op);

/* Returns true if the instruction can be assembled into machine code */
bool xIsValidInstr(Instruction *instr);

/* Returns the number of prefixes 'instr' will use */
int numberOfPrefixes(Instruction *instr);

/*
 *  Updates the bytemode and length of the instruction. false is returned
 *  if the instruction could not be updated correctly
 */
bool xUpdateInstr(Instruction *instr);

/*
 *  Sometimes we may get 0x00 bytes in the text segment that are used for
 *  alignment purposes. These are horrible, and we wish to eliminate them.
 *  This function will report all occurrences of the opcode "00"
 *  since it is very very unlikely a compiler would ever generate this
 *  instruction. For reference 00 adds the 8 bit register AL to the
 *  8 bit memory location specified as an operand.
 */
void xReportSuspiciousInstrs();

/* Returns true if the operand is a floating point register */
bool isSTRegister(int the_reg);

/* Returns true if the instruction is a conditional branch */
bool xIsConditionalInstr(Instruction *inst);

/* Returns true if the instruction is an unconditional control transfer */
bool xIsUnconditionalInstr(Instruction *inst);

/* Returns a pointer to a textual representation of an instruction */
char *xPrintInstruction(Instruction *instr);

/* Returns a pointer to a textual representation of an instruction,
 * using its final address */
char *xPrintInstructionFinal(Instruction *instr);

char *xPrintRegister(int i);

/* Replaces nonsupported instructions with supported ones */
void xReplaceNonsupportedInstructions();

/* Returns the size, in bytes, of the instruction cache */
int xGetIcacheSize();

/* Returns the size, in bytes, of the data cache */
int xGetDcacheSize();

/* Returns the megahertz rating of the processor */
int xGetMegahertz();

/* Returns the penalty, expressed as the number of clock ticks,
 * for an icache miss */
int xGetIcacheMissPenalty();

/* Returns the penalty, expressed as the number of clock ticks, for a CALL */
int xGetCallPenalty();

/* Constructs a hash table for the opcodes */
void xBuildOpcodeHashTable();

/* Returns true if the register is a frame pointer (EBP) */
bool xRegisterIsFramePointer(int aReg);

/* Returns true if the register is a stack pointer (ESP) */
bool xRegisterIsStackPointer(int aReg);

/* Returns true if the register is a floating point register */
bool RegisterIsFloatingPoint(int aReg);

/* Returns true if the instruction can define the PSW */
bool xInstrDefinesPSW(Instruction *instr);

/* Converts the 386 table register to the PLTO register */
int ConvertRegister(int aReg);

UINT8 CvtJcc2SETcc_compl(Instruction *iins);
bool isConditionalMove(Instruction *instr);

bool xInstrUsesReg(Instruction *iins, int areg);
bool xInstrDefinesReg(Instruction *iins, int areg);
bool xInstrUsesDest(Instruction *instr);
Instruction *xPICGetALUForCall(Instruction *call_instr);
extern int xGetFramePointer();
extern int xGetStackPointer();
extern int xGetARegister();
extern int xGetSyscallNoForSyscall(Instruction *syscall);
extern void xRemoveSyscallNoForSyscall(Instruction *syscall);

#endif				// X86_UTIL_H
