/*
 *  This file contains stuff specific to the Block data structure
 *  which represents a single basic block.
 */
#include <stdlib.h>
#include "bbl.h"
#include "utilities.h"



/*
 *  Private Variable
 */
static bool NoBBLAuxInfo = false;



/*
 *  BBLFree(bbl) --
 *
 *  Frees up the memory used by the basic block.  Presumably all the
 *  instructions contained in the basic block have been killed, and the
 *  basic block is disconnected from the control flow graph.
 *
 *  bbl: the basic block whose memory should be freed
 */
void BBLFree(Block *bbl) {
    CASSERT(bbl != NULL, "Trying to free a NULL basic block");

    if (bbl->b_auxinfo != NULL) {
	free(bbl->b_auxinfo);
    }

    free(bbl);
}



/*
 *  BBLCreate() -- returns a pointer to a new basic block.
 */
Block *BBLCreate() {
    Block *bbl;
    static int bbl_id = 0;

    bbl = BBLCreateBase();
    if (!NoBBLAuxInfo) {
	bbl->b_auxinfo = BBLCreateAuxinfo();
    } else {
	bbl->b_auxinfo = NULL;
    }
    bbl->b_id = bbl_id++;

    return bbl;
}



/*
 *  BBLCreateAuxinfo() -- returns a pointer to a new bbl aux info.
 */
BlockInfo *BBLCreateAuxinfo() {
    return zalloc(sizeof(BlockInfo));
}



/*
 *  BBLCreateBase() --
 *
 *  Returns a pointer to a new basic block base.
 *  returns: a pointer to a new basic block base.
 */
Block *BBLCreateBase() {
    return zalloc(sizeof(Block));
}
