/*
 *  cache.analyze.h
 *
 *  Functions for performing icache/dcache analysis
 */
#ifndef CACHE_ANALYZE_H
#define CACHE_ANALYZE_H

typedef struct LoopList {
    Block *oo_header;
    struct LoopList *oo_next;
    struct LoopList *oo_prev;
} LoopList;


/*
 *  ComputeLoopSize will analyze the loop (specified by the first basic
 *  block starting the loop, LoopHeader) and return a fairly accurate
 *  estimate for the size (in bytes) that the loop will consume. Function
 *  calls are recursively examined
 */
int ComputeLoopSize(Block *LoopHeader);

/*
 *  ComputeFunSize will analyze the supplied function and return a number
 *  that represents how large the function is, in bytes. This will not
 *  examine any sub-functions called by 'fun'
 */
int ComputeFunSize(Function *fun);

/*
 *  ComputeFunSizeRecursively will analyze the supplied function and return a
 *  number that represents how large the function and its subfunctions are,
 *  in bytes.
 */
void ComputeFunSizeRecursively(Function *fun, EDGE_WEIGHT_TYPE weight);

/*
 *  ComputeBBLSize will return the number of bytes the supplied basic block
 *  will consume if put into the icache.
 */
int ComputeBBLSize(Block *bbl);

/*
 *  Returns true if it is probably a good idea to inline the given function
 *  from the given callsite.  A cache analysis is performed using an icache
 *  model.
 */
bool GoodIdeaToInlineFunFromCallsite(Function *fun, Instruction *callSite);

/* Returns the loops that contain the supplied instruction */
LoopList *GetLoops(Instruction *callInstr);

#endif				// CACHE_ANALYZE_H
