/*
 *  This file contains stuff specific to the Function data
 *  structure which represents a function.
 */
#include <stdlib.h>
#include "function.h"
#include "utilities.h"
#include "flowgraph.h"



/*
 *  FunFree(fun) --
 *
 *  Frees up the memory associated with the specified function.
 *
 *  fun: the function whose memory should be freed
 */
void FunFree(Function *fun) {

    CASSERT(fun->f_entry != NULL, "No entry block found for function.");
    CASSERT(fun->f_exit != NULL, "No exit block found for function.");

    BBLRemoveEdges(fun->f_entry);
    BBLRemoveEdges(fun->f_exit);
    BBLFree(fun->f_entry);
    BBLFree(fun->f_exit);
    if (fun->f_name != NULL) {
	free(fun->f_name);
    }
    free(fun);
}



/*
 *  FunCreate() --
 *
 *  Returns a pointer to a new Function.
 */
Function *FunCreate() {
    Function *newfun;
    Block *enterBlock;
    Block *exitBlock;
    static int fun_id = 0;

    newfun = zalloc(sizeof(Function));
    enterBlock = BBLCreate();
    exitBlock = BBLCreate();

    enterBlock->b_type = BT_ENTRY;
    exitBlock->b_type = BT_EXIT;

    enterBlock->b_fun = newfun;
    exitBlock->b_fun = newfun;

    newfun->f_exit = exitBlock;
    newfun->f_entry = enterBlock;
    newfun->f_id = fun_id++;

    return newfun;
}
