/*
 *  This file contains stuff specific to the Instruction data structure
 *  which represents a single instruction.
 */
#include <stdlib.h>
#include "instr.h"
#include "utilities.h"



/*
 *  InstrCreate() --
 *
 *  Returns a pointer to a new Instruction.
 */
Instruction *InstrCreate() {
    Instruction *instr;
    static int instr_id = 0;

    instr = zalloc(sizeof(Instruction));
    instr->i_id = instr_id++;
    instr->i_address = MAGIC_NUMBER;

    return instr;
}



/*
 *  InstrFree(instr) --
 *
 *  Frees up the memory used by the instruction and all of its operands.
 *  Presumably the instruction has already been disconnected from the
 *  flowgraph and list of instructions in a basic block at this point.
 *
 *  instr: the instruction to free up memory for
 */
void InstrFree(Instruction *instr) {
    CASSERT(instr != NULL, "Passed a NULL instruction.");
    /* Free up the operands */
    if (instr->i_dest != NULL) {
	free(instr->i_dest);
    }
    if (instr->i_src != NULL) {
	free(instr->i_src);
    }
    if (instr->i_src2 != NULL) {
	free(instr->i_src2);
    }
    free(instr);
    return;
}
