/*
 *  opt.h -- declarations for optimization files
 */

#ifndef OPT_H
#define OPT_H

#include "d_graph.h"
#include "global.h"
#include <stdlib.h>



/*
 *  opt.inline.c -- function inlining
 */
#define INLINE_ALWAYS_INSTRUCTION_CUTOFF 6
extern void OptInlineFuns();



/*
 *  opt.jt.c -- jump table optimization
 */
void OptimizeJumpTables();



/*
 *  opt.memopt.c -- memory optimization
 */
bool InstrLoadsRegFromMemory(Instruction *iins);
bool InstrStoresRegToMemory(Instruction *iins);
bool InstrStoresToMemory(Instruction *iins);
bool OperandsAreIdentical(Operand *op1, Operand *op2);
bool OperandsMayAlias(Operand *op1, Operand *op2);
void OptLoadStoreForward();



/*
 *  opt.peephole.c -- peephole and other miscellaneous optimizations
 */
void PeepholeOptimize();
void NormalizeOperations();
void ExtendSecondJumps();



/*
 *  opt.schedule.c -- instruction scheduler
 */

/*
 *  SchedPoints
 *
 *  This is the data we'd like to associate with each instruction for
 *  scheduling.
 */
typedef struct SchedPoints {
    UINT32 sp_height;		// height of the dependence DAG at any vertex
    UINT32 sp_pairings_below;	// number of pairings below a vertex
    UINT32 sp_cycle;		// when scheduled
    UINT32 sp_points;		// point value awarded to instruction
    Instruction *sp_instr;	// instruction
    bool sp_scheduled;		// whether or not scheduled
} SchedPoints;

#define SCH_POINTS_HEIGHT(v)	((v)->dv_data->sp_height)
#define SCH_POINTS_PAIRINGS(v)	((v)->dv_data->sp_pairings_below)
#define SCH_POINTS_CYCLE(v)	((v)->dv_data->sp_cycle)
#define SCH_POINTS_INSTR(v)	((v)->dv_data->sp_instr)
#define SCH_POINTS_SCHEDULED(v)	((v)->dv_data->sp_scheduled)
#define SCH_POINTS_POINTS(v)	((v)->dv_data->sp_points)

/*
 *  SchedNode
 */
typedef struct SchedNode {
    DigraphVertex *sn_vertex;		// vertex in dependence DAG
    struct SchedNode *sn_next;		// next vertex
} SchedNode;

SchedNode *SchInitInstrList();
SchedNode *SchAddInstrList(SchedNode *list, DigraphVertex *vertex);
SchedNode *SchRemoveInstrList(SchedNode *list, DigraphVertex *vertex);
SchedNode *SchReverseInstrList(SchedNode *list);
void SchDestroyInstrList(SchedNode **list);

void ScheduleInstructions();



/*
 *  opt.specialize.c -- specialization
 */
void ValueSpecialize();

/*
 * opt.compaction.c -- code compaction
 */
bool InstrsAreIndetical(Instruction *ins1, Instruction *ins2);
bool BBLsAreIdentical(Block *bbl1, Block *bbl2);
void OptCompact();



#endif				// OPT_H
