/*  jumble.c -- a mishmash of various constructions  */

#include <math.h>
#include <stdlib.h>
#include <stdio.h>

typedef struct { int x; int y; } coord;



int operate(int i) {			/* lots of straight-line operations */
    int a = 17 * i / 5;
    int b = 19 * i / 4;
    int c = (a + b) * (b - a);
    int d = a | b;
    int e = a & b;
    int f = c % d + e + ~i;
    return f;
}


int fibonacci(int i) {			/* recursion */
    if (i < 3)
	return 1;
    else
    	return fibonacci(i - 1) + fibonacci(i - 2);
}


int wondrous(int i) {			/* if-then-else */
    if (i < 3)
       return 1;
    else if (i & 1)
       return 3 * i + 1;
    else
       return i >> 1;
}


int both(int i) {			/* second-level calls */
    return fibonacci(i) % wondrous(i);
}


int fewcases(int i) {			/* few cases */
    switch (i % 3) {
    	case 0:	 return 2;
    	case 1:	 return 3;
    	case 2:	 return 1;
    }
    return 0;	/*NOTREACHED*/
}


int manycases(int i) {			/* many cases */
    switch (i % 21) {
    	case  0:  return 73;
    	case  1:  return 13;
    	case  2:  return 47;
    	case  3:  return 67;
    	case  4:  return 43;
    	case  5:  return 59;
    	case  6:  return 61;
    	case  7:  return 41;
    	case  8:  return 23;
    	case  9:  return 53;
    	case 10:  return 97;
    	case 11:  return 31;
    	case 12:  return 83;
    	case 13:  return 19;
    	case 14:  return 29;
    	case 15:  return 17;
    	case 16:  return 79;
    	case 17:  return 89;
    	case 18:  return 37;
    	case 19:  return 71;
    	case 20:  return 11;
    }
    return 0;	/*NOTREACHED*/
}


int sparsecases(int i) {		/* sparse cases */
    switch (i % 20) {
	default:  return 0;
	case  3:  return 1;
	case 18:  return 2;
	case 47:  return -1;  /* impossible */
    }
}


int nestcases(int i) {			/* nested cases */
    switch (i % 2) {
    	case 0:
	    switch (i % 3) {
	    	case 0:   return 0;
	    	case 1:   return 1;
	    	case 2:   return 2;
		default:  return -1;
	    }
	case 1:
	    switch (i % 5) {
	    	case 0:   return 5;
	    	case 1:   return 6;
	    	case 2:   return 7;
	    	case 3:   return 8;
	    	case 4:   return 9;
		default:  return -1;
	    }
	default:
	    return -1;
    }
}


coord point(int x, int y) {		/* struct return */
    coord new;
    new.x = x;
    new.y = y;
    return new;
}


int distance(coord a, coord b) {	/* struct args */
    int dx = a.x - b.x;
    int dy = a.y - b.y;
    return (int) (0.5 + sqrt(dx * dx + dy * dy));
}


float floater(float x) {
    return sqrt(10 * x) + 5 * sin(3 * x);
}



int main(int argc, char *argv[]) {

    int i;
    int n = (argc > 1) ? atoi(argv[1]) : 27;

    /* constant calls */
    printf("%3d. %3d %6d %3d% 3d %3d %3d %3d %3d %3d %3d %3d %3d\n",
	3, operate(3), fibonacci(3), wondrous(3), both(3),
	fewcases(3), manycases(3), sparsecases(3), nestcases(3),
	point(6,6).x, point(7,9).y, 3, (int)floater(3));

    /* variable calls */
    for (i = 1; i <= n; i++) {
	int opr = operate(i);
	int fib = fibonacci(i);
    	int wnd = wondrous(i);
	int bo  = both(i);
	int few = fewcases(i);
	int mny = manycases(i);
	int spr = sparsecases(i);
	int nst = nestcases(i);
	coord a = point(i + 3 - i % 3, 2 * i);
	coord b = point(i + 4, 2 * i + i % 5);
	int dist = distance(a, b);
	int flt = floater(i);
	printf("%3d. %3d %6d %3d% 3d %3d %3d %3d %3d %3d %3d %3d %3d\n",
	    i, opr, fib, wnd, bo, few, mny, spr, nst, a.x, b.y, dist, flt);
    }

    return 0;
}

