#!/bin/sh
#
#  runtest.sh -- runs one or more C programs through PLTO and checks results.
#
#  usage:  runtest.sh prog.c ...
#
#  Passes prog.dat or prog.gz, if present, as standard input to the program.
#
#  Uses a copy of ../../bin/plto.defaults, appending prog.plto if present.

CC="gcc -Wall -O"
TLIMIT=200
V="-v4"

PLTO=../../bin/plto
FAILED=


set -x
ulimit -t $TLIMIT
ls -l $PLTO || exit 1
:

for A in "$@"; do
    : -------------------------------------------- Testing $A:
    B=${A%.c}
    test -f $B.plto && export PLTO_CONFIG_FILE=$B.plto || unset PLTO_CONFIG_FILE
    test -f $B.dat  && I=$B.dat || I=/dev/null
    test -f $B.gz   && I=$B.tmp && zcat $B.gz >$I
    rm -f $B.out? $B.log? $B.o $B.r $B.exe
    rm -f $B.counts $B.opt $B.nopt $B.prof
    $CC -c $B.c
    $CC -o $B.exe $B.o -lm
    $CC -o $B.r -static -Wl,-r $B.o -lm
						# check for:
    ./$B.exe <$I >$B.out0 &&			# program runs & exits normally
	$PLTO $V -n -o $B.nopt $B.r >$B.log1 &&	# plto -n runs
	[ -z `find $B.log1 -size +50` ] &&	# log is not too voluminous
	./$B.nopt <$I >$B.out1 &&		# processed version runs
	cmp $B.out0 $B.out1 &&			# output is correct
	$PLTO $V -pa $B.r >$B.log2 &&		# plto -l adds profiling
	[ -z `find $B.log2 -size +50` ] &&	# log is not too voluminous
	./$B.prof <$I >$B.out2 &&		# program runs with profiling
	cmp $B.out0 $B.out2 &&			# output is still correct
	$PLTO $V $B.r $B.counts >$B.log3 &&	# plto runs and optimizes
	[ -z `find $B.log3 -size +100` ] &&	# log is not too voluminous
	./$B.opt <$I >$B.out3 &&		# optimized program runs
	cmp $B.out0 $B.out3 &&			# output is again correct
	true ||
	    FAILED="$FAILED $B"			# otherwise catch any failure
done

set -
echo
if [ "$FAILED" = "" ]; then
    echo + : -------------------------------------------- Summary: ALL PASSED
    echo "all tests passed"
    exit 0
else
    echo + : -------------------------------------------- Summary: TESTS FAILED:
    echo "	$FAILED"
    exit 1
fi
