#!/bin/sh
#
#  tspec [bmk...] -- test plto with SPEC benchmarcs
#
#  Assumes pre-built relocatables in $SPECDIR/*/$RELFILE, where
#  SPECDIR and RELFILE are set in environment or defaulted below.
#
#  Note: Some tests require write access within $SPECDIR/*bm/input.
#
#  Uses a copy of ../../bin/plto.defaults, appending prog.plto if present.

#  set directories, inheriting from environment if available
SPECDIR=${SPECDIR-/cs/solar/plto/benchmarks/specint2000}
RELFILE=${RELFILE-g.rr.O3}

#  make list of benchmarks
#  the "ALL" list excludes nonfunctional C++ benchmark "eon"
ALLBMK="bzip2 crafty gap gcc gzip mcf parser twolf vortex vpr perlbmk"
test $# = 0 && set - $ALLBMK


#  find the PLTO executable
HERE=`pwd`
ROOT=`cd ../..; pwd`
PLTO=$ROOT/bin/plto
ls -l $PLTO || exit 1
V="-v4"

#  link local files required by certain tests
rm -f eon.dat materials spectra.dat words 2.1.dict persons.1k
ln -s $SPECDIR/eon/eon.dat .
ln -s $SPECDIR/eon/materials .
ln -s $SPECDIR/eon/spectra.dat .
ln -s $SPECDIR/parser/words .
ln -s $SPECDIR/parser/2.1.dict .
ln -s $SPECDIR/vortex/persons.1k .

PASSED=
FAILED=
set -x


# iterate through the chosen tests
for B in "$@"; do
    :
    : ============================================================ $B
    BD=$SPECDIR/$B
    CF=$HERE/$B.counts
    rm -f input	$B.out* $B.err*
    rm -f $B.counts $B.opt $B.nopt $B.prof
    ln -s $BD/input .
    test -f $B.plto && export PLTO_CONFIG_FILE=$B.plto || unset PLTO_CONFIG_FILE
    cp $BD/$RELFILE $B.r				&&
	$PLTO $V -n -o $B.nopt $B.r >$B.log1		&&
	$BD/chk $B.nopt $B.out1 2>$B.err1		&&
	cmp $BD/chk.out.std $B.out1			&&
	$PLTO $V -pa -DCountsOutFile=$CF $B.r >$B.log2	&&
	$BD/chk $B.prof $B.out2 2>$B.err2		&&
	cmp $BD/chk.out.std $B.out2			&&
	$PLTO $V $B.r $B.counts >$B.log3		&&
	$BD/chk $B.opt $B.out3 3>$B.err3		&&
	cmp $BD/chk.out.std $B.out3			&&
	PASSED="$PASSED $B"				||
	FAILED="$FAILED $B"
    test -f $B.out1 && diff $BD/chk.out.std $B.out1 | head -n 40
    test -f $B.out2 && diff $BD/chk.out.std $B.out2 | head -n 40
    test -f $B.out3 && diff $BD/chk.out.std $B.out3 | head -n 40
done
rm -f input


# generate a summary
set -
echo
if [ "$FAILED" = "" ]; then
    echo + : ============================================ ALL TESTS PASSED
    echo "all tests passed"
    exit 0
else
    echo + : ============================================ SOME TESTS FAILED:
    echo + " FAILED: $FAILED"
    echo + " PASSED: $PASSED"
    exit 1
fi
