#include "sloop.h"
#include "opts.h"
#include "main.h"

extern funcptr Start();
extern void pcgraph(int argc, char *argv[]);
extern void procopts(int argc, char *argv[]);
extern void getphases(int argc, char *argv[]);
extern void getifile(int argc, char *argv[]);
extern int instrs_reqd(int argc, char *argv[]);
extern gobjptr allfuncs, allloops, allbbls;
extern int func_p, loop_p, bbl_p, need_instrs;
extern char *ofilename, *dotfilename, *dotpath, *dotopts;
extern FILE *dotfile;

void analyze(funcptr f);

#define CopyString(x) {for (cptr1 = x; *cptr1 != '\0'; ) {\
			  *cptr0++ = *cptr1++;\
		        }\
			*cptr0++ = ' ';\
		      }

char *dot_cmd()
{
  int cmdlen, dotflen = 0, oflen = 0;
  char *cmd, *cptr0, *cptr1;

  if (dotfilename) dotflen = strlen(dotfilename);
  if (ofilename) oflen = strlen(ofilename);
  cmdlen = strlen(dotpath)
           + strlen(dotopts)
           + dotflen
	   + 4 /* " -o " */
	   + oflen
	   + 1 /* '\0' */ ;
  cmd = (char *)malloc(cmdlen);
  if (!cmd) error("Out of memory!");

  cptr0 = cmd;
  CopyString(dotpath);
  CopyString(dotopts);
  CopyString(dotfilename);
  if (ofilename) {
    *cptr0++ = '-';
    *cptr0++ = 'o';
    *cptr0++ = ' ';
    CopyString(ofilename);
  }
  *cptr0 = '\0';
  return cmd;
}

main(argc, argv)
int argc;
char *argv[];
{
  funcptr funcs;
  gobjptr g;
  char *dot_command;
  static char tname[] = "/tmp/sloop.XXXXXX";

  outfile = stdout;
  sloopfile = stdout;

  preproc_opts(argc, argv);
  need_instrs = instrs_reqd(argc, argv);

  funcs = Start();
  procopts(argc, argv);

  if (sloop_p) {
    pfunc(allfuncs);
  }

  if (dotfile_p) {
    dotfile = fopen(dotfilename, "w");
  } else {
    dotfile = fdopen(mkstemp(tname), "w");
    dotfilename = tname;
  }
  if (!dotfile) {
    fprintf(stderr, "Cannot open dot file: %s\n", dotfilename);
    exit(-1);
  }
  /*
   * compute edge info for those functions that need it
   */
  for (g = allfuncs; g; g = Next(g)) {
    if (Display(g) && !Processed(GFuncNode(g))) {
      analyze(GFuncNode(g));
    }
  }

  if (cgraph) {
    pcgraph(argc, argv);
  }

  if (is_verbose) {
    pverbose();
  }

  if (bbl_p) {
    print_dot(allbbls, BBLK);
  }
  if (loop_p) {
    print_dot(allloops, LOOP);
  }
  if (func_p) {
    print_dot(allfuncs, FUNC);
  }
  fclose(dotfile);

  /*
   * if a dot file was specified, don't run dot since the user will
   * probably want to modify the dot file himself.
   */
  if (!(dotfile_p || sloop_p)) {
    dot_command = dot_cmd();
    system(dot_command);
    unlink(dotfilename);
  }
  return 0;
}
