/* read in command line options */

#include <stdlib.h>
#include <ctype.h>
#include "sloop.h"
#include "opts.h"

extern void mk_callgraph();
extern void gen_callgraph_by_name(char *s);
extern void gen_callgraph_by_id(int fid);
extern void clear_visited_bits();
extern funcptr func_lookup(long fno);

extern FILE *yyin, *outfile, *sloopfile;
extern char *ofilename, *dotfilename;
extern char *phase_string;
extern int func_p, loop_p, bbl_p, instrs_p, sloop_p, tile, 
           dotfile_p, cpath, cgraph, is_verbose, do_scale;
extern gobjptr allfuncs, allloops, allbbls;

int argcount;
char **argvec;
char *bufptr, buf[1024];
int max_k, k, idval;
int is_int_str, is_float_str;
int cg_threshold = -1;
double owt, fidval;

#define WEIGHT 0
#define COUNT  1

#define CpyToBuf(cond) {while(cond) {\
			buf[i++] = ch; ch = nextch();\
			}\
			buf[i]='\0';\
		       }

#define GetRestOfOpt   {i = 0;\
			for (buf[i++] = nextch(); *bufptr != '\0'; ) {\
			  buf[i++] = *bufptr++;\
			}\
			buf[i] = 0;\
		       }

#if 0
#define Matched(x,typ) (typ==FUNC ? EqString(FuncName(GFuncNode(x)), &buf[0]) :\
			typ==LOOP ? Id(GLoopNode(x)) == idval :\
                        typ==BBLK ? Id(GBBlNode(x))  == idval : error("unrecognized nodetype"))
#endif

#define ForallObjs(Bod) {for (f=gptr; f; f = Next(f)) { Bod }}
#define MarkObjs(test)  {for (f=gptr,found=FALSE; f; f = Next(f)) {\
			   if (test) {\
			      Display(f)=TRUE;\
			      if (nodetype != FUNC) {\
				for (tmp=allfuncs;tmp;tmp=Next(tmp)) {\
				    if (GFuncNode(tmp) == GFuncNode(f)) {\
				       Display(tmp) = TRUE;\
				       break;\
				    }\
				 }\
			      }\
			     Display(GFuncNode(f)) = TRUE;\
			     found=TRUE;\
			  }\
		         }\
		        }
char nextch()
{
  if (k == argcount) return '\0';
 skipwhitesp:
  while (isspace(*bufptr) && *bufptr != '\0') bufptr++;
  if (*bufptr == '\0') {
    k++;
    if (k == argcount) {
      return '\0';
    }
    else {
      bufptr = *(argvec+k);
      goto skipwhitesp;
    }
  }
  else {   /* *bufptr != '\0' */
    return *bufptr++;
  }
}

void retract()
{
  if (bufptr == *(argvec+k)) {
    k--;
    bufptr = *(argvec+k);
    while (*bufptr++ != '\0')
      ;
  }
  --bufptr;
}

int oscan()
{
  char ch;
  int i;

  is_int_str = is_float_str = FALSE;
  while ((ch = nextch()) != '\0') {
    if (isalpha(ch) || ch == '_') {
      i = 0;
      CpyToBuf( (isalnum(ch) || ch == '_' || ch == '.') );
      retract();
      return IDENT;
    }
    i = 0;
    if (isdigit(ch)) {
      is_int_str = TRUE;
      CpyToBuf( isdigit(ch) );
      if (ch == '.') {
	is_int_str = FALSE;
	is_float_str = TRUE;
	ch = nextch();
	if (isdigit(ch)) {
	  buf[i++] = '0'; buf[i++] = '.';
	  CpyToBuf( isdigit(ch) );
	}
	else {
	  error("*** Malformed command line option: number expected after '.'\n");
	}
      }
      if (is_float_str) fidval = atof(&buf[0]);
      else idval = atoi(&buf[0]);
      retract();
      return IDENT;    /* idval/fidval has the numerical value */
    }
    switch (ch) {
    case ':' :
      i = 0;
      for (ch = nextch(); isdigit(ch); ch = nextch()) {
	buf[i++] = ch;
      }
      if (i == 0) 
	error("Malformed command line option: #...");
      max_k = atoi(&buf[0]);
      retract();
      return MAX_K;
    case ',' :
      return COMMA;
    case '=' :
      return EQ;
    case '+' :
      return ALL;
    case '-' : 
      ch = nextch();
      if (ch == '\0') {
	warning("*** Warning: incomplete command line option\n");
	return DONE;
      }
      switch (ch) {
      case 'C' : return CPATH1;
      case 'G' : return CGRAPH;
      case 'I' : return INSTS;
      case 'P' : return PHASE;
      case 'S' : return SCALE;
      case 'T' : return TILE;
      case 'b' : return OBBLK;
      case 'c' : return CPATH0;
      case 'd' : return DOT;
      case 'f' : return OFUNC;
      case 'g' : return CGSTOP;
      case 'i' : return IFILE;
      case 'l' : return OLOOP;
      case 'o' : return OFILE;
      case 's' : return SLOOP;
      case 't' : return THRSHLD;
      case 'v' : return VERBOSE;
      default : fprintf(stderr, "Unrecognized command line option: %c\n", ch);
	exit(-1);
      }
    default: return ch;
    }
  }
  return DONE;
}

int Matched(gobjptr x, int typ)
{
  long num;

  switch (typ) {
  case FUNC:
    if (isdigit(buf[0])) {
      num = atol(buf);
      return (Id(GFuncNode(x)) == num);
    }
    else {
      return EqString(FuncName(GFuncNode(x)), &buf[0]);
    }

  case LOOP:
    return Id(GLoopNode(x)) == idval;

  case BBLK:
    return Id(GBBlNode(x))  == idval;

  default:
    error("unrecognized nodetype");
  }
}

int match_opt_grp(int nodetype, gobjptr gptr)
{
  int otoken, nexttok, found, wtselector;
  long totwt, x;
  double threshold;
  char *id;
  gobjptr f, maxf, tmp, tmp0, tmp1;
  funcptr fptr;
  bblptr bptr;
  loopptr lptr;

  /*
   * Objects --- i.e., functions, loops, and basic blocks --- are specified
   * as a comma-separated list of object specifiers.  An object specifier
   * has the following syntax:
   *
   *   ObjSpec ::= IDENT          (an object name, or loop/block no.)
   *             | wt = 0.nnnnn   (a weight threshold)
   *             | wt = :k        (a weight rank)
   *
   * Alternatively, "+" can be used to specify "all" -- i.e., all functions,
   * or all loops, or all basic blocks.
   */
  do {
    otoken = oscan();
    if (otoken == ALL) {   /* mark everything */
      MarkObjs(TRUE);
      return oscan();
    }
    if (otoken != IDENT) return otoken;
    id = strdup(&buf[0]);
    nexttok = oscan();
    switch (nexttok) {
    case COMMA:     /* id is an object name */
      MarkObjs( (Matched(f,nodetype)) );
      if (!found) fprintf(stderr, "*** object not found: %s\n", id);
      otoken = nexttok;
      continue;
    case EQ:        /* expect: wt = 0.nnn or wt = : k */
      if (EqString(id, "wt")) 
	wtselector = WEIGHT;
      else if (EqString(id, "ct")) 
	wtselector = COUNT;
      else
	cmdlineerror(id);

      otoken = oscan();
      if (is_float_str) {          /* wt = 0.nnn or ct = 0.nnn */
	owt = atof(&buf[0]);
	totwt = 0;
	ForallObjs({totwt += (wtselector == WEIGHT ? Weight(f) : Count(f));});
	threshold = owt*totwt;
	MarkObjs( ((wtselector == WEIGHT ? Weight(f) : Count(f)) >= threshold) );
	if (!found) {
	  warning("*** weight/count threshold too high: nothing found!\n");
	}
      }
      else if (otoken == MAX_K) {  /* wt = : k */
	/*
	 * We need only the max_k "heaviest" objects.  If max_k == 1, which
	 * should be common enough to special-case, we just run down the
	 * list looking for the heaviest.  Otherwise, since we expect
	 * max_k to be not too big, we use a "truncated" bubble sort: i.e.,
	 * one whose outer loop iterates for only max_k iterations, giving
	 * us a O(N * max_k) algorithm, where N is the length of the list.
	 */
	if (max_k == 1) { /* need only heaviest one */
	  maxf = gptr;
	  if (wtselector == WEIGHT) {
	    ForallObjs( {maxf = Weight(f) > Weight(maxf) ? f : maxf;} );
	    MarkObjs( (Weight(f) == Weight(maxf)) );
	  }
	  else {
	    ForallObjs( {maxf = Count(f) > Count(maxf) ? f : maxf;} );
	    MarkObjs( (Count(f) == Count(maxf)) );
	  }
	}
	else {  /* truncated bubble sort */
	  for (tmp0 = gptr; 
	       max_k > 0 && tmp0 && Next(tmp0); 
	       tmp0 = Next(tmp0)) {
	    for (tmp1 = Next(tmp0); tmp1; tmp1 = Next(tmp1)) {
	      if ((wtselector == WEIGHT && Weight(tmp1) > Weight(tmp0)) ||
		  (wtselector == COUNT && Count(tmp1) > Count(tmp0))) { /* swap */
		fptr = GFuncNode(tmp0);
		GFuncNode(tmp0) = GFuncNode(tmp1);
		GFuncNode(tmp1) = fptr;
		lptr = GLoopNode(tmp0);
		GLoopNode(tmp0) = GLoopNode(tmp1);
		GLoopNode(tmp1) = lptr;
		bptr = GBBlNode(tmp0);
		GBBlNode(tmp0) = GBBlNode(tmp1);
		GBBlNode(tmp1) = bptr;
		x = Weight(tmp0);
		Weight(tmp0) = Weight(tmp1);
		Weight(tmp1) = x;
		x = Count(tmp0);
		Count(tmp0) = Count(tmp1);
		Count(tmp1) = x;
	      } /* if */
	    } /* for -- inner */
	    Display(tmp0) = TRUE;
	    if (nodetype != FUNC) {
	      for (tmp = allfuncs; tmp; tmp = Next(tmp)) {
		if (GFuncNode(tmp) == GFuncNode(tmp0)) {
		  Display(tmp) = TRUE;
		  break;
		}
	      }
	    }
	    max_k--;
	  } /* for -- outer */
	} /* if..else */
      }
      else {
	cmdlineerror(id);
      }
      otoken = oscan();
      break;
    default: 
      /*
       * nexttok is a value that is not part of an object spec,
       * which means we're done.  We still have an identifier
       * that we need to process: once that's done, we return
       * nexttok to the caller.
       */
      MarkObjs( (Matched(f,nodetype)) );
      if (!found) {
	fprintf(stderr, "*** object not found: %s\n", id);
      }
      return nexttok;
    }
  } while (otoken == COMMA);
  return otoken;
}

/*
 * instrs_reqd() scans the command line arguments to see
 * if a -I option has been specified.  This is purely for
 * efficiency: if the user doesn't ask for instructions, we
 * don't need to process instructions, allocate storage for
 * them, etc.
 */
int instrs_reqd(int argc, char *argv[])
{
  int otoken, reqd;
  char ch;

  argcount = argc;
  k = 1;
  argvec = argv;
  bufptr = argv[1];

  reqd = FALSE;
  for (otoken = oscan(); otoken != DONE; otoken = oscan()) {
    if (otoken == INSTS || otoken == CPATH1 || otoken == SCALE) {
      reqd = TRUE;
      break;
    }
  }
  return reqd;
}

/*
 * preproc_opts preprocesses the command line arguments.  It looks for
 * input file and phase specifications.  We need to do this before starting
 * to read the input, which is why we do it separately: the other options
 * are processed later, after the input has been read in and rendered into 
 * internal form.
 */
void preproc_opts(int argc, char *argv[])
{
  int i, otoken;
  char ch;

  argcount = argc;
  k = 1;
  argvec = argv;
  bufptr = argv[1];

  for (otoken = oscan(); otoken != DONE; otoken = oscan()) {
    switch (otoken) {
    case IFILE:
      GetRestOfOpt;
      yyin = fopen(&buf[0], "r");
      if (!yyin) {
	fprintf(stderr, "could not open input file: %s\n", buf);
	exit(-1);
      }
      continue;
    
    case PHASE:
      GetRestOfOpt;

      for (i = 0; buf[i] != '\0'; i++) {
	buf[i] = tolower(buf[i]);  /* to avoid case problems */
      }
      phase_string = strdup(buf);
    }
    continue;
  }
}

/*
 * process the command line arguments to get at arguments pertaining
 * to call-graph construction and display.
 */
/*
 * process all options (except for -i infile) and act on them.
 */

void procopts(int argc, char *argv[])
{
  int i, otoken;
  char ch;
  funcptr fptr;
  gobjptr optr;  

  argcount = argc;
  k = 1;
  argvec = argv;
  bufptr = argv[1];

  otoken = oscan();
  while (otoken != DONE) {
    switch (otoken) {
    case CPATH1: 
      instrs_p = TRUE;   /* then fall through to CPATH0 case */
    case CPATH0:
      cpath = TRUE;
      otoken = oscan();
      break;
    case CGRAPH:
      cgraph = TRUE;
      otoken = oscan();
      while (otoken == IDENT && otoken != DONE) {
	otoken = oscan();
	if (otoken == COMMA) {
	  otoken = oscan();
	}
      }
      break;
    case CGSTOP:
      otoken = oscan();
      while (otoken == IDENT) {
	if (is_int_str) {
	  fptr = func_lookup(idval);
	  if (fptr) {
	    Visited(fptr) = 1;
	  }
	}
	else {
	  for (fptr = NULL, optr = allfuncs; optr; optr = Next(optr)) {
	    if (EqString(buf, FuncName(GFuncNode(optr)))) {
	      Visited(GFuncNode(optr)) = 1;
	    }
	  }
	}
	otoken = oscan();
	if (otoken == COMMA) {
	  otoken = oscan();
	}
      }
      break;
    case THRSHLD:
      cg_threshold = 0;
      otoken = oscan();
      if (otoken == IDENT && is_int_str) {
	cg_threshold = idval;
	otoken = oscan();
      }
      break;
    case INSTS: 
      instrs_p = TRUE;
      otoken = oscan();
      break;
    case IFILE:
      GetRestOfOpt;   /* read in the file name, but ignore it: it was
			 handled earlier by getifile() */
      otoken = oscan();
      continue;
    case OFILE:
      GetRestOfOpt;
      ofilename = strdup(&buf[0]);
      outfile = fopen(ofilename, "w");
      if (!outfile) {
	fprintf(stderr, "could not open output file: %s\n", buf);
	exit(-1);
      }
      otoken = oscan();
      continue;
    case OFUNC:
      func_p = TRUE;
      otoken = match_opt_grp(FUNC, allfuncs);
      break;
    case OLOOP:
      loop_p = TRUE;
      otoken = match_opt_grp(LOOP, allloops);
      break;
    case OBBLK:
      bbl_p = TRUE;
      otoken = match_opt_grp(BBLK, allbbls);
      break;
    case SLOOP:
      sloop_p = TRUE;
      instrs_p = TRUE;
      GetRestOfOpt;
      sloopfile = fopen(&buf[0], "w");
      if (!sloopfile) {
	fprintf(stderr, "could not open file: %s\n", buf);
	exit(-1);
      }
      otoken = oscan();
      continue;
    case DOT:
      dotfile_p = TRUE;
      GetRestOfOpt;
      dotfilename = strdup(buf);
      otoken = oscan();
      continue;
    case TILE:
      tile = TRUE;
      otoken = oscan();
      continue;
    case VERBOSE:
      is_verbose = TRUE;
      otoken = oscan();
      continue;
    case SCALE:
      do_scale = TRUE;
      otoken = oscan();
      continue;
    case PHASE:
      GetRestOfOpt;       /* read in string but ignore it: it was handled  */
                          /* earlier by getphases().      */
      otoken = oscan();
      continue;
    default:
      error("Unrecognized option");
    }
  }
}
