#include "scanner.h"
#include "sloop.h"

#define DefaultCount 0   /* default execution count */
#define DefaultWt    1   /* default cumulative weight */

#define IsWhiteSp(x)  (x == ' ' || x == '\t' || x == '\n')
#define GetToken currtok = yylex()
#define Match(x) {if (x==currtok) GetToken; else syntaxerror();}
#define GetStr(x,y) {if (y==currtok) {\
		    x = strdup(yytext);\
		    if (!x) error("Out of memory!");\
		    GetToken;\
		  }\
                  else {\
		    syntaxerror();\
		  }\
		 }
#define GetId(x)    GetStr(x,id)
#define GetInstr(x) GetStr(x,instr)
#define AddToGlobalList(a,b,c,m,n,x) {gobjptr tmp;\
				    tmp = (gobjptr)NewNode(tmp);\
				    if (!tmp) error("Out of memory!");\
				    Display(tmp) = FALSE;\
				    GFuncNode(tmp) = a;\
				    GLoopNode(tmp) = b;\
				    GBBlNode(tmp)  = c;\
				    Count(tmp)     = m;\
				    Weight(tmp)    = n;\
				    Next(tmp)      = x;\
				    x = tmp;}

extern long linenum, numval;
extern int need_instrs;
extern char *yytext;
extern gobjptr allfuncs, allloops, allbbls;

char *currphase = "-unspecified-";
long currtok;
bbllist bbl_table[HASH_TABLE_SZ];
funclist func_table[HASH_TABLE_SZ];

/*********************************************************************/
bblptr bbl_lookup(long bblno);
funcptr func_lookup(long fno);
funcptr Start(void);
funcptr Func(void);
objptr ObjSeq(funcptr currfunc, loopptr currloop);
bblptr Bbl(funcptr currfunc, loopptr currloop);
loopptr Loop(funcptr currfunc, loopptr currloop);
instrptr Instrs(void);
edgeptr Edges(void);
edgeptr EdgeSeq(void);
/*********************************************************************/

bblptr bbl_lookup(long bblno)
{
  long hashval;
  bbllist xbbl;
  bblptr from = NULL;

  hashval = bblno & HASH_MASK;
  for (xbbl = bbl_table[hashval]; xbbl; xbbl = Next(xbbl)) {
    if (bblno == Id(GBBlNode(xbbl))) {
      from = GBBlNode(xbbl);
      break;
    }
  }
  return from;  
}

funcptr func_lookup(long fno)
{
  long hashval;
  funclist xfun;
  funcptr from = NULL;

  hashval = fno & HASH_MASK;
  for (xfun = func_table[hashval]; xfun; xfun = Next(xfun)) {
    if (fno == Id(GFuncNode(xfun))) {
      from = GFuncNode(xfun);
      break;
    }
  }
  return from;  
}

/*
 * start symbol: Start
 * productions:
 *  Start -> Func Start | epsilon
 */
funcptr Start(void)
{
  funcptr rootfunc, currfunc, nextfunc;
  int i;

  for (i = 0; i < HASH_TABLE_SZ; i++) {
    bbl_table[i] = NULL;
    func_table[i] = NULL;
  }
  
  rootfunc = NULL;
  GetToken;

  while (currtok > 0) {
    if (currtok == function) {
      nextfunc = Func();
      Attach(rootfunc, currfunc, nextfunc);
    }
    else {
      syntaxerror();
    }
  }
  return rootfunc;
}

/*
 * Func -> function '{' id '}' '{' num '}' [ '@' ]
 *          '{' 
 *             [ callsites '{' { caller '{' id '{' num '}' ',' num ',' num ',' num '}' } '}' ]
 *                 checked '{' id '}' ObjSeq Edges 
 *          '}'
 */

funcptr Func(void)
{
  funcptr fptr;
  char *fname, *str;
  long fwt = 0, fct = 0;
  int hashval;
  bblptr firstnode;
  edgeptr eptr;
  objptr optr;
  callerptr cptr, currcptr, nextcptr;
  funclist flstptr;

  fptr = (funcptr)NewNode(fptr);
  if (!fptr) error("Out of Memory!");
  Match(function);
  Match('{');
  GetId(fname); FuncName(fptr) = fname;
  Match('}');

  Phase(fptr) = currphase;

  Match('{');
  Match(num); Id(fptr) = numval;
  hashval = numval & HASH_MASK;
  Match('}');

  if (currtok == '@') {
    Match('@');
    Relevant(fptr) = 1;
  }
  else {
    Relevant(fptr) = 0;
  }

  Match('{');
  if (currtok == callsites) {
    fct = 0;
    Match(callsites);
    Match('{');
    CallerList(fptr) = NULL;
    CalleeList(fptr) = NULL;
    cptr = currcptr = NULL;
    while (currtok == caller) {
      Match(caller);
      Match('{');
      nextcptr = (callerptr)NewNode(nextcptr);
      Next(nextcptr) = NULL;
      GetId(str); FuncName(nextcptr)    = str;    Match('{');
      Match(num); CallerId(nextcptr)    = numval; Match('}'); Match(',');
      Match(num); BblCallFrom(nextcptr) = numval; Match(',');
      Match(num); BblCallTo(nextcptr)   = numval; Match(',');
      Match(num); Count(nextcptr)       = numval;
      fct += Count(nextcptr);
      Attach(cptr, currcptr, nextcptr);
      Match('}');
    }
    CallerList(fptr) = cptr;
    Match('}');
  }
  Match(checked);
  Match('{');
  GetId(str);
  Checked(fptr) = EqString(str, "yes");
  Match('}');
  Contents(fptr) = ObjSeq(fptr, NULL);
  EdgeList(fptr) = Edges();
  Match('}');
  Display(fptr) = FALSE;
  Processed(fptr) = FALSE;
  Significant(fptr) = FALSE;
  for (optr = Contents(fptr); optr; optr = Next(optr)) {
    fwt += IsBBlNode(optr) ? Weight(BblNode(optr)) : Weight(LoopNode(optr));
  }
  Weight(fptr) = fwt;
  if (fct > 0) {
    Count(fptr) = fct;
  }
  else {
    if (IsBBlNode(Contents(fptr))) {
      firstnode = BblNode(Contents(fptr));
    }
    else {
      firstnode = Hdr(LoopNode(Contents(fptr)));
    }
    fct = Count(firstnode);
    for (eptr = InEdges(firstnode); eptr; eptr = Next(eptr)) {
      fct -= Count(eptr);
    }
    fct = fct < 0 ? 0 : fct;
    Count(fptr) = fct;
  }
  AddToGlobalList(fptr,NULL,NULL,fct,fwt,allfuncs);

  flstptr = (funclist)NewNode(flstptr);
  if (!flstptr) error("out of memory!");
  GFuncNode(flstptr) = fptr;
  Next(flstptr)      = func_table[hashval];
  func_table[hashval] = flstptr;

  return fptr;
}

/*
 * ObjSeq derives a sequence of Objs, where an Obj is either a basic 
 * block or a loop:
 * 
 * ObjSeq -> Bbl ObjSeq | Loop ObjSeq | epsilon
 */
objptr ObjSeq(currfunc, currloop)
funcptr currfunc;
loopptr currloop;
{
  objptr rootobj, currobj, nextobj;

  rootobj = NULL;
  while (TRUE) {
    nextobj = (objptr)NewNode(nextobj);
    if (!nextobj) error("Out of memory!");
    if (currtok == bbl) {
      NodeType(nextobj) = BBLK;
      BblNode(nextobj) = Bbl(currfunc, currloop);
      Attach(rootobj, currobj, nextobj);
    }
    else if (currtok == loop) {
      NodeType(nextobj) = LOOP;
      LoopNode(nextobj) = Loop(currfunc, currloop);
      Attach(rootobj, currobj, nextobj);
    }
    else break;
  }
  return rootobj;
}

/*
 * Bbl -> bbl '{' num '}' [ '*' ] [ '@' ] '{' Count Instrs '}'
 */
bblptr Bbl(currfunc, currloop)
funcptr currfunc;
loopptr currloop;
{
  bblptr thisbbl;
  bbllist xbbl;
  long ninsts, hashval;
  instrptr iptr;
  char *ch;

  Match(bbl);
  Match('{');
  thisbbl = (bblptr)NewNode(thisbbl);
  if (!thisbbl) error("Out of memory!");
  Match(num);  Id(thisbbl) = numval;
  Match('}');
  if (currtok == '*') {
    Match('*');
    Hot(thisbbl) = 1;
  }
  else {
    Hot(thisbbl) = 0;
  }

  if (currtok == '@') {
    Match('@');
    Relevant(thisbbl) = 1;
  }
  else {
    Relevant(thisbbl) = 0;
  }

  Match('{');
  Count(thisbbl) = DefaultCount;
  if (currtok == count) {
    Match(count);
    Match('{');
    Match(num);
    Count(thisbbl) = numval;
    Match('}');
  }
  iptr = Instrs();
  NumInstrs(thisbbl) = ninsts = NumInstrs(iptr);
  Insts(thisbbl) = Next(iptr);
  if (Insts(thisbbl)) {
    Addr(thisbbl) = Addr(Insts(thisbbl));  /* addr of first instr */
  }
  /*
   * if there's a subroutine call in this basic block, set BSR(thisbbl)
   * to point to the name/address of the called routine.  
   */
  BSR(thisbbl) = NULL;
  for (iptr = Insts(thisbbl); iptr; iptr = Next(iptr)) {
    /*
     * check to see if there's a call instruction -- if there is, it must be 
     * the last instruction.
     */
    if (Next(iptr)) {   /* not the last instruction in the block */
      continue;
    }
    if ( (ch = strstr(Instruction(iptr), "call ")) != NULL) {
      /*
       * ch starts out pointing to "call": skip this word to get to the call target. 
       */
      for ( ; !isspace(*ch) && *ch != '\0'; ch++) {
      }

      if (*ch != '\0') {
	BSR(thisbbl) = ch;
      }
    }
  }

  NumPreds(thisbbl) = NumSuccs(thisbbl) = 0;
  Weight(thisbbl) = ninsts*Count(thisbbl);
  GLoopNode(thisbbl) = currloop;
  GFuncNode(thisbbl) = currfunc;
  Addr(thisbbl) = 0;
  OutEdges(thisbbl) = InEdges(thisbbl) = NULL;
  Match('}');
  AddToGlobalList(currfunc,currloop,thisbbl,Count(thisbbl),Weight(thisbbl),allbbls);
  xbbl = (bbllist)NewNode(xbbl);
  hashval = Id(thisbbl) & HASH_MASK;     /* extract low bits */
  GBBlNode(xbbl) = thisbbl;
  Next(xbbl) = bbl_table[hashval];
  bbl_table[hashval] = xbbl;
  return thisbbl;
}

/*
 * Loop -> loop '{' num '}' '{' loophdr '{' Bbl '}' ObjSeq Edges '}'
 */
loopptr Loop(currfunc, currloop)
funcptr currfunc;
loopptr currloop;
{
  loopptr thisloop;
  bblptr currhdr, newbbl;
  long lwt = 0;
  objptr optr;
  
  Match(loop);
  Match('{');
  thisloop = (loopptr)NewNode(thisloop);
  if (!thisloop) error("Out of memory!");
  Match(num); Id(thisloop) = numval;
  Match('}');
  Match('{');
  Match(loophdr);
  Match('{');
  currhdr = Bbl(currfunc, thisloop);
  Hdr(thisloop) = currhdr;
  Count(thisloop) = Count(Hdr(thisloop));
  Match('}');
  Contents(thisloop) = ObjSeq(currfunc, thisloop);
  EdgeList(thisloop) = Edges();
  /*
   * the loop header is a basic block that isn't listed separetely:
   * add it to the list of loop-body objects.
   */
  optr = (objptr)NewNode(optr);
  NodeType(optr) = BBLK;
  BblNode(optr) = currhdr;
  Next(optr) = Contents(thisloop); 
  Contents(thisloop) = optr;
  for (optr = Contents(thisloop); optr; optr = Next(optr)) {
    lwt += IsBBlNode(optr) ? Weight(BblNode(optr)) : Weight(LoopNode(optr));
  }
  Weight(thisloop) = lwt;
  Match('}');
  GFuncNode(thisloop) = currfunc;
  EnclLoop(thisloop) = currloop;   /* loop enclosing this loop */
  AddToGlobalList(currfunc,thisloop,NULL,Count(thisloop),lwt,allloops);
  return thisloop;
}

/*
 * Instrs -> instr Instrs | epsilon
 */
instrptr Instrs( void )
{
  char *iptr, *istart, *x, *y, addrbuf[16];
  int i, ninsts;
  long iaddr;
  instrptr rootinstr, currinstr, nextinstr;

  ninsts = 0;
  rootinstr = NULL;
  while (TRUE) {
    if (currtok == instr) {
      GetInstr(iptr);
      ninsts += 1;
      if (need_instrs || strstr(iptr, "call ")) {
	iptr += 3; /* "skip leading "%I{" */
	x = strstr(iptr, "}%I");
	if (!x) {
	  fprintf(stderr,
		  "Error: line %d: malformed instruction: %%I{%s\n", 
		  linenum,
		  iptr);
	  exit(-1);
	}
	/*
	 * work backwards to skip trailing "[ ... ]" -- then add three blanks
	 * so things print out nicely.  We know there must be space for these
	 * blanks because, at the very least, there's the trailing "%}I" at the
	 * end of the string.
	 */
	for (y = x; *y != '[' && y != iptr; y--)
	  ;
	if (*y == '[') {
	  *y = '\0';
	}
	else {
	  *x = '\0'; /* skip trailing "}%I" */
	}

	nextinstr = (instrptr)NewNode(nextinstr);
	if (!nextinstr) error("Out of memory!");
	Attach(rootinstr, currinstr, nextinstr);
	/* 
	 * now extract the address
	 */
	x = strstr(iptr, " 0x");
	if (x == NULL) {  /* for some reason we didn't find an address */
	  istart = iptr;
	  iaddr = 0;
	}
	else {
	  /* skip the address and any following whitespace */
	  for (istart = ++x; !isspace(*istart); istart++) 
	    ;
	  for ( ; isspace(*istart); istart++) 
	    ;
	  /* copy the address into addrbuf */
	  for (i = 0; !isspace(*x); x++) {
	    addrbuf[i++] = *x;
	  }
	  addrbuf[i] = '\0';
	  iaddr = strtol(addrbuf, (char **)NULL, 16);
	}
	Addr(nextinstr) = iaddr;
	NumInstrs(nextinstr) = ninsts;
	Instruction(nextinstr) = istart;
      }
    }
    else break;
  }
  nextinstr = (instrptr)NewNode(nextinstr);  /* allocate and fill in header */
  NumInstrs(nextinstr) = ninsts;
  Next(nextinstr) = rootinstr;
  rootinstr = nextinstr;

  return rootinstr;
}

/*
 * Edges -> edges '{' EdgeSeq '}' | epsilon
 */
edgeptr Edges(void)
{
  edgeptr edgeseq;

  if (currtok == edges) {
    Match(edges);
    Match('{');
    edgeseq = EdgeSeq();
    Match('}');
    return edgeseq;
  }
  else return NULL;
}

/*
 * EdgeSeq -> '{' num arrow num [':' num] '}' EdgeSeq | epsilon
 */
edgeptr EdgeSeq(void)
{
  edgeptr rootedge, curredge, nextedge;

  rootedge = NULL;
  while (TRUE) {
    if (currtok == '{') {
      nextedge = (edgeptr)NewNode(nextedge);
      if (!nextedge) error("Out of memory!");
      Match('{');
      Match(num); FromId(nextedge) = numval;
      Match(arrow);
      Match(num); ToId(nextedge) = numval;
      if (currtok == ':') {
	Match(':');
	Match(num);
	Count(nextedge) = numval;
      }
      else {
	Count(nextedge) = -1;
      }
      EdgeInfo(nextedge) = NULL;
      Attach(rootedge, curredge, nextedge);
      Match('}');
    }
    else break;
  }
  return rootedge;
}

