/* print out the dot specification */

#include <stdio.h>
#include "sloop.h"
#include "opts.h"

extern int oscan();
extern int argcount;
extern char **argvec;
extern char *bufptr, buf[128];
extern int max_k, k, idval;
extern int is_int_str, is_float_str;
extern int cg_threshold;
extern double owt, fidval;

extern FILE *dotfile, *sloopfile;
extern int instrs_p, tile, cpath, cgraph, do_scale;
extern long totedges, totunknown;
extern bblptr bbl_lookup(long bblno);
funcptr func_lookup(long fno);
extern int get_entry_pt(edgeptr epts, funcptr func);
extern gobjptr allfuncs;

#define MAX_ENTRY_PTS  5

#define INLOOP    0
#define NOTINLOOP 1
#define SHOWALL   2

#define DIGRAPH  0
#define SUBGRAPH 1
#define Tab(f, n)  {int i; for (i=0;i<n;i++) fprintf(f, "    ");}

#define Bbl_is_Dummy(bptr)     (Count(bptr) == -99999)

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% DEFAULTS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/

#define NodeWidth /*4.5*/ 3.8
char *FONT = "Helvetica";
char *Padding = " ";

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/


/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
void pfunc_dot(funcptr f);
void pbblk_dot(bblptr bptr, int grdesc, int n);
void ploop_dot(loopptr lptr, int grdesc, int n);
void pedges_dot(edgeptr eptr, int n, int cond);
void pobjseq_dot(objptr optr, int n);
void pfunc(gobjptr fptr);
void pentry_pts(funcptr func);
void pedges(edgeptr eptr, int ndepth);
void pobjseq(objptr optr, int ndepth);
void ploop(loopptr lptr, int ndepth);
void pbbl(bblptr bptr, int ndepth);
void pinstrs(instrptr iptr, int ndepth);
void clear_visited_bits();
void sort_entry_points(callerptr epts);
void gencallgraph_name(char *fname);
void gencallgraph_num(int fid);
void gencallgraph(funcptr fptr);
void pcallgraph_dot(funcptr fptr);
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/

int numblks(objptr optr)
{
  int n;

  n = 0;
  for ( ; optr; optr = Next(optr)) {
    switch (NodeType(optr)) {
    case LOOP:
      n += numblks(Contents(LoopNode(optr)));
      break;
    case BBLK:
      n += 1;
      break;
    default:
      error("numblks: unrecognized node type!");
    }
  }
  return n;
}

void print_dot(gobjptr optr, int queue)
{
  gobjptr gobj;

  switch (queue) {
  case FUNC: 
    for (gobj = optr; gobj; gobj = Next(gobj)) {
      if (!Display(gobj)) continue;
      pfunc_dot(GFuncNode(gobj));
    }
    break;
  case LOOP:
    for (gobj = optr; gobj; gobj = Next(gobj)) {
      if (!Display(gobj)) continue;
      ploop_dot(GLoopNode(gobj), DIGRAPH, 1);
    }
    break;
  case BBLK:
    for (gobj = optr; gobj; gobj = Next(gobj)) {
      if (!Display(gobj)) continue;
      pbblk_dot(GBBlNode(gobj), DIGRAPH, 1);
    }
    break;
  default:
    error(stderr, "internal error in print_dot: unrecognized queue\n");
  }
}

void pfunc_dot(funcptr func)
{
  fprintf(dotfile, "digraph \"%s\" {   /* ct = %ld, wt = %ld */\n",
	  FuncName(func), Count(func), Weight(func));
  fprintf(dotfile, "    fontname = %s;\n", FONT);
  fprintf(dotfile, "    center = TRUE;\n");
  if (Checked(func)) {
    fprintf(dotfile, "    label = \"%s{%d} [ ct = %ld ]\\n[ wt = %ld ]\\nPhase = %s\";\n",
	    FuncName(func), Id(func), Count(func), Weight(func), Phase(func));
  }
  else {
    fprintf(dotfile, "    label = \"%s{%d} [ ct = %ld, wt = %ld ]\\nPhase = %s\\n\";  /* no dominator info available */\n",
	    FuncName(func), Id(func), Count(func), Weight(func), Phase(func));
  }
  if (tile) {
    fprintf(dotfile, "    ratio = auto;\n    page = \"7.5,10\";\n");
  }
  else {
    fprintf(dotfile, "    size = \"7.5,10\";\n");
    if (numblks(Contents(func)) > 20) {
      fprintf(dotfile, "    ratio = auto;\n");
    }
  }
  fprintf(dotfile, "    /* basic blocks and edges */\n");
  pobjseq_dot(Contents(func), 1);
  /*
   * for some reason, dot has a _really_ hard time dealing with graphs
   * that have no edges.  So if necessary we just make one up.
   */
  if (EdgeList(func)) {
    pedges_dot(EdgeList(func), 1, SHOWALL);
  }
  else {
    fprintf(dotfile, "    D_U_M_M_Y_0 [style=invis]\n");
    fprintf(dotfile, "    D_U_M_M_Y_1 [style=invis]\n");
    fprintf(dotfile, "    D_U_M_M_Y_0 -> D_U_M_M_Y_1 [style=invis]\n");
  }

  pentry_pts(func);

  fprintf(dotfile, "}\n");
}

void pentry_pts(funcptr func)
{
  callerptr epts;
  bblptr cbbl, csite;
  long fct;
  int i, n, ncallsites;

  ncallsites = 0;
  for (epts = CallerList(func); epts; epts = Next(epts)) {
    ncallsites++;
  }
  if (ncallsites <= 3) {
    fct = Count(func);
    for (n = 0, epts = CallerList(func); epts; n++, epts = Next(epts)) {
      if (Bbl_is_Dummy(bbl_lookup(BblCallTo(epts)))) 
	continue;

      if (BblCallFrom(epts) < 0) {
	fprintf(dotfile, "    \"%s\" [shape=plaintext, fontsize=10]\n",
		FuncName(epts));
	fprintf(dotfile, "    \"%s\" -> B%d [weight=100, label=\"%d\", fontsize=10]\n",
	      FuncName(epts), 
	      BblCallTo(epts),
	      Count(epts));
      }
      else {
	fprintf(dotfile, "    \"%s\\nB%d\" [shape=plaintext, fontsize=10]\n",
		FuncName(epts),
		BblCallFrom(epts));
	fprintf(dotfile, "    \"%s\\nB%d\" -> B%d [weight=100, label=\"%d\", fontsize=10]\n",
		FuncName(epts),
		BblCallFrom(epts),
		BblCallTo(epts),
		Count(epts));
      }
    }
  }
  else {
    sort_entry_points(CallerList(func));
    fprintf(dotfile, "    C_A_L_L_E_R_S [shape=plaintext, fontname=Helvetica, label=\"");
    n = MAX_ENTRY_PTS;
    for (epts = CallerList(func); n > 0 && epts; epts = Next(epts)) {
      if (Bbl_is_Dummy(bbl_lookup(BblCallTo(epts))))
	continue;
      if (BblCallFrom(epts) < 0) {
	fprintf(dotfile, "%s -> B%d [ct=%ld]\\l",
		FuncName(epts), BblCallTo(epts), Count(epts));
      }
      else {
	fprintf(dotfile, "%s (B%d) -> B%d [ct=%ld]\\l",
		FuncName(epts), BblCallFrom(epts), BblCallTo(epts), Count(epts));
      }
      n--;
    }
    if (epts) {
      for (i = 0; epts; epts = Next(epts)) {
	i++;
      }
      fprintf(dotfile, "... %d more ...", i);
    }
    fprintf(dotfile, "\"]\n");
  }
}

void ploop_dot(loopptr lptr, int grdesc, int n)
{
  objptr optr;
  edgeptr eptr;
  int n1 = n+1;

  if (grdesc == DIGRAPH) {
    fprintf(dotfile, "digraph L%d {\n", Id(lptr));
    Tab(dotfile, n); fprintf(dotfile, "fontname = %s;\n", FONT);
    Tab(dotfile, n); fprintf(dotfile, "center = TRUE;\n");
    Tab(dotfile, n); fprintf(dotfile, "label = \"function: %s{%d}\\lcount = %ld\\l weight = %ld\";\n",
			     FuncName(GFuncNode(lptr)), 
			     Id(GFuncNode(lptr)),
			     Count(lptr), Weight(lptr));
    if (tile) {
      Tab(dotfile, n); fprintf(dotfile, "ratio = auto;\n");
      Tab(dotfile, n); fprintf(dotfile, "page = \"7.5,10\";\n");
    }
    else {
      Tab(dotfile, n); fprintf(dotfile, "size = \"7.5,10\";\n");
      if (numblks(Contents(lptr)) > 20) {
	Tab(dotfile, n); fprintf(dotfile, "ratio = auto;\n");
      }
    }
  }
  Tab(dotfile, n); fprintf(dotfile, "subgraph cluster_L%d {  /* wt = %ld */\n",
		  Id(lptr), Weight(lptr));
  Tab(dotfile,n1); fprintf(dotfile, "{ rank = min; B%d; }\n", Id(lptr));
  Tab(dotfile,n1); fprintf(dotfile, "  style=dashed;\n");
  Tab(dotfile,n1); fprintf(dotfile, "/* basic blocks and edges */\n");
  Tab(dotfile,n1); fprintf(dotfile, "label = \"L%d\";\n", Id(lptr));
  pobjseq_dot(Contents(lptr), n1);
  /*
   * print those edges in the loop where both endpoints are within the loop.
   */
  pedges_dot(EdgeList(lptr), n1, INLOOP);
  Tab(dotfile, n); fprintf(dotfile, "}\n");
  /*
   * print those edges in the loop where one endpoint is outside the loop.
   */
  pedges_dot(EdgeList(lptr), n, NOTINLOOP);

  if (grdesc == DIGRAPH) {
    Tab(dotfile, n); fprintf(dotfile, "L%d_ENTRY [style=invis]\n", Id(lptr));
    Tab(dotfile, n); fprintf(dotfile, "L%d_ENTRY -> B%d [weight=100]\n", Id(lptr), Id(lptr));
    fprintf(dotfile, "}\n");
  }
}
  
void pbblk_dot(bblptr bptr, int grdesc, int n)
{
  instrptr iptr;
  char *s;

  if (grdesc != DIGRAPH && cpath && !Critical(bptr)) {
    return;
  }

  if (Bbl_is_Dummy(bptr)) { 
    fprintf(dotfile, "    B%d [style=invis]\n", Id(bptr));
  }
  else {
    if (grdesc == DIGRAPH) {
      fprintf(dotfile, "digraph B_%d {\n", Id(bptr));
      Tab(dotfile, n); fprintf(dotfile, "fontname = %s;\n", FONT);
      Tab(dotfile, n); fprintf(dotfile, "center = TRUE;\n");
      Tab(dotfile, n); fprintf(dotfile, "label = B%d;\n", Id(bptr));
      if (tile) {
	Tab(dotfile, n); fprintf(dotfile, "ratio = auto;\n");
	Tab(dotfile, n); fprintf(dotfile, "page = \"7.5,10\";\n");
      }
      else {
	Tab(dotfile, n); fprintf(dotfile, "size = \"7.5,10\";\n\n");
      }
    }
    s = Hot(bptr) ? ", color=peachpuff, style=filled" : "";
    if (instrs_p || do_scale) {
      Tab(dotfile, n);
      fprintf(dotfile,
	      "B%d [shape=box%s, width=%.2f, fontname=%s, label=\"B%d  [ct = %d]                      \\l",
	      Id(bptr),
	      s,
	      NodeWidth,
	      FONT,
	      Id(bptr),
	      Count(bptr));
      for (iptr = Insts(bptr); iptr; iptr = Next(iptr)) {
	if (instrs_p) {
	  fprintf(dotfile, " %s%s\\l", Instruction(iptr), Padding);
	}
	else {
	  fprintf(dotfile, "  \\l");
	}
      }
      fprintf(dotfile, "\"]\n");
    }
    else {
      Tab(dotfile, n);
      fprintf(dotfile, "B%d [shape=box%s];\n", Id(bptr), s);
    }
    if (BSR(bptr)) {
      Tab(dotfile, n);
      fprintf(dotfile, "B%d_call [shape=plaintext,label=\"%s\"]\n", Id(bptr), BSR(bptr));
      Tab(dotfile, n);
      fprintf(dotfile, "B%d -> B%d_call [style=bold,weight=100,color=red]\n",
	      Id(bptr), Id(bptr));
    }
    if (grdesc == DIGRAPH) {
      fprintf(dotfile, "}\n");
    }
  }
}

void pobjseq_dot(objptr optr, int n)
{
  for ( ; optr; optr = Next(optr)) {
    switch (NodeType(optr)) {
    case BBLK : 
      pbblk_dot(BblNode(optr), SUBGRAPH, n);
      break;
    case LOOP:
      if (cpath && !Critical(Hdr(LoopNode(optr)))) {
	continue;
      }
      ploop_dot(LoopNode(optr), SUBGRAPH, n);
      break;
    default:
      error("*** internal error: unknown node type in pobjseq_dot()");
    }
  }
}

void pedges_dot(edgeptr eptr, int n, int cond)
{
  int show, wt;
  bblptr bptr;
  char *style;

  for ( ; eptr; eptr = Next(eptr)) {
    if (cpath && !IsCriticalEdge(eptr)) {
      if (Critical(From(eptr)) || Critical(To(eptr))) {
	/* 
	 * It turns out that every function has a dummy exit node, which has
	 * no instructions and is never executed --- all returns are directed
	 * to this node.  If this node is on the critical path, we can get a
	 * lot of spurious edges in the graph just because this dummy node is
	 * adjacent to every return block.  So at this point we check to see
	 * whether we're looking at this dummy node --- if we are, there's no
	 * need to print the edge.
	 */
	if (!Insts(To(eptr))) {
	  continue;
	}
	/*
	 * one of the end-points of the edge is critical, but the edge itself is
	 * not on the critical path.  In this case, the end point that is not
	 * critical should be in plaintext.
	 */
	bptr = NULL;
	if (!Critical(From(eptr))) {
	  bptr = From(eptr);
	}
	else if (!Critical(To(eptr))) {
	  bptr = To(eptr);
	}
	if (bptr) {
	  Tab(dotfile, n);
	  fprintf(dotfile, "B%d [shape=plaintext];\n", Id(bptr));
	}
	Tab(dotfile, n);
	fprintf(dotfile, "B%d -> B%d [fontsize=10, style=dotted, label=\"%.3f\"]  /* ct=%d */\n", 
		Id(From(eptr)), Id(To(eptr)), Prob(eptr), ECount(eptr));
      }
      continue;
    }
    switch (cond) {
    case INLOOP: 
      show = GLoopNode(From(eptr)) == GLoopNode(To(eptr)); 
      break;
    case NOTINLOOP:
      show = GLoopNode(From(eptr)) != GLoopNode(To(eptr)); 
      break;
    case SHOWALL:
      show = TRUE;
      break;
    default:
      error(stderr, "internal error: unknown cond in pedges_dot()\n");
    }
    if (Valid(eptr) && show) {
      wt = 1;
      Tab(dotfile, n);
/*
      if (Prob(eptr) > 0.5) {
*/
      if (EinfoLink(eptr)) {
	style = "dashed";
	wt = 120;
      }
      else if (IsCriticalEdge(eptr)) {
	style = "bold, color=green1";
	wt = GLoopNode(From(eptr)) == GLoopNode(To(eptr)) ? 100 : 10;
      }
      else {
	style = "solid";
	wt = 5;
      }
      if ( Bbl_is_Dummy(From(eptr)) ) {  /* dummy entry node */
	if (CallerList( GFuncNode(From(eptr)) ) == NULL) {
	  fprintf(dotfile, "B%d -> B%d [style=%s, weight=%d, fontsize=10]\n",
		  Id(From(eptr)), Id(To(eptr)), style, wt);
	}
      }
      else {
	fprintf(dotfile, "B%d -> B%d [style=%s, weight=%d, fontsize=10, label=\"%ld\"]\n",
		Id(From(eptr)), Id(To(eptr)), style, wt, ECount(eptr));
      }
    }
  }
}

void gen_callgraph_by_name(char *s)
{
  gobjptr optr;
  short found = 0;

  if (!s) {
    error("illegal argument (NULL) to gen_callgraph_by_name()");
  }

  fprintf(dotfile, "digraph CG_%s {\n", s);
  fprintf(dotfile, "    fontname = %s;\n", FONT);
  fprintf(dotfile, "    center = TRUE;\n");
  fprintf(dotfile, "    size = \"7.5,10\";\n");
  fprintf(dotfile, "    ratio=auto;\n");

  for (optr = allfuncs; optr; optr = Next(optr)) {
    if (EqString(s, FuncName(GFuncNode(optr)))) {
      found = 1;
      pcallgraph_dot(GFuncNode(optr));
    }
  }
  if (!found) {
    fprintf(stderr, "Warning: can't find function %s\n", s);
    /* the following two lines are needed to prevent dot from aborting */
    fprintf(dotfile, "    DUMMY [shape=plaintext, label=\"%s: NOT FOUND\"]\n", s);
    fprintf(dotfile, "    DUMMY -> DUMMY [style=invis]\n");  }

  fprintf(dotfile, "}\n");
}

void gen_callgraph_by_id(int fid)
{
  funcptr fptr;

  if (fid < 0) {
    error( ("illegal argument (%d) to gen_callgraph_by_id()", fid) );
  }

  fprintf(dotfile, "digraph CG_%d {\n", fid);
  fprintf(dotfile, "    fontname = %s;\n", FONT);
  fprintf(dotfile, "    center = TRUE;\n");
  fprintf(dotfile, "    size = \"7.5,10\";\n");
  fprintf(dotfile, "    ratio=auto;\n");

  fptr = func_lookup(fid);
  if (!fptr) {
    fprintf(stderr, "Warning: can't find function with id = %d\n", fid);
    /* the following two lines are needed to prevent dot from aborting */
    fprintf(dotfile, "    DUMMY [shape=plaintext, label=\"%d: NOT FOUND\"]\n", fid);
    fprintf(dotfile, "    DUMMY -> DUMMY [style=invis]\n");
  }
  else {
    pcallgraph_dot(fptr);
  }

  fprintf(dotfile, "}\n");
}

/*
 * HORRIBLE HACK ALERT: We assume that a function is a pseudo-function
 * if its name begins and ends with '-'.
 */
int fun_is_pseudo(funcptr fptr)
{
  char *c = FuncName(fptr);

  if (*c != '-') return 0;
  while (*c != '\0')  c++;
  if (*--c != '-') return 0;

  return 1;
}

void pcallgraph_dot(funcptr fptr)
{
  funclist flist, currf, ftmp;
  funcptr callerf, calleef;
  cgptr cptr, c0;
  short invis_edges;
  char *shape, *plural, *color;
  int dummyctr = 0;

  if (Visited(fptr)) return;

  flist = (funclist)NewNode(flist);
  if (!flist) error("out of memory!");
  Visited(fptr) = 1;
  GFuncNode(flist) = fptr;
  Next(flist) = NULL;

  while (flist) {
    currf = flist;
    flist = Next(flist);
    callerf = GFuncNode(currf);
    invis_edges = 0;
    if ( !CalleeList(callerf) ) {
      continue;
    }

    for (cptr = CalleeList(callerf); cptr; cptr = Next(cptr)) {
      calleef = GFuncNode(Callee(cptr));
      if (!callerf || !calleef) {
	error("NULL caller and/or callee pointer in pcallgraph!");
      }

      if (fun_is_pseudo(calleef)) {
	shape = "shape=plaintext, ";
	continue;
      }
      else {
	shape = "";
      }

      if (!Visited(calleef)) {
	Visited(calleef) = 1;
	ftmp = (funclist)NewNode(ftmp);
	if (!ftmp) error("out of memory!");
	GFuncNode(ftmp) = calleef;
	Next(ftmp) = flist;
	flist = ftmp;
      }
      
      if (!Significant(callerf)) {
	continue;
      }

      color = Relevant(callerf) ? ",color=\"red\",style=filled" : "";
    
      fprintf(dotfile,
	      "  \"%s{%d}\" [shape=box%s]\n", FuncName(callerf), Id(callerf), color);

      if (Count(cptr) <= cg_threshold) {
	if (invis_edges == 0) {
	  for (c0 = CalleeList(GFuncNode(currf)); c0; c0 = Next(c0)) {
	    if (Count(c0) <= cg_threshold) {
	      invis_edges++;
	    }
	  }
	  plural = invis_edges > 1 ? "s" : "";
	    
	  fprintf(dotfile,
		  "    \"$Dummy_%s{%d}\" [shape=plaintext,fontsize=10,label=\"%d edge%s\\n(ct < %d)\"]\n",
		  FuncName(callerf), Id(callerf),
		  invis_edges, plural, cg_threshold+1);
	  fprintf(dotfile, "    \"%s{%d}\" -> \"$Dummy_%s{%d}\" [style=dotted]\n",
		  FuncName(callerf), Id(callerf),
		  FuncName(callerf), Id(callerf));
	}
	continue;
      }
      if (fun_is_pseudo(calleef)) {
	fprintf(dotfile, "   DUMMY_%d [shape=plaintext, label = \"%s\"]\n",
		dummyctr,
		FuncName(calleef));
	fprintf(dotfile, "   \"%s{%d}\" -> \"DUMMY_%d\" [label=\"B%d\\l%ld\"]\n",
		FuncName(callerf),
		Id(callerf),
		dummyctr++,
		Id(Caller(cptr)),
		Count(cptr));
      }
      else {
	fprintf(dotfile, "   \"%s{%d}\" -> \"%s{%d}\" [label=\"B%d\\l%ld\"]\n",
		FuncName(callerf),
		Id(callerf),
		FuncName(calleef),
		Id(calleef),
		Id(Caller(cptr)),
		Count(cptr));
      }
    }
  }  
}

void pcgraph(int argc, char *argv[])
{
  int i, otoken;
  char ch;

  argcount = argc;
  k = 1;
  argvec = argv;
  bufptr = argv[1];

  mk_callgraph();

  otoken = oscan();
  while (otoken != DONE) {
    if (otoken == CGRAPH) {
      otoken = oscan();
      if (otoken != IDENT) {
	gen_callgraph_by_name("__start");
      }
      else {
	while (otoken == IDENT) {
	  if (is_int_str) {
	    gen_callgraph_by_id(idval);
	  }
	  else {
	    gen_callgraph_by_name(buf);
	  }
	  otoken = oscan();
	  if (otoken == COMMA) {
	    otoken = oscan();
	  }
	}
	clear_visited_bits();
      }
    }
    else {
      otoken = oscan();
    }
  }
}

void clear_visited_bits()
{
  gobjptr optr;

  for (optr = allfuncs; optr; optr = Next(optr)) {
    Visited(GFuncNode(optr)) = 0;
  }
}

void sort_entry_points(callerptr epts)
{
  long tmpcnt;
  callerptr tmp0, tmp1, tmp;
  struct callernode cnode;

  tmp = &cnode;

  for (tmp0 = epts; tmp0 && Next(tmp0); tmp0 = Next(tmp0)) {
    for (tmp1 = Next(tmp0); tmp1; tmp1 = Next(tmp1)) {
      if (Count(tmp1) > Count(tmp0)) {
	FuncName(tmp)     = FuncName(tmp0);
	CallerId(tmp)     = CallerId(tmp0);
	BblCallFrom(tmp)  = BblCallFrom(tmp0);
	BblCallTo(tmp)    = BblCallTo(tmp0);
	Count(tmp)        = Count(tmp0);

	FuncName(tmp0)    = FuncName(tmp1);
	CallerId(tmp0)    = CallerId(tmp1);
	BblCallFrom(tmp0) = BblCallFrom(tmp1);
	BblCallTo(tmp0)   = BblCallTo(tmp1);
	Count(tmp0)       = Count(tmp1);

	FuncName(tmp1)    = FuncName(tmp);
	CallerId(tmp1)    = CallerId(tmp);
	BblCallFrom(tmp1) = BblCallFrom(tmp);
	BblCallTo(tmp1)   = BblCallTo(tmp);
	Count(tmp1)       = Count(tmp);
      }
    }
  }
}

/***********************************************************************
*                                                                      *
*                        source output routines                        *
*                                                                      *
***********************************************************************/

/*
 * print out a list of function nodes
 */
void pfunc(gobjptr f)
{
  funcptr fptr;

  fprintf(sloopfile, "%%StartSloop\n");
  for ( ; f ; f = Next(f)) {
    if (Display(f)) {
      fptr = GFuncNode(f);
      fprintf(sloopfile, "%%FUNCTION{%s}{%d}{\n", FuncName(fptr), Id(fptr));
      fprintf(sloopfile, "    %%CHECKED{%s}\n", Checked(fptr) ? "yes" : "no");
      pobjseq(Contents(fptr), 1);
      if (EdgeList(fptr) != NULL) {
	pedges(EdgeList(fptr), 1);
      }
      fprintf(sloopfile, "} # End of function %s\n\n", FuncName(fptr));
    }
  }
  fprintf(sloopfile, "%%EndSloop\n");
}

void pedges(edgeptr eptr, int ndepth)
{
  Tab(sloopfile, ndepth);
  fprintf(sloopfile, "%%EDGES{");
  for ( ; eptr; eptr = Next(eptr)) {
    fprintf(sloopfile, "{%d -> %d} ", FromId(eptr), ToId(eptr));
  }
  fprintf(sloopfile, "}\n");
}

void pobjseq(objptr optr, int ndepth)
{
  for ( ; optr; optr = Next(optr)) {
    switch (NodeType(optr)) {
      case LOOP:
        ploop(LoopNode(optr), ndepth); 
	break;
      case BBLK: 
	pbbl(BblNode(optr), ndepth);
	break;
      default:
	fprintf(sloopfile, "*** Unrecognized object node type: %d\n", NodeType(optr));
      break;
    }
  }
}

void ploop(loopptr lptr, int ndepth)
{
  int ndepth1;

  ndepth1 = ndepth+1;
  Tab(sloopfile,ndepth);
  fprintf(sloopfile, "%%LOOP{%d}{  #  [ Count = %ld, Weight = %ld ]\n",
	  Id(lptr), Count(lptr), Weight(lptr));
  Tab(sloopfile,ndepth1);
  fprintf(sloopfile, "%%LOOPHEADER{\n");
  pbbl(Hdr(lptr), ndepth+2);
  Tab(sloopfile,ndepth1); fprintf(sloopfile, "}\n");
  /*
   * The first object in the "contents" list is the loop header --
   * since it's being printed elsewhere already, skip it.
   */
  assert(NodeType(Contents(lptr)) == BBLK
	 && Id(BblNode(Contents(lptr))) == Id(Hdr(lptr)));
  pobjseq(Next(Contents(lptr)), ndepth+1);
  pedges(EdgeList(lptr), ndepth+1);
  Tab(sloopfile,ndepth);
  fprintf(sloopfile, "} # End of loop\n");
}

void pbbl(bblptr bptr, int ndepth)
{
  int ndepth1;
  char *is_hot;

  Tab(sloopfile,ndepth);
  is_hot = Hot(bptr) ? "*" : "";
  ndepth1 = ndepth+1;
  fprintf(sloopfile, "%%BBL{%d}%s{  # [ Count = %ld, Weight = %ld ]\n",
	  Id(bptr), is_hot, Count(bptr), Weight(bptr));
  if (Count(bptr) > 0) {
    Tab(sloopfile,ndepth1);
    fprintf(sloopfile, "%%COUNT{%d}\n", Count(bptr));
  }
  pinstrs(Insts(bptr), ndepth1);
  Tab(sloopfile,ndepth);
  fprintf(sloopfile, "} # End of BBL %d\n", Id(bptr));
}

void pinstrs(instrptr iptr, int ndepth)
{
  for (; iptr; iptr = Next(iptr)) {
    Tab(sloopfile,ndepth);
    fprintf(sloopfile, "%%I{%s}%%I\n", Instruction(iptr));
  }
}

void pverbose()
{
  printf("EDGES: total = %ld, unknown = %ld\n", totedges, totunknown);
}
