#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <malloc.h>
#include <assert.h>

extern void *malloc();

#define FUNC 0
#define LOOP 1
#define BBLK 2

#define FALSE 0
#define TRUE  1

#define HASH_TABLE_SZ 1024
#define HASH_MASK     0x3ff
#define BUFSZ 128

#define Attach(root,curr,next) {if (!root) {\
			          root = next;\
			          curr = next;\
				}\
				else {\
				  Next(curr) = next;\
				  curr = next;\
				}\
				Next(next) = NULL;\
			       }

typedef struct instrnode {
  long addr;
  long ninstrs;       /* how many instructions from here on */
  char *instruction;
  struct instrnode *next;
} *instrptr;

typedef struct bblnode {
  long addr;
  int ident;
  char *bsr;
  long ninstrs, cnt, wt;
  int critical:1;
  int hot:1;
  int relevant:1;
  struct loopnode *lnode;
  struct funcnode *fnode;
  instrptr instrlist;
  struct edgenode *inedges, *outedges;
  int npreds, nsuccs;
  } *bblptr;

typedef struct bbllstnode {
  struct bblnode *bnode;
  struct bbllstnode *next;
} *bbllist;

typedef struct {
  bblptr from, to;
  long cnt;
  int critical:1;
  int link_edge:1;
  float prob;
} *edgeinfo;

typedef struct edgenode {
  long fromid, toid;
  long cnt;
  int valid;
  edgeinfo einfoptr;
  struct edgenode *next;
} *edgeptr;

typedef struct loopnode {
  int ident;
  long cnt, wt;
  bblptr hdr;
  struct funcnode *fnode;
  struct loopnode *encloop;
  edgeptr edge_ptr;
  struct objnode *contents;
} *loopptr;  

typedef struct callernode {
  char *fname;
  int callerid;
  int bblcallfrom;
  int bblcallto;
  long cnt;
  struct callernode *next;
} *callerptr;

typedef struct funcnode {
  char *fname;
  char *phasename;
  int display:1;
  int proc:1;
  int significant:1;
  int visited:1;
  int relevant:1;
  int ident;
  long cnt;
  long wt;
  int chk;
  edgeptr edge_ptr;
  struct objnode *contents;
  callerptr callerlist;
  struct cgnode *calleelist;
  struct funcnode *next;
} *funcptr;

typedef struct funclstnode {
  funcptr fnode;
  struct funclstnode *next;
} *funclist;

typedef struct objnode {
  int ntype;
  union {
    struct bblnode *bnode;
    struct loopnode *lnode;
  } val;
  struct objnode *next;
} *objptr;

typedef struct gobjnode {
  long wt, cnt;
  int display;
  struct funcnode *fnode;
  struct bblnode  *bnode;
  struct loopnode *lnode;
  struct gobjnode *next;
} *gobjptr;

typedef struct cgnode {
  bblptr caller_f, callee_f;
  long cnt;
  struct cgnode *next;
} *cgptr;

#define Addr(x)       (x->addr)
#define BblNode(x)    ((x->val).bnode)
#define BblNum(x)     (x->bblno)
#define BSR(x)        (x->bsr)
#define Callee(x)     (x->callee_f)
#define Caller(x)     (x->caller_f)
#define CalleeList(x) (x->calleelist)
#define CallerList(x) (x->callerlist)
#define CallerId(x)   (x->callerid)
#define BblCallFrom(x) (x->bblcallfrom)
#define BblCallTo(x)  (x->bblcallto)
#define Checked(x)    (x->chk)
#define Contents(x)   (x->contents)
#define Count(x)      (x->cnt)
#define Critical(x)   (x->critical)
#define EnclLoop(x)   (x->encloop)
#define IsCriticalEdge(x) ((x->einfoptr)->critical)
#define Weight(x)     (x->wt)
#define Display(x)    (x->display)
#define EdgeList(x)   (x->edge_ptr)
#define EdgeInfo(x)   (x->einfoptr)
#define ECount(x)     ((x->einfoptr)->cnt)
#define EFrom(x)      (x->from)
#define ELink(x)      (x->link_edge)
#define EinfoLink(x)  ((x->einfoptr)->link_edge)
#define EProb(x)      (x->prob)
#define ETo(x)        (x->to)
#define From(x)       ((x->einfoptr)->from)
#define FromId(x)     (x->fromid)
#define FuncName(x)   (x->fname)
#define Phase(x)      (x->phasename)
#define FuncNode(x)   ((x->val).fnode)
#define GBBlNode(x)   (x->bnode)
#define GFuncNode(x)  (x->fnode)
#define GLoopNode(x)  (x->lnode)
#define Hdr(x)        (x->hdr)
#define Hot(x)        (x->hot)
#define Relevant(x)   (x->relevant)
#define Id(x)         (x->ident)
#define InEdges(x)    (x->inedges)
#define Instruction(x) (x->instruction)
#define Insts(x)      (x->instrlist)
#define IsBBlNode(x)  (BBLK == NodeType(x))
#define IsFuncNode(x) (FUNC == NodeType(x))
#define IsLoopNode(x) (LOOP == NodeType(x))
#define LoopNode(x)   ((x->val).lnode)
#define NodeType(x)   (x->ntype)
#define NumInstrs(x)  (x->ninstrs)
#define NumPreds(x)   (x->npreds)
#define NumSuccs(x)   (x->nsuccs)
#define OutEdges(x)   (x->outedges)
#define Prob(x)       ((x->einfoptr)->prob)
#define Processed(x)  ((x->proc))
#define Significant(x) (x->significant)
#define To(x)         ((x->einfoptr)->to)
#define ToId(x)       (x->toid)
#define Valid(x)      (x->valid)
#define Visited(x)    (x->visited)

#define NewNode(x) malloc(sizeof(*x))

#define Next(x) (x->next)

#define EqString(x,y) (!strcmp(x, y))
