#include <stdbool.h>
#include <stdlib.h>
#include <string.h>

#include "util.h"



/*
 *  alloc(amount) -- allocate memory with error checking.
 */
void *alloc(int amount) {
    void *mem;

    if (amount < 0) {
	MALFUNCTION("Negative allocation amount");
    }
    mem = malloc(amount);
    if (mem == NULL) {
	FATAL("Out of memory");
    }
    return mem;
}



/*
 *  salloc(string) -- allocate memory and initialize with string value.
 */
void *salloc(char *string) {
    return strcpy(alloc(strlen(string) + 1), string);
}



/*
 *  zalloc(amount) -- allocate zeroed memory, with error checking.
 */
void *zalloc(int amount) {
    void *mem;

    if (amount < 0) {
	MALFUNCTION("Negative allocation amount");
    }
    mem = calloc(1, amount);
    if (mem == NULL) {
	FATAL("Out of memory");
    }
    return mem;
}



/*
 *  roundup(value,quantum) --
 *
 *  Rounds a number upward (if necessary) to the next multiple of n.
 */
int roundup(int value, int quantum) {
    int r = value % quantum;
    if (r == 0) {
	return value;
    } else {
	return value + quantum - r;
    }
}



/*
 *  memberOf(item,arr,size) -- search array for integer.
 *
 *  Returns true if the given integer is in the given array.
 */
bool memberOf(int item, int arr[], int size) {
    int i;

    for (i = 0; i < size; i++) {
	if (arr[i] == item) {
	    return true;
	}
    }
    return false;
}



/*
 *  loadLE(address) --
 *
 *  Interprets four consecutive bytes of memory as a little-endian integer.
 */
int loadLE(void *address) {
    unsigned char *a = address;
    unsigned int x = *a++;
    x |= *a++ << 8;
    x |= *a++ << 16;
    x |= *a++ << 24;
    return x;
}



/*
 *  storeLE(address,value) --
 *
 *  Stores a value in little-endian fashion in four bytes of memory.
 */
void storeLE(void *address, int value) {
    unsigned char *a = address;
    *a++ = value;
    *a++ = (value >>= 8);
    *a++ = (value >>= 8);
    *a++ = (value >>= 8);
    return;
}



/*
 *  storeLE2(address,value) --
 *
 *  Stores a 16-bit value in little-endian fashion in two bytes of memory.
 */
void storeLE2(void *address, int value) {
    unsigned char *a = address;
    *a++ = value;
    *a++ = value >> 8;
    return;
}
