/*
 *  x86/histogram.c -- for creating various histograms.
 */
#include "global.h"
#include "x86-histogram.h"
#include "x86-table-defs.h"
#include "x86-dis.h"
#include "utilities.h"



/*
 *  xGenerateStaticOpcodeHistogram() --
 *
 *  Gathers the information collected from the disassembler to generate an
 *  opcode histogram.
 */
void xGenerateStaticOpcodeHistogram() {
    char *histoFile;
    char buffer[1024];
    FILE *fp;
    int i, totalOpcodes, j, temp;
    int numonebyte, numtwobyte, numgrps, numfgrps, numfloatreg, numfloatmem;

    histoFile = MakeFilename("StaticHistogramOutFile", binFilename);
    fp = fopen(histoFile, "w");
    if (fp == 0) {
	PFATAL(histoFile);
    }

    REPORT(1, "Writing static opcode histogram to %s", histoFile);

    numonebyte = numtwobyte = numgrps = numfgrps = numfloatreg = numfloatmem =
	0;
    for (i = 0; i < 0xff; i++) {
	numonebyte = numonebyte + onebyteOpcodes[i];
	numtwobyte = numtwobyte + twobyteOpcodes[i];
	numgrps = numgrps + grpsOpcodes[i];
	numfgrps = numfgrps + fgrpsOpcodes[i];
	numfloatreg = numfloatreg + floatregOpcodes[i];
	numfloatmem = numfloatmem + floatmemOpcodes[i];
    }

    totalOpcodes = numonebyte + numtwobyte + numgrps + numfgrps + numfloatreg +
	numfloatmem;

    /*
     *                 Output the histogram for dis386[]
     */

    sprintf(buffer, "One-byte integer opcodes: [%f%%: %d] (dis386[])\n",
	(float) numonebyte * 100.0 / totalOpcodes, numonebyte);
    fputs(buffer, fp);
    memset(buffer, 0, 1024);

    for (i = temp = 0; i < 0xff; i++) {
	if (onebyteOpcodes[i] > 0) {
	    temp++;
	}
    }

    for (i = 0; i < 0xff; i++) {
	int highest, highestIndex;
	highest = highestIndex = -1;
	for (j = 0; j < 0xff; j++) {
	    if (onebyteOpcodes[j] > highest) {
		highest = onebyteOpcodes[j];
		highestIndex = j;
	    }
	}
	onebyteOpcodes[highestIndex] = -1;
	sprintf(buffer, "  [%f%% : %-9d] Opcode: 0x%2.2x (%s)\n",
	    (float) highest * 100.0 / totalOpcodes, highest, highestIndex,
	    IndexToOneByteOpcode(highestIndex).x_name);
	fputs(buffer, fp);
	memset(buffer, 0, 1024);
	if ((--temp) <= 0) {
	    break;
	}
    }

    /*
     *            Output the histogram for dis386_twobyte[]
     */

    sprintf(buffer,
	"\n\nTwo-byte integer opcodes: [%f%%: %d] (dis386_twobyte[])\n",
	(float) numtwobyte * 100.0 / totalOpcodes, numtwobyte);
    fputs(buffer, fp);
    memset(buffer, 0, 1024);

    for (i = temp = 0; i < 0xff; i++) {
	if (twobyteOpcodes[i] > 0) {
	    temp++;
	}
    }

    for (i = 0; i < 0xff; i++) {
	int highest, highestIndex;
	highest = highestIndex = -1;
	for (j = 0; j < 0xff; j++) {
	    if (twobyteOpcodes[j] > highest) {
		highest = twobyteOpcodes[j];
		highestIndex = j;
	    }
	}
	twobyteOpcodes[highestIndex] = -1;
	sprintf(buffer, "  [%f%% : %-9d] Opcode: 0x0F%2.2x (%s)\n",
	    (float) highest * 100.0 / totalOpcodes, highest, highestIndex,
	    IndexToTwoByteOpcode(highestIndex).x_name);
	fputs(buffer, fp);
	memset(buffer, 0, 1024);
	if ((--temp) <= 0) {
	    break;
	}
    }

    /*
     *                 Output the histogram for grps[]
     */

    sprintf(buffer,
	"\n\nGroup Placeholder integer opcodes: [%f%%: %d] (grps[])\n",
	(float) numgrps * 100.0 / totalOpcodes, numgrps);
    fputs(buffer, fp);
    memset(buffer, 0, 1024);

    for (i = temp = 0; i < 0xff; i++) {
	if (grpsOpcodes[i] > 0) {
	    temp++;
	}
    }

    for (i = 0; i < 0xff; i++) {
	int highest, highestIndex;
	highest = highestIndex = -1;
	for (j = 0; j < 0xff; j++) {
	    if (grpsOpcodes[j] > highest) {
		highest = grpsOpcodes[j];
		highestIndex = j;
	    }
	}
	grpsOpcodes[highestIndex] = -1;
	sprintf(buffer, "  [%f%% : %-9d] Opcode: 0x%2.2x (%s)\n",
	    (float) highest * 100.0 / totalOpcodes, highest, highestIndex,
	    IndexToGRPSOpcode(highestIndex).x_name);
	fputs(buffer, fp);
	memset(buffer, 0, 1024);
	if ((--temp) <= 0) {
	    break;
	}
    }

    /*
     *                 Output the histogram for fgrps[]
     */

    sprintf(buffer,
	"\n\nGroup Placeholder floating point opcodes: [%f%%: %d] (fgrps[])\n",
	(float) numfgrps * 100.0 / totalOpcodes, numfgrps);
    fputs(buffer, fp);
    memset(buffer, 0, 1024);

    for (i = temp = 0; i < 0xff; i++) {
	if (fgrpsOpcodes[i] > 0) {
	    temp++;
	}
    }

    for (i = 0; i < 0xff; i++) {
	int highest, highestIndex;
	highest = highestIndex = -1;
	for (j = 0; j < 0xff; j++) {
	    if (fgrpsOpcodes[j] > highest) {
		highest = fgrpsOpcodes[j];
		highestIndex = j;
	    }
	}
	fgrpsOpcodes[highestIndex] = -1;
	sprintf(buffer, "  [%f%% : %-9d] Opcode: 0x%2.2x (%s)\n",
	    (float) highest * 100.0 / totalOpcodes, highest, highestIndex,
	    IndexToFGRPSOpcode(highestIndex).x_name);
	fputs(buffer, fp);
	memset(buffer, 0, 1024);
	if ((--temp) <= 0) {
	    break;
	}
    }

    /*
     *                 Output the histogram for float_mem[]
     */

    sprintf(buffer,
	"\n\nFloating point memory opcodes: [%f%%: %d] (float_mem[])\n",
	(float) numfloatmem * 100.0 / totalOpcodes, numfloatmem);
    fputs(buffer, fp);
    memset(buffer, 0, 1024);

    for (i = temp = 0; i < 0xff; i++) {
	if (floatmemOpcodes[i] > 0) {
	    temp++;
	}
    }

    for (i = 0; i < 0xff; i++) {
	int highest, highestIndex;
	highest = highestIndex = -1;
	for (j = 0; j < 0xff; j++) {
	    if (floatmemOpcodes[j] > highest) {
		highest = floatmemOpcodes[j];
		highestIndex = j;
	    }
	}
	floatmemOpcodes[highestIndex] = -1;
	sprintf(buffer, "  [%f%% : %-9d] Opcode: 0x%2.2x (%s)\n",
	    (float) highest * 100.0 / totalOpcodes, highest, highestIndex,
	    IndexToFloatMemOpcode(highestIndex).x_name);
	fputs(buffer, fp);
	memset(buffer, 0, 1024);
	if ((--temp) <= 0) {
	    break;
	}
    }

    /*
     *                 Output the histogram for float_mem[]
     */

    sprintf(buffer,
	"\n\nFloat point register opcodes: [%f%%: %d] (float_reg[])\n",
	(float) numfloatreg * 100.0 / totalOpcodes, numfloatreg);
    fputs(buffer, fp);
    memset(buffer, 0, 1024);

    for (i = temp = 0; i < 0xff; i++) {
	if (floatregOpcodes[i] > 0) {
	    temp++;
	}
    }

    for (i = 0; i < 0xff; i++) {
	int highest, highestIndex;
	highest = highestIndex = -1;
	for (j = 0; j < 0xff; j++) {
	    if (floatregOpcodes[j] > highest) {
		highest = floatregOpcodes[j];
		highestIndex = j;
	    }
	}
	floatregOpcodes[highestIndex] = -1;
	sprintf(buffer, "  [%f%% : %-9d] Opcode: 0x%2.2x (%s)\n",
	    (float) highest * 100.0 / totalOpcodes, highest, highestIndex,
	    IndexToFloatRegOpcode(highestIndex).x_name);
	fputs(buffer, fp);
	memset(buffer, 0, 1024);
	if ((--temp) <= 0) {
	    break;
	}
    }

    fclose(fp);
}



/*
 *  xGenerateWeightedOpcodeHistogram() --
 *
 *  Gathers the information collected from the assembler to generate an
 *  opcode histogram.
 */
void xGenerateWeightedOpcodeHistogram() {
    char *histoFile;
    char buffer[1024];
    FILE *fp;
    int i, j, temp;
    long long totalOpcodes;
    long long numonebyte, numtwobyte, numgrps, numfgrps, numfloatreg,
	numfloatmem;
    long long highest;
    int highestIndex;

    histoFile = MakeFilename("WeightedHistogramOutFile", binFilename);
    fp = fopen(histoFile, "w");
    if (fp == 0) {
	PFATAL(histoFile);
    }

    REPORT(1, "Writing weighted opcode histogram to %s", histoFile);

    numonebyte = numtwobyte = numgrps = numfgrps = numfloatreg = numfloatmem =
	0;
    for (i = 0; i < 0xff; i++) {
	numonebyte = numonebyte + onebyteOpcodesWeighted[i];
	numtwobyte = numtwobyte + twobyteOpcodesWeighted[i];
	numgrps = numgrps + grpsOpcodesWeighted[i];
	numfgrps = numfgrps + fgrpsOpcodesWeighted[i];
	numfloatreg = numfloatreg + floatregOpcodesWeighted[i];
	numfloatmem = numfloatmem + floatmemOpcodesWeighted[i];
    }

    totalOpcodes = numonebyte + numtwobyte + numgrps + numfgrps + numfloatreg +
	numfloatmem;

    /*
     *                 Output the histogram for dis386[]
     */

    sprintf(buffer, "One-byte integer opcodes: [%f%%: %f] (dis386[])\n",
	(float) numonebyte * 100.0 / totalOpcodes, (float) numonebyte);
    fputs(buffer, fp);
    memset(buffer, 0, 1024);

    for (i = temp = 0; i < 0xff; i++) {
	if (onebyteOpcodesWeighted[i] > 0) {
	    temp++;
	}
    }

    for (i = 0; i < 0xff; i++) {
	highest = highestIndex = -1;
	for (j = 0; j < 0xff; j++) {
	    if (onebyteOpcodesWeighted[j] > highest) {
		highest = onebyteOpcodesWeighted[j];
		highestIndex = j;
	    }
	}
	onebyteOpcodesWeighted[highestIndex] = -1;
	sprintf(buffer, "  [%f%% : %-20f] Opcode: 0x%2.2x (%s)\n",
	    (float) highest * 100.0 / totalOpcodes, (float) highest,
	    highestIndex, IndexToOneByteOpcode(highestIndex).x_name);
	fputs(buffer, fp);
	memset(buffer, 0, 1024);
	if ((--temp) <= 0) {
	    break;
	}
    }

    /*
     *            Output the histogram for dis386_twobyte[]
     */

    sprintf(buffer,
	"\n\nTwo-byte integer opcodes: [%f%%: %f] (dis386_twobyte[])\n",
	(float) numtwobyte * 100.0 / totalOpcodes, (float) numtwobyte);
    fputs(buffer, fp);
    memset(buffer, 0, 1024);

    for (i = temp = 0; i < 0xff; i++) {
	if (twobyteOpcodesWeighted[i] > 0) {
	    temp++;
	}
    }

    for (i = 0; i < 0xff; i++) {
	highest = highestIndex = -1;
	for (j = 0; j < 0xff; j++) {
	    if (twobyteOpcodesWeighted[j] > highest) {
		highest = twobyteOpcodesWeighted[j];
		highestIndex = j;
	    }
	}
	twobyteOpcodesWeighted[highestIndex] = -1;
	sprintf(buffer, "  [%f%% : %-20f] Opcode: 0x0F%2.2x (%s)\n",
	    (float) highest * 100.0 / totalOpcodes, (float) highest,
	    highestIndex, IndexToTwoByteOpcode(highestIndex).x_name);
	fputs(buffer, fp);
	memset(buffer, 0, 1024);
	if ((--temp) <= 0) {
	    break;
	}
    }

    /*
     *                 Output the histogram for grps[]
     */

    sprintf(buffer,
	"\n\nGroup Placeholder integer opcodes: [%f%%: %f] (grps[])\n",
	(float) numgrps * 100.0 / totalOpcodes, (float) numgrps);
    fputs(buffer, fp);
    memset(buffer, 0, 1024);

    for (i = temp = 0; i < 0xff; i++) {
	if (grpsOpcodesWeighted[i] > 0) {
	    temp++;
	}
    }

    for (i = 0; i < 0xff; i++) {
	highest = highestIndex = -1;
	for (j = 0; j < 0xff; j++) {
	    if (grpsOpcodesWeighted[j] > highest) {
		highest = grpsOpcodesWeighted[j];
		highestIndex = j;
	    }
	}
	grpsOpcodesWeighted[highestIndex] = -1;
	sprintf(buffer, "  [%f%% : %-20f] Opcode: 0x%2.2x (%s)\n",
	    (float) highest * 100.0 / totalOpcodes, (float) highest,
	    highestIndex, IndexToGRPSOpcode(highestIndex).x_name);
	fputs(buffer, fp);
	memset(buffer, 0, 1024);
	if ((--temp) <= 0) {
	    break;
	}
    }

    /*
     *                 Output the histogram for fgrps[]
     */

    sprintf(buffer,
	"\n\nGroup Placeholder floating point opcodes: [%f%%: %f] (fgrps[])\n",
	(float) numfgrps * 100.0 / totalOpcodes, (float) numfgrps);
    fputs(buffer, fp);
    memset(buffer, 0, 1024);

    for (i = temp = 0; i < 0xff; i++) {
	if (fgrpsOpcodesWeighted[i] > 0) {
	    temp++;
	}
    }

    for (i = 0; i < 0xff; i++) {
	highest = highestIndex = -1;
	for (j = 0; j < 0xff; j++) {
	    if (fgrpsOpcodesWeighted[j] > highest) {
		highest = fgrpsOpcodesWeighted[j];
		highestIndex = j;
	    }
	}
	fgrpsOpcodesWeighted[highestIndex] = -1;
	sprintf(buffer, "  [%f%% : %-20f] Opcode: 0x%2.2x (%s)\n",
	    (float) highest * 100.0 / totalOpcodes, (float) highest,
	    highestIndex, IndexToFGRPSOpcode(highestIndex).x_name);
	fputs(buffer, fp);
	memset(buffer, 0, 1024);
	if ((--temp) <= 0) {
	    break;
	}
    }

    /*
     *                 Output the histogram for float_mem[]
     */

    sprintf(buffer,
	"\n\nFloating point memory opcodes: [%f%%: %f] (float_mem[])\n",
	(float) numfloatmem * 100.0 / totalOpcodes, (float) numfloatmem);
    fputs(buffer, fp);
    memset(buffer, 0, 1024);

    for (i = temp = 0; i < 0xff; i++) {
	if (floatmemOpcodesWeighted[i] > 0) {
	    temp++;
	}
    }

    for (i = 0; i < 0xff; i++) {
	highest = highestIndex = -1;
	for (j = 0; j < 0xff; j++) {
	    if (floatmemOpcodesWeighted[j] > highest) {
		highest = floatmemOpcodesWeighted[j];
		highestIndex = j;
	    }
	}
	floatmemOpcodesWeighted[highestIndex] = -1;
	sprintf(buffer, "  [%f%% : %-20f] Opcode: 0x%2.2x (%s)\n",
	    (float) highest * 100.0 / totalOpcodes, (float) highest,
	    highestIndex, IndexToFloatMemOpcode(highestIndex).x_name);
	fputs(buffer, fp);
	memset(buffer, 0, 1024);
	if ((--temp) <= 0) {
	    break;
	}
    }

    /*
     *                 Output the histogram for float_mem[]
     */

    sprintf(buffer,
	"\n\nFloat point register opcodes: [%f%%: %f] (float_reg[])\n",
	(float) numfloatreg * 100.0 / totalOpcodes, (float) numfloatreg);
    fputs(buffer, fp);
    memset(buffer, 0, 1024);

    for (i = temp = 0; i < 0xff; i++) {
	if (floatregOpcodesWeighted[i] > 0) {
	    temp++;
	}
    }

    for (i = 0; i < 0xff; i++) {
	highest = highestIndex = -1;
	for (j = 0; j < 0xff; j++) {
	    if (floatregOpcodesWeighted[j] > highest) {
		highest = floatregOpcodesWeighted[j];
		highestIndex = j;
	    }
	}
	floatregOpcodesWeighted[highestIndex] = -1;
	sprintf(buffer, "  [%f%% : %-20f] Opcode: 0x%2.2x (%s)\n",
	    (float) highest * 100.0 / totalOpcodes, (float) highest,
	    highestIndex, IndexToFloatRegOpcode(highestIndex).x_name);
	fputs(buffer, fp);
	memset(buffer, 0, 1024);
	if ((--temp) <= 0) {
	    break;
	}
    }

    fclose(fp);
}
