/*
 *  x86/table.c -- Definition of disassembly tables declared in x86-table.h.
 */
#include "global.h"
#include "utilities.h"
#include "x86-classes.h"
#include "x86-asm.h"
#include "x86-hash.h"
#include "x86-table.h"

/*
 *  See x86-table-defs.h for X86op struct, and also x86-table.h for macros.
 *  Table entries have the following form:
 *
 *  { type, class, mnemonic, uopcount,
 *	op1 & size, op2 & size, op3 & size, impldst1, idst2, isrc1, isrc2,
 *	nbytes_pushed, nbytes_popped,
 *	psw_changed, ps_wused }
 */

X86op dis386[] = {
    /* 00 */
    { TYPE_NONE, CLASS_ADD, "addb", 1,
        Eb, Gb, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        PSW_OF | PSW_SF | PSW_ZF | PSW_AF | PSW_CF | PSW_PF | DUF, 0 },
    { TYPE_NONE, CLASS_ADD, "addS", 1,
        Ev, Gv, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
	PSW_OF | PSW_SF | PSW_ZF | PSW_AF | PSW_CF | PSW_PF | DUF, 0 },
    { TYPE_NONE, CLASS_ADD, "addb", 1,
        Gb, Eb, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
	PSW_OF | PSW_SF | PSW_ZF | PSW_AF | PSW_CF | PSW_PF | DUF, 0 },
    { TYPE_NONE, CLASS_ADD, "addS", 1,
        Gv, Ev, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
	PSW_OF | PSW_SF | PSW_ZF | PSW_AF | PSW_CF | PSW_PF | DUF, 0 },
    { TYPE_NONE, CLASS_ADD, "addb", 1,
        AL, Ib, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
	PSW_OF | PSW_SF | PSW_ZF | PSW_AF | PSW_CF | PSW_PF | DUF, 0 },
    { TYPE_NONE, CLASS_ADD, "addS", 1,
        eAX, Iv, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
	PSW_OF | PSW_SF | PSW_ZF | PSW_AF | PSW_CF | PSW_PF | DUF, 0 },
    { TYPE_NONE, CLASS_PUSHES, "pushS", 4,
        NULL_OPERANDS_1, NULL_OPERANDS_2, eSP_reg, no_reg, eSP_reg, es_reg,
        ADDRESS_SIZE, 0,
        0, 0 },
    { TYPE_NONE, CLASS_POPES, "popS", COMPLEX,
        NULL_OPERANDS_1, NULL_OPERANDS_2, eSP_reg, es_reg, eSP_reg, no_reg,
        0, ADDRESS_SIZE,
        0, 0 },
      
    /* 08 */
    { TYPE_NONE, CLASS_OR, "orb", 1,
        Eb, Gb, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
	PSW_OF | PSW_CF | PSW_SF | PSW_ZF | PSW_PF | DUF, 0 },
    { TYPE_NONE, CLASS_OR, "orS", 1,
        Ev, Gv, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
	PSW_OF | PSW_CF | PSW_SF | PSW_ZF | PSW_PF | DUF, 0 },
    { TYPE_NONE, CLASS_OR, "orb", 1,
        Gb, Eb, NULL_OPERANDS_1, NO_REGISTERS_ALL, 0, 0,
	PSW_OF | PSW_CF | PSW_SF | PSW_ZF | PSW_PF | DUF, 0 },
    { TYPE_NONE, CLASS_OR, "orS", 1,
        Gv, Ev, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
	PSW_OF | PSW_CF | PSW_SF | PSW_ZF | PSW_PF | DUF, 0 },
    { TYPE_NONE, CLASS_OR, "orb", 1,
        AL, Ib, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
	PSW_OF | PSW_CF | PSW_SF | PSW_ZF | PSW_PF | DUF, 0 },
    { TYPE_NONE, CLASS_OR, "orS", 1,
        eAX, Iv, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
	PSW_OF | PSW_CF | PSW_SF | PSW_ZF | PSW_PF | DUF, 0 },
    { TYPE_NONE, CLASS_PUSHCS, "pushS", 4,
        NULL_OPERANDS_1, NULL_OPERANDS_2, eSP_reg, no_reg, cs_reg, eSP_reg,
        ADDRESS_SIZE, 0,
        0, 0 },
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },	// 0x0f extended opcode escape
    /* 10 */
    { TYPE_NONE, CLASS_ADDC, "adcb", 2,
        Eb, Gb, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
	PSW_CF | PSW_SF | PSW_ZF | PSW_AF | PSW_CF | PSW_PF | DUF, PSW_CF },
    { TYPE_NONE, CLASS_ADDC, "adcS", 2,
        Ev, Gv, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
	PSW_CF | PSW_SF | PSW_ZF | PSW_AF | PSW_CF | PSW_PF | DUF, PSW_CF },
    { TYPE_NONE, CLASS_ADDC, "adcb", 2,
        Gb, Eb, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
	PSW_CF | PSW_SF | PSW_ZF | PSW_AF | PSW_CF | PSW_PF | DUF, PSW_CF },
    { TYPE_NONE, CLASS_ADDC, "adcS", 2,
        Gv, Ev, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
	PSW_CF | PSW_SF | PSW_ZF | PSW_AF | PSW_CF | PSW_PF | DUF, PSW_CF },
    { TYPE_NONE, CLASS_ADDC, "adcb", 2,
        AL, Ib, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
	PSW_CF | PSW_SF | PSW_ZF | PSW_AF | PSW_CF | PSW_PF | DUF, PSW_CF },
    { TYPE_NONE, CLASS_ADDC, "adcS", 2,
        eAX, Iv, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
	PSW_CF | PSW_SF | PSW_ZF | PSW_AF | PSW_CF | PSW_PF | DUF, PSW_CF },
    { TYPE_NONE, CLASS_PUSHSS, "pushS", 4,
        NULL_OPERANDS_1, NULL_OPERANDS_2, eSP_reg, no_reg, ss_reg, eSP_reg,
        ADDRESS_SIZE, 0,
        0, 0 },
    { TYPE_NONE, CLASS_POPSS, "popS", COMPLEX,
        NULL_OPERANDS_1, NULL_OPERANDS_2, ss_reg, eSP_reg, eSP_reg, no_reg,
        ADDRESS_SIZE, 0,
        0, 0 },
    /* 18 */
    { TYPE_NONE, CLASS_SUBB, "sbbb", 2,
        Eb, Gb, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
	PSW_CF | PSW_OF | PSW_PF | PSW_AF | PSW_SF | PSW_ZF | DUF, PSW_CF },
    { TYPE_NONE, CLASS_SUBB, "sbbS", 2,
        Ev, Gv, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
	PSW_CF | PSW_OF | PSW_PF | PSW_AF | PSW_SF | PSW_ZF | DUF, PSW_CF },
    { TYPE_NONE, CLASS_SUBB, "sbbb", 2, Gb, Eb, NULL_OPERANDS_1,
	    NO_REGISTERS_ALL, 0, 0,
	PSW_CF | PSW_OF | PSW_PF | PSW_AF | PSW_SF | PSW_ZF | DUF, PSW_CF },
    { TYPE_NONE, CLASS_SUBB, "sbbS", 2,
        Gv, Ev, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
	PSW_CF | PSW_OF | PSW_PF | PSW_AF | PSW_SF | PSW_ZF | DUF, PSW_CF },
    { TYPE_NONE, CLASS_SUBB, "sbbb", 2,
        AL, Ib, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
	PSW_CF | PSW_OF | PSW_PF | PSW_AF | PSW_SF | PSW_ZF | DUF, PSW_CF },
    { TYPE_NONE, CLASS_SUBB, "sbbS", 2,
        eAX, Iv, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
	PSW_CF | PSW_OF | PSW_PF | PSW_AF | PSW_SF | PSW_ZF | DUF, PSW_CF },
    { TYPE_NONE, CLASS_PUSHDS, "pushS", 4,
        NULL_OPERANDS_1, NULL_OPERANDS_2,
        eSP_reg, no_reg, ds_reg, eSP_reg,
        ADDRESS_SIZE, 0,
        0, 0 },
    { TYPE_NONE, CLASS_POPDS, "popS", COMPLEX,
        NULL_OPERANDS_1, NULL_OPERANDS_2, eSP_reg, ds_reg, eSP_reg, no_reg,
        0, ADDRESS_SIZE,
        0, 0 },
    /* 20 */
    { TYPE_NONE, CLASS_AND, "andb", 1,
        Eb, Gb, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
	PSW_OF | PSW_CF | PSW_AF | PSW_SF | PSW_ZF | PSW_PF | DUF, 0 },
    { TYPE_NONE, CLASS_AND, "andS", 1,
        Ev, Gv, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
	PSW_OF | PSW_CF | PSW_AF | PSW_SF | PSW_ZF | PSW_PF | DUF, 0 },
    { TYPE_NONE, CLASS_AND, "andb", 1,
        Gb, Eb, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
	PSW_OF | PSW_CF | PSW_AF | PSW_SF | PSW_ZF | PSW_PF | DUF, 0 },
    { TYPE_NONE, CLASS_AND, "andS", 1,
        Gv, Ev, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
	PSW_OF | PSW_CF | PSW_AF | PSW_SF | PSW_ZF | PSW_PF | DUF, 0 },
    { TYPE_NONE, CLASS_AND, "andb", 1,
        AL, Ib, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
	PSW_OF | PSW_CF | PSW_AF | PSW_SF | PSW_ZF | PSW_PF | DUF, 0 },
    { TYPE_NONE, CLASS_AND, "andS", 1,
        eAX, Iv, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
	PSW_OF | PSW_CF | PSW_AF | PSW_SF | PSW_ZF | PSW_PF | DUF, 0 },
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },	// SEG ES prefix
    { TYPE_NONE, CLASS_AADJUST, "daa", 1,
        NULL_OPERANDS_3, al_reg, no_reg, al_reg, no_reg,
	0, 0,
        PSW_ZF | PSW_CF | PSW_SF | PSW_AF | PSW_PF, 0 },
    /* 28 */
    { TYPE_NONE, CLASS_SUB, "subb", 1,
        Eb, Gb, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
	PSW_OF | PSW_SF | PSW_ZF | PSW_AF | PSW_PF | PSW_CF | DUF, 0 },
    { TYPE_NONE, CLASS_SUB, "subS", 1,
        Ev, Gv, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
	PSW_OF | PSW_SF | PSW_ZF | PSW_AF | PSW_PF | PSW_CF | DUF, 0 },
    { TYPE_NONE, CLASS_SUB, "subb", 1,
        Gb, Eb, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
	PSW_OF | PSW_SF | PSW_ZF | PSW_AF | PSW_PF | PSW_CF | DUF, 0 },
    { TYPE_NONE, CLASS_SUB, "subS", 1,
        Gv, Ev, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
	PSW_OF | PSW_SF | PSW_ZF | PSW_AF | PSW_PF | PSW_CF | DUF, 0 },
    { TYPE_NONE, CLASS_SUB, "subb", 1,
        AL, Ib, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
	PSW_OF | PSW_SF | PSW_ZF | PSW_AF | PSW_PF | PSW_CF | DUF, 0 },
    { TYPE_NONE, CLASS_SUB, "subS", 1,
        eAX, Iv, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
	PSW_OF | PSW_SF | PSW_ZF | PSW_AF | PSW_PF | PSW_CF | DUF, 0 },
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },	// SEG CS prefix
    { TYPE_NONE, CLASS_SADJUST, "das", 1,
        NULL_OPERANDS_3, al_reg, no_reg, al_reg, no_reg,
	0, 0,
        PSW_ZF | PSW_CF | PSW_SF | PSW_AF | PSW_PF, 0 },
    /* 30 */
    { TYPE_NONE, CLASS_XOR, "xorb", 1,
        Eb, Gb, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
	PSW_OF | PSW_CF | PSW_SF | PSW_ZF | PSW_PF | DUF },
    { TYPE_NONE, CLASS_XOR, "xorS", 1,
        Ev, Gv, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
	PSW_OF | PSW_CF | PSW_SF | PSW_ZF | PSW_PF | DUF },
    { TYPE_NONE, CLASS_XOR, "xorb", 1,
        Gb, Eb, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
	PSW_OF | PSW_CF | PSW_SF | PSW_ZF | PSW_PF | DUF },
    { TYPE_NONE, CLASS_XOR, "xorS", 1,
        Gv, Ev, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
	PSW_OF | PSW_CF | PSW_SF | PSW_ZF | PSW_PF | DUF },
    { TYPE_NONE, CLASS_XOR, "xorb", 1,
        AL, Ib, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
	PSW_OF | PSW_CF | PSW_SF | PSW_ZF | PSW_PF | DUF },
    { TYPE_NONE, CLASS_XOR, "xorS", 1,
        eAX, Iv, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
	PSW_OF | PSW_CF | PSW_SF | PSW_ZF | PSW_PF | DUF },
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },	// SEG SS prefix
    { TYPE_NONE, CLASS_AAA, "aaa", 1,
        NULL_OPERANDS_3, al_reg, no_reg, al_reg, no_reg,
        0, 0,
	PSW_AF | PSW_CF, 0 },
    /* 38 */
    { TYPE_NONE, CLASS_CMP, "cmpb", 1,
        NULL_OPERANDS_1, Eb, Gb, NO_REGISTERS_ALL,
        0, 0,
	PSW_CF | PSW_OF | PSW_ZF | PSW_SF | PSW_PF | PSW_AF, 0 },
    { TYPE_NONE, CLASS_CMP, "cmpS", 1,
        NULL_OPERANDS_1, Ev, Gv, NO_REGISTERS_ALL,
        0, 0,
	PSW_CF | PSW_OF | PSW_SF | PSW_ZF | PSW_AF | PSW_PF, 0 },
    { TYPE_NONE, CLASS_CMP, "cmpb", 1,
        NULL_OPERANDS_1, Gb, Eb, NO_REGISTERS_ALL,
        0, 0,
	PSW_CF | PSW_OF | PSW_SF | PSW_ZF | PSW_AF | PSW_PF, 0 },
    { TYPE_NONE, CLASS_CMP, "cmpS", 1,
        NULL_OPERANDS_1, Gv, Ev, NO_REGISTERS_ALL,
        0, 0,
	PSW_CF | PSW_OF | PSW_SF | PSW_ZF | PSW_AF | PSW_PF, 0 },
    { TYPE_NONE, CLASS_CMP, "cmpb", 1,
        NULL_OPERANDS_1, AL, Ib, NO_REGISTERS_ALL,
        0, 0,
	PSW_CF | PSW_OF | PSW_SF | PSW_ZF | PSW_AF | PSW_PF, 0 },
    { TYPE_NONE, CLASS_CMP, "cmpS", 1,
        NULL_OPERANDS_1, eAX, Iv, NO_REGISTERS_ALL,
        0, 0,
	PSW_CF | PSW_OF | PSW_SF | PSW_ZF | PSW_AF | PSW_PF, 0 },
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },	// SEG DS prefix
    { TYPE_NONE, CLASS_AAS, "aas", 1,
        NULL_OPERANDS_3, ah_reg, no_reg, ah_reg, no_reg,
        0, 0,
	PSW_AF | PSW_CF, 0 },
    /* 40 */
    { TYPE_NONE, CLASS_INC, "incS", 1,
        eAX, NULL_OPERANDS_2, NO_REGISTERS_ALL,
        0, 0,
	PSW_OF | PSW_SF | PSW_ZF | PSW_AF | PSW_PF | DUF, 0 },
    { TYPE_NONE, CLASS_INC, "incS", 1,
        eCX, NULL_OPERANDS_2, NO_REGISTERS_ALL,
        0, 0,
	PSW_OF | PSW_SF | PSW_ZF | PSW_AF | PSW_PF | DUF, 0 },
    { TYPE_NONE, CLASS_INC, "incS", 1,
        eDX, NULL_OPERANDS_2, NO_REGISTERS_ALL,
        0, 0,
	PSW_OF | PSW_SF | PSW_ZF | PSW_AF | PSW_PF | DUF, 0 },
    { TYPE_NONE, CLASS_INC, "incS", 1,
        eBX, NULL_OPERANDS_2, NO_REGISTERS_ALL,
        0, 0,
	PSW_OF | PSW_SF | PSW_ZF | PSW_AF | PSW_PF | DUF, 0 },
    { TYPE_NONE, CLASS_INC, "incS", 1,
        eSP, NULL_OPERANDS_2, NO_REGISTERS_ALL,
        0, 0,
	PSW_OF | PSW_SF | PSW_ZF | PSW_AF | PSW_PF | DUF, 0 },
    { TYPE_NONE, CLASS_INC, "incS", 1,
        eBP, NULL_OPERANDS_2, NO_REGISTERS_ALL,
        0, 0,
	PSW_OF | PSW_SF | PSW_ZF | PSW_AF | PSW_PF | DUF, 0 },
    { TYPE_NONE, CLASS_INC, "incS", 1,
        eSI, NULL_OPERANDS_2, NO_REGISTERS_ALL,
        0, 0,
	PSW_OF | PSW_SF | PSW_ZF | PSW_AF | PSW_PF | DUF, 0 },
    { TYPE_NONE, CLASS_INC, "incS", 1,
        eDI, NULL_OPERANDS_2, NO_REGISTERS_ALL,
        0, 0,
	PSW_OF | PSW_SF | PSW_ZF | PSW_AF | PSW_PF | DUF, 0 },
    /* 48 */
    { TYPE_NONE, CLASS_DEC, "decS", 1,
        eAX, NULL_OPERANDS_2, no_reg, no_reg, eAX_reg, no_reg,
        0, 0,
	PSW_ZF | PSW_SF | PSW_AF | PSW_PF | PSW_OF | DUF, 0 },
    { TYPE_NONE, CLASS_DEC, "decS", 1,
        eCX, NULL_OPERANDS_2, no_reg, no_reg, eCX_reg, no_reg,
        0, 0,
	PSW_ZF | PSW_SF | PSW_AF | PSW_PF | PSW_OF | DUF, 0 },
    { TYPE_NONE, CLASS_DEC, "decS", 1,
        eDX, NULL_OPERANDS_2, no_reg, no_reg, eDX_reg, no_reg,
        0, 0,
	PSW_ZF | PSW_SF | PSW_AF | PSW_PF | PSW_OF | DUF, 0 },
    { TYPE_NONE, CLASS_DEC, "decS", 1,
        eBX, NULL_OPERANDS_2, no_reg, no_reg, eBX_reg, no_reg,
        0, 0,
	PSW_ZF | PSW_SF | PSW_AF | PSW_PF | PSW_OF | DUF, 0 },
    { TYPE_NONE, CLASS_DEC, "decS", 1,
        eSP, NULL_OPERANDS_2, no_reg, no_reg, eSP_reg, no_reg,
        0, 0,
	PSW_ZF | PSW_SF | PSW_AF | PSW_PF | PSW_OF | DUF, 0 },
    { TYPE_NONE, CLASS_DEC, "decS", 1,
        eBP, NULL_OPERANDS_2, no_reg, no_reg, eBP_reg, no_reg,
        0, 0,
	PSW_ZF | PSW_SF | PSW_AF | PSW_PF | PSW_OF | DUF, 0 },
    { TYPE_NONE, CLASS_DEC, "decS", 1,
        eSI, NULL_OPERANDS_2, no_reg, no_reg, eSI_reg, no_reg,
        0, 0,
	PSW_ZF | PSW_SF | PSW_AF | PSW_PF | PSW_OF | DUF, 0 },
    { TYPE_NONE, CLASS_DEC, "decS", 1,
        eDI, NULL_OPERANDS_2, no_reg, no_reg, eDI_reg, no_reg,
        0, 0,
	PSW_ZF | PSW_SF | PSW_AF | PSW_PF | PSW_OF | DUF, 0 },
    /* 50 */
    { TYPE_NONE, CLASS_PUSH, "pushS", 3,
        NULL_OPERANDS_1, eAX, NULL_OPERANDS_1, eSP_reg, no_reg,	eSP_reg, no_reg,
        ADDRESS_SIZE, 0,
        0, 0 },
    { TYPE_NONE, CLASS_PUSH, "pushS", 3,
        NULL_OPERANDS_1, eCX, NULL_OPERANDS_1, eSP_reg, no_reg, eSP_reg, no_reg,
        ADDRESS_SIZE, 0,
        0, 0 },
    { TYPE_NONE, CLASS_PUSH, "pushS", 3,
        NULL_OPERANDS_1, eDX, NULL_OPERANDS_1, eSP_reg, no_reg,	eSP_reg, no_reg,
        ADDRESS_SIZE, 0,
        0, 0 },
    { TYPE_NONE, CLASS_PUSH, "pushS", 3,
        NULL_OPERANDS_1, eBX, NULL_OPERANDS_1, eSP_reg, no_reg, eSP_reg, no_reg,
        ADDRESS_SIZE, 0,
        0, 0 },
    { TYPE_NONE, CLASS_PUSH, "pushS", 3,
        NULL_OPERANDS_1, eSP, NULL_OPERANDS_1, eSP_reg, no_reg,	eSP_reg, no_reg,
        ADDRESS_SIZE, 0,
        0, 0 },
    { TYPE_NONE, CLASS_PUSH, "pushS", 3,
        NULL_OPERANDS_1, eBP, NULL_OPERANDS_1, eSP_reg, no_reg,	eSP_reg, no_reg,
        ADDRESS_SIZE, 0,
        0, 0 },
    { TYPE_NONE, CLASS_PUSH, "pushS", 3,
        NULL_OPERANDS_1, eSI, NULL_OPERANDS_1, eSP_reg, no_reg,	eSP_reg, no_reg,
        ADDRESS_SIZE, 0,
        0, 0 },
    { TYPE_NONE, CLASS_PUSH, "pushS", 3,
        NULL_OPERANDS_1, eDI, NULL_OPERANDS_1, eSP_reg, no_reg,	eSP_reg, no_reg,
        ADDRESS_SIZE, 0,
        0, 0 },
    /* 58 */
    { TYPE_NONE, CLASS_POP, "popS", 2,
        eAX, NULL_OPERANDS_2, eSP_reg, no_reg, eSP_reg, no_reg,
        0, ADDRESS_SIZE,
        0, 0 },
    { TYPE_NONE, CLASS_POP, "popS", 2,
        eCX, NULL_OPERANDS_2, eSP_reg, no_reg, eSP_reg, no_reg,
	0, ADDRESS_SIZE,
        0, 0 },
    { TYPE_NONE, CLASS_POP, "popS", 2,
        eDX, NULL_OPERANDS_2, eSP_reg, no_reg, eSP_reg, no_reg,
	0, ADDRESS_SIZE,
        0, 0 },
    { TYPE_NONE, CLASS_POP, "popS", 2,
        eBX, NULL_OPERANDS_2, eSP_reg, no_reg, eSP_reg, no_reg,
	0, ADDRESS_SIZE,
        0, 0 },
    { TYPE_NONE, CLASS_POP, "popS", 3,
        eSP, NULL_OPERANDS_2, eSP_reg, no_reg, eSP_reg, no_reg,
	0, ADDRESS_SIZE,
        0, 0 },
    { TYPE_NONE, CLASS_POP, "popS", 2,
        eBP, NULL_OPERANDS_2, eSP_reg, no_reg, eSP_reg, no_reg,
	0, ADDRESS_SIZE,
        0, 0 },
    { TYPE_NONE, CLASS_POP, "popS", 2,
        eSI, NULL_OPERANDS_2, eSP_reg, no_reg, eSP_reg, no_reg,
	0, ADDRESS_SIZE,
        0, 0 },
    { TYPE_NONE, CLASS_POP, "popS", 2,
        eDI, NULL_OPERANDS_2, eSP_reg, no_reg, eSP_reg, no_reg,
        0, ADDRESS_SIZE,
        0, 0 },
    /* 60 */
    { TYPE_NONE, CLASS_PUSHALL, "pushaS", COMPLEX,
        NULL_OPERANDS_3, eSP_reg, no_reg, all_regs, no_reg,
	ADDRESS_SIZE * 8, 0,
        0, 0 },
    { TYPE_NONE, CLASS_POPALL, "popaS", COMPLEX,
        NULL_OPERANDS_3, all_regs, no_reg, eSP_reg, no_reg,
        0, ADDRESS_SIZE * 8,
        0, 0 },
    { TYPE_NONE, CLASS_BOUND, "boundS", COMPLEX,
        NULL_OPERANDS_1, Gv, Ma, NO_REGISTERS_ALL,
        0, 0,
        0, 0 },
    { TYPE_NONE, CLASS_ARPL, "arpl", COMPLEX,
        Ew, Gw, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
	PSW_ZF, 0 },
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },	// seg fs
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },	// seg gs
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },	// op size prefix
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },	// adr size prefix
    /* 68 */
    { TYPE_NONE, CLASS_PUSH, "pushS", 3,
        NULL_OPERANDS_1, Iv, NULL_OPERANDS_1,
        eSP_reg, no_reg, eSP_reg, no_reg,
        ADDRESS_SIZE, 0,
        0, 0 },
    { TYPE_NONE, CLASS_IMUL, "imulS", 1,
        Gv, Ev, Iv, NO_REGISTERS_ALL,
        0, 0,
        PSW_CF | PSW_OF, 0 },
    { TYPE_NONE, CLASS_PUSH, "pushS", 4,
        NULL_OPERANDS_1, sIb, NULL_OPERANDS_1, eSP_reg, no_reg,	eSP_reg, no_reg,
        ADDRESS_SIZE, 0,
        0, 0 },
    { TYPE_NONE, CLASS_IMUL, "imulS", 1,
        Gv, Ev, Ib, NO_REGISTERS_ALL,
        0, 0,
        PSW_CF | PSW_OF, 0 },
    { TYPE_NONE, CLASS_INPUT, "insb", COMPLEX,
        Yb, indirDX, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, 0 },
    { TYPE_NONE, CLASS_INPUT, "insS", COMPLEX,
        Yv, indirDX, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, 0 },
    { TYPE_NONE, CLASS_OUTPUT, "outsb", COMPLEX,
        indirDX, Xb, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, 0 },
    { TYPE_NONE, CLASS_OUTPUT, "outsS", COMPLEX,
        indirDX, Xv, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, 0 },
    /* 70 */
    { TYPE_CONDITIONAL, CLASS_JO, "jo", 1,
        NULL_OPERANDS_1, Jb, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, PSW_OF },
    { TYPE_CONDITIONAL, CLASS_JNO, "jno", 1,
        NULL_OPERANDS_1, Jb, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, PSW_OF },
    { TYPE_CONDITIONAL, CLASS_JB, "jb", 1,
        NULL_OPERANDS_1, Jb, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, PSW_CF },
    { TYPE_CONDITIONAL, CLASS_JAE, "jae", 1,
        NULL_OPERANDS_1, Jb, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, PSW_CF },
    { TYPE_CONDITIONAL, CLASS_JE, "je", 1,
        NULL_OPERANDS_1, Jb, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, PSW_ZF },
    { TYPE_CONDITIONAL, CLASS_JNE, "jne", 1,
        NULL_OPERANDS_1, Jb, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, PSW_ZF },
    { TYPE_CONDITIONAL, CLASS_JBE, "jbe", 1,
        NULL_OPERANDS_1, Jb, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, PSW_CF | PSW_ZF },
    { TYPE_CONDITIONAL, CLASS_JA, "ja", 1,
        NULL_OPERANDS_1, Jb, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, PSW_CF | PSW_ZF },
    /* 78 */
    { TYPE_CONDITIONAL, CLASS_JS, "js", 1,
        NULL_OPERANDS_1, Jb, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, PSW_SF },
    { TYPE_CONDITIONAL, CLASS_JNS, "jns", 1,
        NULL_OPERANDS_1, Jb, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, PSW_SF },
    { TYPE_CONDITIONAL, CLASS_JP, "jp", 1,
        NULL_OPERANDS_1, Jb, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, PSW_PF },
    { TYPE_CONDITIONAL, CLASS_JNP, "jnp", 1,
        NULL_OPERANDS_1, Jb, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, PSW_PF },
    { TYPE_CONDITIONAL, CLASS_JL, "jl", 1,
        NULL_OPERANDS_1, Jb, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, PSW_OF | PSW_SF },
    { TYPE_CONDITIONAL, CLASS_JNL, "jnl", 1,
        NULL_OPERANDS_1, Jb, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, PSW_OF | PSW_SF },
    { TYPE_CONDITIONAL, CLASS_JLE, "jle", 1,
        NULL_OPERANDS_1, Jb, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, PSW_OF | PSW_ZF | PSW_SF },
    { TYPE_CONDITIONAL, CLASS_JG, "jg", 1,
        NULL_OPERANDS_1, Jb, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, PSW_OF | PSW_ZF | PSW_SF },
    /* 80 */
    { GRP1b },
    { GRP1S },
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    { GRP1Ss },
    { TYPE_NONE, CLASS_TEST, "testb", 1,
        NULL_OPERANDS_1, Eb, Gb, NO_REGISTERS_ALL,
        0, 0,
	PSW_CF | PSW_OF | PSW_PF | PSW_SF | PSW_ZF, 0 },
    { TYPE_NONE, CLASS_TEST, "testS", 1,
        NULL_OPERANDS_1, Ev, Gv, NO_REGISTERS_ALL,
        0, 0,
	PSW_OF | PSW_CF | PSW_SF | PSW_ZF | PSW_PF, 0 },
    { TYPE_NONE, CLASS_XCHG, "xchgb", 3,
        Eb, Gb, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        DUF | S1D, 0 },
    { TYPE_NONE, CLASS_XCHG, "xchgS", 3,
        Ev, Gv, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        DUF | S1D, 0 },
    /* 88 */
    { TYPE_NONE, CLASS_MOV, "movb", 1,
        Eb, Gb, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, 0 },
    { TYPE_NONE, CLASS_MOV, "movS", 1,
        Ev, Gv, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, 0 },
    { TYPE_NONE, CLASS_MOV, "movb", 1,
        Gb, Eb, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, 0 },
    { TYPE_NONE, CLASS_MOV, "movS", 1,
        Gv, Ev, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, 0 },
    { TYPE_NONE, CLASS_MOV_SEG, "movS", COMPLEX,
        Sw, Ev, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, 0 },
    { TYPE_NONE, CLASS_LEA, "leaS", 1,
        Gv, M, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, 0 },
    { TYPE_NONE, CLASS_MOV, "movS", COMPLEX,
        Sw, Ev, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, 0 },
    { TYPE_NONE, CLASS_POP, "popS", COMPLEX,
        Ev, NULL_OPERANDS_2, eSP_reg, no_reg, eSP_reg, no_reg,
	0, ADDRESS_SIZE,
        0, 0 },
    /* 90 */
    { TYPE_NONE, CLASS_NOP, "nop", 1,
        NULL_OPERANDS_3, NO_REGISTERS_ALL,
        0, 0,
        0, 0 },
    { TYPE_NONE, CLASS_XCHG, "xchgS", COMPLEX,
        eCX, eAX, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        DUF | S1D, 0 },
    { TYPE_NONE, CLASS_XCHG, "xchgS", COMPLEX,
        eDX, eAX, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        DUF | S1D, 0 },
    { TYPE_NONE, CLASS_XCHG, "xchgS", COMPLEX,
        eBX, eAX, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        DUF | S1D, 0 },
    { TYPE_NONE, CLASS_XCHG, "xchgS", COMPLEX,
        eSP, eAX, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        DUF | S1D, 0 },
    { TYPE_NONE, CLASS_XCHG, "xchgS", COMPLEX,
        eBP, eAX, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        DUF | S1D, 0 },
    { TYPE_NONE, CLASS_XCHG, "xchgS", COMPLEX,
        eSI, eAX, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        DUF | S1D, 0 },
    { TYPE_NONE, CLASS_XCHG, "xchgS", COMPLEX,
        eDI, eAX, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        DUF | S1D, 0 },
    /* 98 */
    { TYPE_ERROR, CLASS_CWTS, "cWtS", COMPLEX,
        NULL_OPERANDS_3, eAX_reg, no_reg, eAX_reg, eDX_reg,
        0, 0,
        0, 0 },
    { TYPE_ERROR, CLASS_CSTD, "cStd", COMPLEX,
        NULL_OPERANDS_3, eAX_reg, eDX_reg, eAX_reg, no_reg,
        0, 0,
        0, 0 },
    { TYPE_UNCONDITIONAL, CLASS_LCALL, "lcall", COMPLEX,
        NULL_OPERANDS_1, Ap, NULL_OPERANDS_1, NO_REGISTERS_ALL,
	ADDRESS_SIZE, 0,
        PSW_ALL, 0 },
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },	// fwait
    { TYPE_NONE, CLASS_PUSHF, "pushf", COMPLEX,
        NULL_OPERANDS_3, eSP_reg, no_reg, eSP_reg, no_reg,
        ADDRESS_SIZE, 0,
	0, PSW_ALL },
    { TYPE_NONE, CLASS_POPF, "popf", COMPLEX,
        NULL_OPERANDS_3, eSP_reg, no_reg, eSP_reg, no_reg,
	0, ADDRESS_SIZE,
        PSW_ALL, 0 },
    { TYPE_NONE, CLASS_SAHF, "sahf", 1,
        NULL_OPERANDS_3, no_reg, no_reg, ah_reg, no_reg,
        0, 0,
	PSW_SF | PSW_ZF | PSW_CF | PSW_AF | PSW_PF, 0 },
    { TYPE_NONE, CLASS_LAHF, "lahf", 1,
        NULL_OPERANDS_3, ah_reg, no_reg, no_reg, no_reg,
        0, 0,
        0, PSW_SF | PSW_ZF | PSW_CF | PSW_AF | PSW_PF },
    /* a0 */
    { TYPE_NONE, CLASS_MOV, "movb", 1,
        AL, Ob, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, 0 },
    { TYPE_NONE, CLASS_MOV, "movS", 1,
        eAX, Ov, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, 0 },
    { TYPE_NONE, CLASS_MOV, "movb", 2,
        Ob, AL, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, 0 },
    { TYPE_NONE, CLASS_MOV, "movS", 2,
        Ov, eAX, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, 0 },
    { TYPE_NONE, CLASS_MOV, "movsb", COMPLEX,
        Yb, Xb, NULL_OPERANDS_1, eDI_reg, eSI_reg, no_reg, no_reg,
        0, 0,
        0, PSW_DF },
    { TYPE_NONE, CLASS_MOV, "movsS", COMPLEX,
        Yv, Xv, NULL_OPERANDS_1, eDI_reg, eSI_reg, no_reg, no_reg,
        0, 0,
        0, PSW_DF },
    { TYPE_NONE, CLASS_CMPS, "cmpsb", COMPLEX,
        NULL_OPERANDS_1, Yb, Xb, eDI_reg, eSI_reg, eDI_reg, eSI_reg,
        0, 0,
	PSW_OF | PSW_CF | PSW_SF | PSW_ZF | PSW_AF | PSW_PF, PSW_DF },
    { TYPE_NONE, CLASS_CMPS, "cmpsS", COMPLEX,
        NULL_OPERANDS_1, Yv, Xv, eDI_reg, eSI_reg, eDI_reg, eSI_reg,
        0, 0,
	PSW_OF | PSW_CF | PSW_SF | PSW_ZF | PSW_AF | PSW_PF, PSW_DF },
    /* a8 */
    { TYPE_NONE, CLASS_TEST, "testb", 1,
        NULL_OPERANDS_1, AL, Ib, NO_REGISTERS_ALL,
        0, 0,
	PSW_OF | PSW_CF | PSW_SF | PSW_ZF | PSW_PF, 0 },
    { TYPE_NONE, CLASS_TEST, "testS", 1,
        NULL_OPERANDS_1, eAX, Iv, NO_REGISTERS_ALL,
        0, 0,
	PSW_CF | PSW_OF | PSW_PF | PSW_SF | PSW_ZF, 0 },
    { TYPE_NONE, CLASS_STOS, "stosb", 3,
        Yb, AL, NULL_OPERANDS_1, no_reg, no_reg, eDI_reg, no_reg,
        0, 0,
        0, PSW_DF },
    { TYPE_NONE, CLASS_STOS, "stosS", COMPLEX,
        Yv, eAX, NULL_OPERANDS_1, no_reg, no_reg, eDI_reg, no_reg,
        0, 0,
        0, PSW_DF },
    { TYPE_NONE, CLASS_LODS, "lodsb", 2,
        AL, Xb, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, PSW_DF },
    { TYPE_NONE, CLASS_LODS, "lodsS", COMPLEX,
        eAX, Xv, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, PSW_DF },
    { TYPE_NONE, CLASS_SCAS, "scasb", 3,
        NULL_OPERANDS_1, AL, Yb, no_reg, eDI_reg, eDI_reg, no_reg,
        0, 0,
	PSW_CF | PSW_OF | PSW_AF | PSW_PF | PSW_SF | PSW_ZF, PSW_DF },
    { TYPE_NONE, CLASS_SCAS, "scasS", COMPLEX,
        NULL_OPERANDS_1, eAX, Yv, no_reg, eDI_reg, eDI_reg, no_reg,
        0, 0,
	PSW_CF | PSW_OF | PSW_AF | PSW_PF | PSW_SF | PSW_ZF, PSW_DF },
    /* b0 */
    { TYPE_NONE, CLASS_MOV, "movb", 1,
        AL, Ib, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, 0 },
    { TYPE_NONE, CLASS_MOV, "movb", 1,
        CL, Ib, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, 0 },
    { TYPE_NONE, CLASS_MOV, "movb", 1,
        DL, Ib, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, 0 },
    { TYPE_NONE, CLASS_MOV, "movb", 1,
        BL, Ib, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, 0 },
    { TYPE_NONE, CLASS_MOV, "movb", 1,
        AH, Ib, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, 0 },
    { TYPE_NONE, CLASS_MOV, "movb", 1,
        CH, Ib, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, 0 },
    { TYPE_NONE, CLASS_MOV, "movb", 1,
        DH, Ib, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, 0 },
    { TYPE_NONE, CLASS_MOV, "movb", 1,
        BH, Ib, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, 0 },
    /* b8 */
    { TYPE_NONE, CLASS_MOV, "movS", 1,
        eAX, Iv, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, 0 },
    { TYPE_NONE, CLASS_MOV, "movS", 1,
        eCX, Iv, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, 0 },
    { TYPE_NONE, CLASS_MOV, "movS", 1,
        eDX, Iv, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, 0 },
    { TYPE_NONE, CLASS_MOV, "movS", 1,
        eBX, Iv, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, 0 },
    { TYPE_NONE, CLASS_MOV, "movS", 1,
        eSP, Iv, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, 0 },
    { TYPE_NONE, CLASS_MOV, "movS", 1,
        eBP, Iv, NULL_OPERANDS_1,
        NO_REGISTERS_ALL,
        0, 0,
        0, 0 },
    { TYPE_NONE, CLASS_MOV, "movS", 1,
        eSI, Iv, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, 0 },
    { TYPE_NONE, CLASS_MOV, "movS", 1,
        eDI, Iv, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, 0 },
    /* c0 */
    { GRP2b },
    { GRP2S },
    { TYPE_NONE, CLASS_RET, "ret", COMPLEX,
        NULL_OPERANDS_1, Iw, NULL_OPERANDS_1, no_reg, no_reg, eSP_reg, no_reg,
        0, ADDRESS_SIZE,
        0, 0 },
    { TYPE_NONE, CLASS_RET, "ret", COMPLEX,
        NULL_OPERANDS_3, no_reg, no_reg, eSP_reg, no_reg,
        0, ADDRESS_SIZE,
        0, 0 },
    { TYPE_NONE, CLASS_LES, "lesS", COMPLEX,
        Gv, Mp, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, 0 },
    { TYPE_NONE, CLASS_LDS, "ldsS", COMPLEX,
        Gv, Mp, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, 0 },
    { TYPE_NONE, CLASS_MOV, "movb", 1,
        Eb, Ib, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, 0 },
    { TYPE_NONE, CLASS_MOV, "movS", 1,
        Ev, Iv, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, 0 },
    /* c8 */
    { TYPE_NONE, CLASS_ENTER, "enter", COMPLEX,
        NULL_OPERANDS_1, Iw, Ib, no_reg, no_reg, eBP_reg, eSP_reg,
	ADDRESS_SIZE, 0,
        0, 0 },
    { TYPE_NONE, CLASS_LEAVE, "leave", 3,
        NULL_OPERANDS_3, eSP_reg, eBP_reg, eBP_reg, no_reg,
        0, ADDRESS_SIZE,
        0, 0 },
    { TYPE_UNCONDITIONAL, CLASS_LRET, "lret", COMPLEX,
        NULL_OPERANDS_1, Iw, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, ADDRESS_SIZE,
        0, 0 },
    { TYPE_UNCONDITIONAL, CLASS_LRET, "lret", COMPLEX,
        NULL_OPERANDS_3, NO_REGISTERS_ALL,
        0, ADDRESS_SIZE,
        0, 0 },
    { TYPE_NONE, CLASS_INT3, "int3", COMPLEX,
        NULL_OPERANDS_3, no_reg, no_reg, all_regs, no_reg,
        0, 0,
        0, 0 },
    { TYPE_NONE, CLASS_INT, "int", COMPLEX,
        NULL_OPERANDS_1, Ib, NULL_OPERANDS_1, no_reg, no_reg, all_regs, no_reg,
        0, 0,
        0, 0 },
    { TYPE_NONE, CLASS_INTO, "into", COMPLEX,
        NULL_OPERANDS_3, no_reg, no_reg, all_regs, no_reg,
        0, 0,
        0, PSW_OF },
    { TYPE_NONE, CLASS_IRET, "iret", COMPLEX,
        NULL_OPERANDS_3, NO_REGISTERS_ALL,
        0, 0,
        PSW_ALL, 0 },
    /* d0 */
    { GRP2b_one },
    { GRP2S_one },
    { GRP2b_cl },
    { GRP2S_cl },
    { TYPE_NONE, CLASS_MADJUST, "aam", 4,
        NULL_OPERANDS_1, Ib, NULL_OPERANDS_1, al_reg, ah_reg, al_reg, no_reg,
        0, 0,
        PSW_SF | PSW_ZF | PSW_PF, 0 },
    { TYPE_NONE, CLASS_DADJUST, "aad", 3,
        NULL_OPERANDS_1, Ib, NULL_OPERANDS_1, al_reg, ah_reg, al_reg, ah_reg,
        0, 0,
        PSW_SF | PSW_ZF | PSW_PF, 0 },
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    { TYPE_NONE, CLASS_XLAT, "xlat", 2,
        NULL_OPERANDS_3, al_reg, no_reg, al_reg, eBX_reg,
        0, 0,
        0, 0 },
    /* d8 */
    { FLOAT },
    { FLOAT },
    { FLOAT },
    { FLOAT },
    { FLOAT },
    { FLOAT },
    { FLOAT },
    { FLOAT },
    /* e0 */
    { TYPE_CONDITIONAL, CLASS_LOOPNE, "loopne", 4,
        NULL_OPERANDS_1, Jb, NULL_OPERANDS_1, no_reg, no_reg, eCX_reg, no_reg,
        0, 0,
        0, PSW_ZF },
    { TYPE_CONDITIONAL, CLASS_LOOPE, "loope", 4,
        NULL_OPERANDS_1, Jb, NULL_OPERANDS_1, no_reg, no_reg, eCX_reg, no_reg,
        0, 0,
        0, PSW_ZF },
    { TYPE_CONDITIONAL, CLASS_LOOP, "loop", 4,
        NULL_OPERANDS_1, Jb, NULL_OPERANDS_1, no_reg, no_reg, eCX_reg, no_reg,
        0, 0,
        0, 0 },
    { TYPE_CONDITIONAL, CLASS_JCXZ, "jCcxz", COMPLEX,
        NULL_OPERANDS_1, Jb, NULL_OPERANDS_1, no_reg, no_reg, eCX_reg,	no_reg,
        0, 0,
        0, 0 },
    { TYPE_NONE, CLASS_INPUT, "inb", COMPLEX,
        AL, Ib, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, 0 },
    { TYPE_NONE, CLASS_INPUT, "inS", COMPLEX,
        eAX, Ib, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, 0 },
    { TYPE_NONE, CLASS_OUTPUT, "outb", COMPLEX,
        Ib, AL, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, 0 },
    { TYPE_NONE, CLASS_OUTPUT, "outS", COMPLEX,
        Ib, eAX, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, 0 },
    /* e8 */
    { TYPE_UNCONDITIONAL, CLASS_CALL, "call", COMPLEX,
        NULL_OPERANDS_1, Av, NULL_OPERANDS_1, NO_REGISTERS_ALL,
	ADDRESS_SIZE, 0,
        PSW_ALL, 0 },
    { TYPE_UNCONDITIONAL, CLASS_JUMP, "jmp", 1,
        NULL_OPERANDS_1, Jv,  NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, 0 },
    { TYPE_UNCONDITIONAL, CLASS_LJUMP1, "ljmp", COMPLEX,
        NULL_OPERANDS_1, Id, Iv, NO_REGISTERS_ALL,
        0, 0,
        PSW_ALL, 0 },
    { TYPE_UNCONDITIONAL, CLASS_JUMP, "jmp", 1,
        NULL_OPERANDS_1, Jb, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, 0 },
    { TYPE_NONE, CLASS_INPUT, "inb", COMPLEX,
        AL, indirDX, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, 0 },
    { TYPE_NONE, CLASS_INPUT, "inS", COMPLEX,
        eAX, indirDX, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, 0 },
    { TYPE_NONE, CLASS_OUTPUT, "outb", COMPLEX,
        indirDX, AL, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, 0 },
    { TYPE_NONE, CLASS_OUTPUT, "outS", COMPLEX,
        indirDX, eAX, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, 0 },
    /* f0 */
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },	// lock prefix
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },	// repne
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },	// repz
    { TYPE_NONE, CLASS_HALT, "hlt", COMPLEX,
        NULL_OPERANDS_3, NO_REGISTERS_ALL,
        0, 0,
        0, 0 },
    { TYPE_NONE, CLASS_CMC, "cmc", 1,
        NULL_OPERANDS_3, NO_REGISTERS_ALL,
        0, 0,
        PSW_CF, PSW_CF },
    { GRP3b },
    { GRP3S },
    /* f8 */
    { TYPE_NONE, CLASS_CLC, "clc", 1,
        NULL_OPERANDS_3, NO_REGISTERS_ALL,
        0, 0,
        PSW_CF, 0 },
    { TYPE_NONE, CLASS_STC, "stc", 1,
        NULL_OPERANDS_3, NO_REGISTERS_ALL,
        0, 0,
        PSW_CF, 0 },
    { TYPE_NONE, CLASS_CLI, "cli", COMPLEX,
        NULL_OPERANDS_3, NO_REGISTERS_ALL,
        0, 0,
        PSW_IF | PSW_VM | PSW_IOPL1, 0 },
    { TYPE_NONE, CLASS_STI, "sti", COMPLEX,
        NULL_OPERANDS_3, NO_REGISTERS_ALL,
        0, 0,
        PSW_IF, 0 },
    { TYPE_NONE, CLASS_CLD, "cld", 4,
        NULL_OPERANDS_3, NO_REGISTERS_ALL,
        0, 0,
        PSW_DF, 0 },
    { TYPE_NONE, CLASS_STD, "std", 4,
        NULL_OPERANDS_3, NO_REGISTERS_ALL,
        0, 0,
        PSW_DF, 0 },
    { GRP4 },
    { GRP5 }
};

X86op dis386_twobyte[] = {
    /* 00 */
    { GRP6 },
    { GRP7 },
    { TYPE_NONE, CLASS_LAR, "larS", COMPLEX,
        Gv, Ew, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        PSW_ZF, 0 },
    { TYPE_NONE, CLASS_LSL, "lslS", COMPLEX,
        Gv, Ew, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        PSW_ZF, 0 },
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    { TYPE_NONE, CLASS_CLTS, "clts", COMPLEX,
        NULL_OPERANDS_3, NO_REGISTERS_ALL,
        0, 0,
        0, 0 },
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    /* 08 */
    { TYPE_NONE, CLASS_INVD, "invd", COMPLEX,
        NULL_OPERANDS_3, NO_REGISTERS_ALL,
        0, 0,
        0, 0 },
    { TYPE_NONE, CLASS_WBINVD, "wbinvd", COMPLEX,
        NULL_OPERANDS_3, NO_REGISTERS_ALL,
        0, 0,
        0, 0 },
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    { TYPE_NONE, CLASS_UD2, "ud2a", COMPLEX,
        NULL_OPERANDS_3, NO_REGISTERS_ALL,
        0, 0,
        0, 0 },
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },    
    { TYPE_NONE, CLASS_FEMMS, "femms", COMPLEX,
        NULL_OPERANDS_3, NO_REGISTERS_ALL,
        0, 0,
        0, 0 },
    { TYPE_NONE, CLASS_3DNOW, "3dnow", COMPLEX,
        TD, EM,
        SUF },
    /* 10 */
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    /* 18 */
    { TYPE_NONE, CLASS_PREFETCH, "prefetch", COMPLEX,
        PF },
    //    { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    /* 20 */
    /* these are all backward in appendix A of the intel book */
    { TYPE_NONE, CLASS_MOV, "movl", COMPLEX,
        Rd, Cd, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, 0 },
    { TYPE_NONE, CLASS_MOV, "movl", COMPLEX,
        Rd, Dd, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, 0 },
    { TYPE_NONE, CLASS_MOV, "movl", COMPLEX,
        Cd, Rd, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, 0 },
    { TYPE_NONE, CLASS_MOV, "movl", COMPLEX,
        Dd, Rd, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, 0 },
    { TYPE_NONE, CLASS_MOV, "movl", COMPLEX,
        Rd, Td, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, 0 },
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    { TYPE_NONE, CLASS_MOV, "movl", COMPLEX,
        Td, Rd, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, 0 },
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    /* 28 */
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    /* 30 */
    { TYPE_NONE, CLASS_WRMSR, "wrmsr", COMPLEX,
        NULL_OPERANDS_3, no_reg, no_reg, all_regs, no_reg,
        0, 0,
        0, 0 },
    { TYPE_NONE, CLASS_RDTSC, "rdtsc", COMPLEX,
        NULL_OPERANDS_3, eAX_reg, eDX_reg, no_reg, no_reg,
        0, 0,
        0, 0 },
    { TYPE_NONE, CLASS_RDMSR, "rdmsr", COMPLEX,
        NULL_OPERANDS_3, eAX_reg, eDX_reg, eCX_reg, no_reg,
        0, 0,
        0, 0 },
    { TYPE_NONE, CLASS_RDPMC, "rdpmc", COMPLEX,
        NULL_OPERANDS_3, eAX_reg, eDX_reg, eCX_reg, no_reg,
        0, 0,
        0, 0 },
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    /* 38 */
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    /* 40 */
    { TYPE_NONE, CLASS_CMOVO, "cmovo", 3,
        Gv, Ev, NULL_OPERANDS_1,
        NO_REGISTERS_ALL, 0, 0, 0, PSW_OF },
    { TYPE_NONE, CLASS_CMOVNO, "cmovno", 3,
        Gv, Ev, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, PSW_OF },
    { TYPE_NONE, CLASS_CMOVB, "cmovb", 3,
        Gv, Ev, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, PSW_CF },
    { TYPE_NONE, CLASS_CMOVAE, "cmovae", 3,
        Gv, Ev, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, PSW_CF },
    { TYPE_NONE, CLASS_CMOVE, "cmove", 3,
        Gv, Ev, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, PSW_ZF },
    { TYPE_NONE, CLASS_CMOVNE, "cmovne", 3,
        Gv, Ev, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, PSW_ZF },
    { TYPE_NONE, CLASS_CMOVBE, "cmovbe", 3,
        Gv, Ev, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, PSW_CF | PSW_ZF },
    { TYPE_NONE, CLASS_CMOVA, "cmova", 3,
        Gv, Ev, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, PSW_CF | PSW_ZF },
    /* 48 */
    { TYPE_NONE, CLASS_CMOVS, "cmovs", 3,
        Gv, Ev, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, PSW_SF },
    { TYPE_NONE, CLASS_CMOVNS, "cmovns", 3,
        Gv, Ev, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, PSW_SF },
    { TYPE_NONE, CLASS_CMOVP, "cmovp", 3,
        Gv, Ev, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, PSW_PF },
    { TYPE_NONE, CLASS_CMOVNP, "cmovnp", 3,
        Gv, Ev, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, PSW_PF },
    { TYPE_NONE, CLASS_CMOVL, "cmovl", 3,
        Gv, Ev, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, PSW_SF | PSW_OF },
    { TYPE_NONE, CLASS_CMOVGE, "cmovge", 3,
        Gv, Ev, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, PSW_SF | PSW_OF },
    { TYPE_NONE, CLASS_CMOVLE, "cmovle", 3,
        Gv, Ev, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, PSW_OF | PSW_ZF | PSW_SF },
    { TYPE_NONE, CLASS_CMOVG, "cmovg", 3,
        Gv, Ev, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, PSW_OF | PSW_ZF | PSW_SF },
    /* 50 */
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    /* 58 */
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    /* 60 */
    { TYPE_NONE, CLASS_PACK, "punpcklbw", COMPLEX,
        MX, EM },
    { TYPE_NONE, CLASS_PACK, "punpcklwd", COMPLEX,
        MX, EM },
    { TYPE_NONE, CLASS_PACK, "punpckldq", COMPLEX,
        MX, EM },
    { TYPE_NONE, CLASS_PACK, "packsswb", COMPLEX,
        MX, EM },
    { TYPE_NONE, CLASS_PACK, "pcmpgtb", COMPLEX,
        MX, EM },
    { TYPE_NONE, CLASS_PACK, "pcmpgtw", COMPLEX,
        MX, EM },
    { TYPE_NONE, CLASS_PACK, "pcmpgtd", COMPLEX,
        MX, EM },
    { TYPE_NONE, CLASS_PACK, "packuswb", COMPLEX,
        MX, EM },
    /* 68 */
    { TYPE_NONE, CLASS_PACK, "punpckhbw", COMPLEX,
        MX, EM },
    { TYPE_NONE, CLASS_PACK, "punpckhwd", COMPLEX,
        MX, EM },
    { TYPE_NONE, CLASS_PACK, "punpckhdq", COMPLEX,
        MX, EM },
    { TYPE_NONE, CLASS_PACK, "packssdw", COMPLEX,
        MX, EM },
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    { TYPE_NONE, CLASS_MOV, "movd", COMPLEX,
        MX, Ev },
    { TYPE_NONE, CLASS_MOV, "movq", COMPLEX,
        MX, EM },
    /* 70 */
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    { GRP10 },
    { GRP11 },
    { GRP12 },
    { TYPE_NONE, CLASS_PACK, "pcmpeqb", COMPLEX,
        MX, EM },
    { TYPE_NONE, CLASS_PACK, "pcmpeqw", COMPLEX,
        MX, EM },
    { TYPE_NONE, CLASS_PACK, "pcmpeqd", COMPLEX,
        MX, EM },
    { TYPE_NONE, CLASS_MISC, "emms", COMPLEX,
        NULL_OPERANDS_3, NO_REGISTERS_ALL,
        0, 0,
        0, 0 },
    /* 78 */
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    { TYPE_NONE, CLASS_MOV, "movd", COMPLEX,
        Ev, MX },
    { TYPE_NONE, CLASS_MOV, "movq", COMPLEX,
        EM, MX },
    /* 80 */
    { TYPE_CONDITIONAL, CLASS_JO, "jo", 1,
        NULL_OPERANDS_1, Jv, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, PSW_OF },
    { TYPE_CONDITIONAL, CLASS_JNO, "jno", 1,
        NULL_OPERANDS_1, Jv, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, PSW_OF },
    { TYPE_CONDITIONAL, CLASS_JB, "jb", 1,
        NULL_OPERANDS_1, Jv, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, PSW_CF },
    { TYPE_CONDITIONAL, CLASS_JAE, "jae", 1,
        NULL_OPERANDS_1, Jv, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, PSW_CF },
    { TYPE_CONDITIONAL, CLASS_JE, "je", 1,
        NULL_OPERANDS_1, Jv, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, PSW_ZF },
    { TYPE_CONDITIONAL, CLASS_JNE, "jne", 1,
        NULL_OPERANDS_1, Jv, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, PSW_ZF },
    { TYPE_CONDITIONAL, CLASS_JBE, "jbe", 1,
        NULL_OPERANDS_1, Jv, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, PSW_CF | PSW_ZF },
    { TYPE_CONDITIONAL, CLASS_JA, "ja", 1,
        NULL_OPERANDS_1, Jv, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, PSW_CF | PSW_ZF },
    /* 88 */
    { TYPE_CONDITIONAL, CLASS_JS, "js", 1,
        NULL_OPERANDS_1, Jv, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, PSW_SF },
    { TYPE_CONDITIONAL, CLASS_JNS, "jns", 1,
        NULL_OPERANDS_1, Jv, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, PSW_SF },
    { TYPE_CONDITIONAL, CLASS_JP, "jp", 1,
        NULL_OPERANDS_1, Jv, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, PSW_PF },
    { TYPE_CONDITIONAL, CLASS_JNP, "jnp", 1,
        NULL_OPERANDS_1, Jv, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, PSW_PF },
    { TYPE_CONDITIONAL, CLASS_JL, "jl", 1,
        NULL_OPERANDS_1, Jv, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, PSW_OF | PSW_SF },
    { TYPE_CONDITIONAL, CLASS_JGE, "jge", 1,
        NULL_OPERANDS_1, Jv, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, PSW_OF | PSW_SF },
    { TYPE_CONDITIONAL, CLASS_JLE, "jle", 1,
        NULL_OPERANDS_1, Jv, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, PSW_OF | PSW_ZF | PSW_SF },
    { TYPE_CONDITIONAL, CLASS_JG, "jg", 1,
        NULL_OPERANDS_1, Jv, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, PSW_OF | PSW_ZF | PSW_SF },
    /* 90 */
    { TYPE_NONE, CLASS_SETO, "seto", 1,
        Eb, NULL_OPERANDS_2, NO_REGISTERS_ALL,
        0, 0,
        0, PSW_OF },
    { TYPE_NONE, CLASS_SETNO, "setno", 1,
        Eb, NULL_OPERANDS_2, NO_REGISTERS_ALL,
        0, 0,
        0, PSW_OF },
    { TYPE_NONE, CLASS_SETB, "setb", 1,
        Eb, NULL_OPERANDS_2, NO_REGISTERS_ALL,
        0, 0,
        0, PSW_CF },
    { TYPE_NONE, CLASS_SETAE, "setae", 1,
        Eb, NULL_OPERANDS_2, NO_REGISTERS_ALL,
        0, 0,
        0, PSW_CF },
    { TYPE_NONE, CLASS_SETE, "sete", 1,
        Eb, NULL_OPERANDS_2, NO_REGISTERS_ALL,
        0,  0, 0, PSW_ZF },
    { TYPE_NONE, CLASS_SETNE, "setne", 1,
        Eb, NULL_OPERANDS_2, NO_REGISTERS_ALL,
        0, 0,
        0, PSW_ZF },
    { TYPE_NONE, CLASS_SETBE, "setbe", 1,
        Eb, NULL_OPERANDS_2, NO_REGISTERS_ALL,
        0, 0,
        0, PSW_CF | PSW_ZF },
    { TYPE_NONE, CLASS_SETA, "seta", 1,
        Eb, NULL_OPERANDS_2, NO_REGISTERS_ALL,
        0, 0,
        0, PSW_CF | PSW_ZF },
    /* 98 */
    { TYPE_NONE, CLASS_SETS, "sets", 1,
        Eb, NULL_OPERANDS_2, NO_REGISTERS_ALL,
        0, 0,
        0, PSW_SF },
    { TYPE_NONE, CLASS_SETNS, "setns", 1,
        Eb, NULL_OPERANDS_2, NO_REGISTERS_ALL,
        0, 0,
        0, PSW_SF },
    { TYPE_NONE, CLASS_SETP, "setp", 1,
        Eb, NULL_OPERANDS_2, NO_REGISTERS_ALL,
        0, 0,
        0, PSW_PF },
    { TYPE_NONE, CLASS_SETNP, "setnp", 1,
        Eb, NULL_OPERANDS_2, NO_REGISTERS_ALL,
        0, 0,
        0, PSW_PF },
    { TYPE_NONE, CLASS_SETL, "setl", 1,
        Eb, NULL_OPERANDS_2, NO_REGISTERS_ALL,
        0, 0,
        0, PSW_OF | PSW_SF },
    { TYPE_NONE, CLASS_SETGE, "setge", 1,
        Eb, NULL_OPERANDS_2, NO_REGISTERS_ALL,
        0, 0,
        0, PSW_OF | PSW_SF },
    { TYPE_NONE, CLASS_SETLE, "setle", 1,
        Eb, NULL_OPERANDS_2, NO_REGISTERS_ALL,
        0, 0,
        0, PSW_ZF | PSW_OF | PSW_SF },
    { TYPE_NONE, CLASS_SETG, "setg", 1,
        Eb, NULL_OPERANDS_2, NO_REGISTERS_ALL,
        0, 0,
        0, PSW_ZF | PSW_OF | PSW_SF },
    /* a0 */
    { TYPE_NONE, CLASS_PUSHEFLAGS, "pushS", 4,
        NULL_OPERANDS_3, eSP_reg, no_reg, eSP_reg, no_reg,
        ADDRESS_SIZE, 0,
        0, 0 },
    { TYPE_NONE, CLASS_POPEFLAGS, "popS", COMPLEX,
        NULL_OPERANDS_3, eSP_reg, no_reg, eSP_reg, no_reg,
	0, ADDRESS_SIZE,
        0, 0 },
    { TYPE_NONE, CLASS_CPUID, "cpuid", COMPLEX,
        NULL_OPERANDS_3, all_regs, no_reg, eAX_reg, no_reg,
        0, 0,
        0, 0 },
    { TYPE_NONE, CLASS_MISC, "btS", COMPLEX,
        NULL_OPERANDS_1, Ev, Gv, NO_REGISTERS_ALL,
        0, 0, PSW_CF, 0 },
    { TYPE_NONE, CLASS_MISC, "shldS", 2,
        Ev, Gv, Ib, NO_REGISTERS_ALL,
        0, 0,
	PSW_AF | PSW_PF | PSW_CF | PSW_OF | PSW_ZF | PSW_SF | DUF | S1D,
        PSW_CF },
    { TYPE_NONE, CLASS_MISC, "shldS", 2,
        Ev, Gv, CL, NO_REGISTERS_ALL,
        0, 0,
	PSW_AF | PSW_PF | PSW_CF | PSW_OF | PSW_ZF | PSW_SF | DUF | S1D,
        PSW_CF },
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    /* a8 */
    { TYPE_NONE, CLASS_PUSH, "pushS", 4,
        NULL_OPERANDS_3, eSP_reg, no_reg, gs_reg, eSP_reg,
        ADDRESS_SIZE, 0,
        0, 0 },
    { TYPE_NONE, CLASS_POP, "popS", COMPLEX,
        NULL_OPERANDS_3, gs_reg, eSP_reg, eSP_reg, no_reg,
        0, ADDRESS_SIZE,
        0, 0 },
    { TYPE_NONE, CLASS_RSM, "rsm", COMPLEX,
        NULL_OPERANDS_3, all_regs, no_reg, all_regs, no_reg,
        0, 0,
        PSW_ALL, PSW_ALL },
    { TYPE_NONE, CLASS_BTS, "btsS", 1,
        Ev, Gv, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        PSW_CF, 0 },
    { TYPE_NONE, CLASS_SHRD, "shrdS", 2,
        Ev, Gv, Ib, NO_REGISTERS_ALL,
        0, 0,
	PSW_AF | PSW_PF | PSW_CF | PSW_OF | PSW_ZF | PSW_SF | DUF | S1D,
        PSW_CF },
    { TYPE_NONE, CLASS_SHRD, "shrdS", 2,
        Ev, Gv, CL, NO_REGISTERS_ALL,
        0, 0,
	PSW_AF | PSW_PF | PSW_CF | PSW_OF | PSW_ZF | PSW_SF | DUF | S1D,
        PSW_CF },
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    { TYPE_NONE, CLASS_IMUL, "imulS", 1,
        Gv, Ev, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        PSW_CF | PSW_OF | DUF, 0 },
    /* b0 */
    { TYPE_NONE, CLASS_CMPXCHG, "cmpxchgb", COMPLEX,
        Eb, Gb, NULL_OPERANDS_1, eAX_reg, no_reg, eAX_reg, no_reg,
	0, 0,
        PSW_ZF | PSW_CF | PSW_AF | PSW_PF | PSW_SF | PSW_OF | DUF, 0 },
    { TYPE_NONE, CLASS_CMPXCHG, "cmpxchgS", COMPLEX,
        Ev, Gv, NULL_OPERANDS_1, eAX_reg, no_reg, eAX_reg, no_reg,
	0, 0,
        PSW_ZF | PSW_CF | PSW_AF | PSW_PF | PSW_SF | PSW_OF | DUF, 0 },
    { TYPE_NONE, CLASS_LSS, "lssS", COMPLEX,
        Gv, Mp, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, 0 },
    { TYPE_NONE, CLASS_BTR, "btrS", 1,
        Ev, Gv, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        PSW_CF, 0 },
    { TYPE_NONE, CLASS_LFS, "lfsS", COMPLEX,
        Gv, Mp, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, 0 },
    { TYPE_NONE, CLASS_LGS, "lgsS", COMPLEX,
        Gv, Mp, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, 0 },
    { TYPE_NONE, CLASS_MOVZB, "movzbS", COMPLEX,
        Gv, Eb, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, 0 },
    { TYPE_NONE, CLASS_MOVZW, "movzwS", COMPLEX,
        Gv, Ew, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, 0 },
    /* b8 */
    { TYPE_NONE, CLASS_UD2B, "ud2b" },
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    { GRP8 },
    { TYPE_NONE, CLASS_BTC, "btcS", 1,
        Ev, Gv, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        PSW_CF | DUF, 0 },
    { TYPE_NONE, CLASS_BSF, "bsfS", 2,
        Gv, Ev, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        PSW_ZF, 0 },
    { TYPE_NONE, CLASS_BSR, "bsrS", 2,
        Gv, Ev, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        PSW_ZF, 0 },
    { TYPE_NONE, CLASS_MOVSB, "movsbS", COMPLEX,
        Gv, Eb, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, 0 },
    { TYPE_NONE, CLASS_MOVSW, "movswS", COMPLEX,
        Gv, Ew, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, 0 },
    /* c0 */
    { TYPE_NONE, CLASS_XADD, "xaddb", 4,
        Eb, Gb, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
	DUF | S1D | PSW_ZF | PSW_CF | PSW_AF | PSW_PF | PSW_SF | PSW_OF, 0 },
    { TYPE_NONE, CLASS_XADD, "xaddS", 4,
        Ev, Gv, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
	DUF | S1D | PSW_ZF | PSW_CF | PSW_AF | PSW_PF | PSW_SF | PSW_OF, 0 },
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    { GRP9 },
    /* c8 */
    { TYPE_NONE, CLASS_BSWAP, "bswap", 2,
        eAX, NULL_OPERANDS_2, no_reg, no_reg, eAX_reg, no_reg,
        0, 0,
        DUF, 0 },
    { TYPE_NONE, CLASS_BSWAP, "bswap", 2,
        eCX, NULL_OPERANDS_2, no_reg, no_reg, eCX_reg, no_reg,
        0, 0,
        DUF, 0 },
    { TYPE_NONE, CLASS_BSWAP, "bswap", 2,
        eDX, NULL_OPERANDS_2, no_reg, no_reg, eDX_reg, no_reg,
        0, 0,
        DUF, 0 },
    { TYPE_NONE, CLASS_BSWAP, "bswap", 2,
        eBX, NULL_OPERANDS_2, no_reg, no_reg, eBX_reg, no_reg,
        0, 0,
        DUF, 0 },
    { TYPE_NONE, CLASS_BSWAP, "bswap", 2,
        eSP, NULL_OPERANDS_2, no_reg, no_reg, eSP_reg, no_reg, 
        0, 0,
        DUF, 0 },
    { TYPE_NONE, CLASS_BSWAP, "bswap", 2,
        eBP, NULL_OPERANDS_2, no_reg, no_reg, eBP_reg, no_reg, 
        0, 0,
        DUF, 0 },
    { TYPE_NONE, CLASS_BSWAP, "bswap", 2,
        eSI, NULL_OPERANDS_2, no_reg, no_reg, eSI_reg, no_reg, 
        0, 0,
        DUF, 0 },
    { TYPE_NONE, CLASS_BSWAP, "bswap", 2,
        eDI, NULL_OPERANDS_2, no_reg, no_reg, eDI_reg, no_reg, 
        0, 0,
        DUF, 0 },
    /* d0 */
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    { TYPE_NONE, CLASS_PSRLW, "psrlw", COMPLEX,
        MX, EM },
    { TYPE_NONE, CLASS_PSRLD, "psrld", COMPLEX,
        MX, EM },
    { TYPE_NONE, CLASS_PSRLQ, "psrlq", COMPLEX,
        MX, EM },
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    { TYPE_NONE, CLASS_PMULLW, "pmullw", COMPLEX,
        MX, EM },
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    /* d8 */
    { TYPE_NONE, CLASS_NONE, "psubusb", COMPLEX,
        MX, EM },
    { TYPE_NONE, CLASS_NONE, "psubusw", COMPLEX,
        MX, EM },
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    { TYPE_NONE, CLASS_NONE, "pand", COMPLEX,
        MX, EM },
    { TYPE_NONE, CLASS_NONE, "paddusb", COMPLEX,
        MX, EM },
    { TYPE_NONE, CLASS_NONE, "paddusw", COMPLEX,
        MX, EM },
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    { TYPE_NONE, CLASS_NONE, "pandn", COMPLEX,
        MX, EM },
    /* e0 */
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    { TYPE_NONE, CLASS_NONE, "psraw", COMPLEX,
        MX, EM },
    { TYPE_NONE, CLASS_NONE, "psrad", COMPLEX,
        MX, EM },
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    { TYPE_NONE, CLASS_NONE, "pmulhw", COMPLEX,
        MX, EM },
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    /* e8 */
    { TYPE_NONE, CLASS_NONE, "psubsb", COMPLEX,
        MX, EM },
    { TYPE_NONE, CLASS_NONE, "psubsw", COMPLEX,
        MX, EM },
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    { TYPE_NONE, CLASS_NONE, "por", COMPLEX,
        MX, EM },
    { TYPE_NONE, CLASS_NONE, "paddsb", COMPLEX,
        MX, EM },
    { TYPE_NONE, CLASS_NONE, "paddsw", COMPLEX,
        MX, EM },
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    { TYPE_NONE, CLASS_NONE, "pxor", COMPLEX,
        MX, EM },
    /* f0 */
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    { TYPE_NONE, CLASS_NONE, "psllw", COMPLEX,
        MX, EM },
    { TYPE_NONE, CLASS_NONE, "pslld", COMPLEX,
        MX, EM },
    { TYPE_NONE, CLASS_NONE, "psllq", COMPLEX,
        MX, EM },
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    { TYPE_NONE, CLASS_NONE, "pmaddwd", COMPLEX,
        MX, EM },
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    /* f8 */
    { TYPE_NONE, CLASS_NONE, "psubb", COMPLEX,
        MX, EM },
    { TYPE_NONE, CLASS_NONE, "psubw", COMPLEX,
        MX, EM },
    { TYPE_NONE, CLASS_NONE, "psubd", COMPLEX,
        MX, EM },
    { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    { TYPE_NONE, CLASS_NONE, "paddb", COMPLEX,
        MX, EM },
    { TYPE_NONE, CLASS_NONE, "paddw", COMPLEX,
        MX, EM },
    { TYPE_NONE, CLASS_NONE, "paddd", COMPLEX,
        MX, EM },
    { TYPE_ERROR, CLASS_ERROR, "(bad)" }
};

X86op grps[][8] = {
    /* GRP1b */
    {
        { TYPE_NONE, CLASS_ADD, "addb", 4,
            Eb, Ib, NULL_OPERANDS_1, NO_REGISTERS_ALL,
            0, 0,
            PSW_OF | PSW_ZF | PSW_CF | PSW_AF | PSW_SF | PSW_PF | DUF, 0 },
        { TYPE_NONE, CLASS_OR, "orb", 4,
            Eb, Ib, NULL_OPERANDS_1, NO_REGISTERS_ALL,
            0, 0,
            PSW_OF | PSW_CF | PSW_SF | PSW_ZF | PSW_PF | DUF, 0 },
        { TYPE_NONE, CLASS_ADDC, "adcb", 4,
            Eb, Ib, NULL_OPERANDS_1, NO_REGISTERS_ALL,
            0, 0,
            PSW_CF | PSW_SF | PSW_ZF | PSW_AF | PSW_CF | PSW_PF | DUF, PSW_CF },
        { TYPE_NONE, CLASS_SUBB, "sbbb", 4,
            Eb, Ib, NULL_OPERANDS_1,  NO_REGISTERS_ALL,
            0, 0,
            PSW_CF | PSW_OF | PSW_PF | PSW_AF | PSW_SF | PSW_ZF | DUF, PSW_CF },
        { TYPE_NONE, CLASS_AND, "andb", 4,
            Eb, Ib, NULL_OPERANDS_1, NO_REGISTERS_ALL,
            0, 0,
            PSW_OF | PSW_CF | PSW_SF | PSW_ZF | PSW_PF | DUF, 0 },
        { TYPE_NONE, CLASS_SUB, "subb", 4,
            Eb, Ib, NULL_OPERANDS_1, NO_REGISTERS_ALL,
            0, 0,
            PSW_CF | PSW_OF | PSW_PF | PSW_AF | PSW_SF | PSW_ZF | DUF, PSW_CF },
        { TYPE_NONE, CLASS_XOR, "xorb", 4,
            Eb, Ib, NULL_OPERANDS_1, NO_REGISTERS_ALL,
            0, 0,
            PSW_OF | PSW_CF | PSW_SF | PSW_ZF | PSW_PF | DUF, 0 },
        { TYPE_NONE, CLASS_CMP, "cmpb", 1,
            NULL_OPERANDS_1, Eb, Ib,  NO_REGISTERS_ALL,
            0, 0,
            PSW_CF | PSW_OF | PSW_SF | PSW_ZF | PSW_AF | PSW_PF, 0 }
    },
    /* GRP1S */
    {
        { TYPE_NONE, CLASS_ADD, "addS", 4,
            Ev, Iv, NULL_OPERANDS_1, NO_REGISTERS_ALL,
            0, 0,
            PSW_OF | PSW_SF | PSW_ZF | PSW_ZF | PSW_CF | PSW_PF | DUF, 0 },
        { TYPE_NONE, CLASS_OR, "orS", 4,
            Ev, Iv, NULL_OPERANDS_1, NO_REGISTERS_ALL,
            0, 0,
            PSW_OF | PSW_CF | PSW_SF | PSW_ZF | PSW_PF | DUF, 0 },
        { TYPE_NONE, CLASS_ADDC, "adcS", 4,
            Ev, Iv, NULL_OPERANDS_1, NO_REGISTERS_ALL,
            0, 0,
            PSW_CF | PSW_SF | PSW_ZF | PSW_AF | PSW_CF | PSW_PF | DUF, PSW_CF },
        { TYPE_NONE, CLASS_SUBB, "sbbS", 4,
            Ev, Iv, NULL_OPERANDS_1, NO_REGISTERS_ALL,
            0, 0,
            PSW_CF | PSW_OF | PSW_PF | PSW_AF | PSW_SF | PSW_ZF | DUF, PSW_CF },
        { TYPE_NONE, CLASS_AND, "andS", 4,
            Ev, Iv, NULL_OPERANDS_1, NO_REGISTERS_ALL,
            0, 0,
            PSW_OF | PSW_CF | PSW_SF | PSW_ZF | PSW_PF | DUF, 0 },
        { TYPE_NONE, CLASS_SUB, "subS", 4,
            Ev, Iv, NULL_OPERANDS_1, NO_REGISTERS_ALL,
            0, 0,
            PSW_OF | PSW_SF | PSW_ZF | PSW_AF | PSW_PF | PSW_CF | DUF, 0 },
        { TYPE_NONE, CLASS_XOR, "xorS", 4,
            Ev, Iv, NULL_OPERANDS_1, NO_REGISTERS_ALL,
            0, 0,
            PSW_OF | PSW_CF | PSW_SF | PSW_ZF | PSW_PF | DUF },
        { TYPE_NONE, CLASS_CMP, "cmpS", 1,
            NULL_OPERANDS_1, Ev, Iv, NO_REGISTERS_ALL,
            0, 0,
            PSW_CF | PSW_OF | PSW_SF | PSW_ZF | PSW_AF | PSW_PF, 0 }
    },
    /* GRP1Ss */
    {
        { TYPE_NONE, CLASS_ADD, "addS", 4,
            Ev, sIb, NULL_OPERANDS_1, NO_REGISTERS_ALL,
            0, 0,
            PSW_OF | PSW_SF | PSW_ZF | PSW_ZF | PSW_CF | PSW_PF | DUF, 0 },
        { TYPE_NONE, CLASS_OR, "orS", 4,
            Ev, sIb, NULL_OPERANDS_1, NO_REGISTERS_ALL,
            0, 0,
            PSW_OF | PSW_CF | PSW_SF | PSW_ZF | PSW_PF | DUF, 0 },
        { TYPE_NONE, CLASS_ADDC, "adcS", 4,
            Ev, sIb, NULL_OPERANDS_1, NO_REGISTERS_ALL,
            0, 0,
            PSW_CF | PSW_SF | PSW_ZF | PSW_AF | PSW_CF | PSW_PF | DUF, PSW_CF },
        { TYPE_NONE, CLASS_SUBB, "sbbS", 4,
            Ev, sIb, NULL_OPERANDS_1, NO_REGISTERS_ALL,
            0, 0,
            PSW_CF | PSW_OF | PSW_PF | PSW_AF | PSW_SF | PSW_ZF | DUF, PSW_CF },
        { TYPE_NONE, CLASS_AND, "andS", 4,
            Ev, sIb, NULL_OPERANDS_1, NO_REGISTERS_ALL,
            0, 0,
            PSW_OF | PSW_CF | PSW_SF | PSW_ZF | PSW_PF | DUF, 0 },
        { TYPE_NONE, CLASS_SUB, "subS", 4,
            Ev, sIb, NULL_OPERANDS_1, NO_REGISTERS_ALL,
            0, 0,
            PSW_OF | PSW_SF | PSW_ZF | PSW_AF | PSW_PF | PSW_CF | DUF, 0 },
        { TYPE_NONE, CLASS_XOR, "xorS", 4,
            Ev, sIb, NULL_OPERANDS_1, NO_REGISTERS_ALL,
            0, 0,
            PSW_OF | PSW_CF | PSW_SF | PSW_ZF | PSW_PF | DUF },
        { TYPE_NONE, CLASS_CMP, "cmpS", 2,
            NULL_OPERANDS_1, Ev, sIb, NO_REGISTERS_ALL,
            0, 0,
            PSW_CF | PSW_OF | PSW_SF | PSW_ZF | PSW_AF | PSW_PF, 0 }
    },
    /* GRP2b */
    {
        { TYPE_NONE, CLASS_ROL, "rolb", 1,
            Eb, Ib, NULL_OPERANDS_1, NO_REGISTERS_ALL,
            0, 0, 
            PSW_OF | DUF, 0 },
        { TYPE_NONE, CLASS_ROR, "rorb", 1,
            Eb, Ib, NULL_OPERANDS_1, NO_REGISTERS_ALL,
            0, 0, 
            PSW_OF | DUF, 0 },
        { TYPE_NONE, CLASS_RCL, "rclb", COMPLEX,
            Eb, Ib, NULL_OPERANDS_1, NO_REGISTERS_ALL,
            0, 0, 
            PSW_OF | PSW_CF | DUF, PSW_CF },
        { TYPE_NONE, CLASS_RCR, "rcrb", COMPLEX,
            Eb, Ib, NULL_OPERANDS_1, NO_REGISTERS_ALL,
            0, 0, 
            PSW_OF | PSW_CF | DUF, PSW_CF },
        { TYPE_NONE, CLASS_SHLB, "shlb", 1,
            Eb, Ib, NULL_OPERANDS_1, NO_REGISTERS_ALL,
            0, 0,
            PSW_CF | PSW_OF | PSW_SF | PSW_ZF | PSW_PF | PSW_AF | DUF, PSW_CF },
        { TYPE_NONE, CLASS_SHR, "shrb", 1,
            Eb, Ib, NULL_OPERANDS_1, NO_REGISTERS_ALL,
            0, 0,
            PSW_CF | PSW_OF | PSW_SF | PSW_ZF | PSW_PF | PSW_AF | DUF, PSW_CF },
        { TYPE_ERROR, CLASS_ERROR, "(bad)" },
        { TYPE_NONE, CLASS_SAR, "sarb", 1,
            Eb, Ib, NULL_OPERANDS_1, NO_REGISTERS_ALL,
            0, 0,
            PSW_CF | PSW_OF | PSW_SF | PSW_ZF | PSW_PF | PSW_AF | DUF, PSW_CF}
    },
    /* GRP2S */
    {
        { TYPE_NONE, CLASS_ROL, "rolS", 1,
            Ev, Ib, NULL_OPERANDS_1, NO_REGISTERS_ALL,
            0, 0,
            PSW_OF | DUF, 0 },
        { TYPE_NONE, CLASS_ROR, "rorS", 1,
            Ev, Ib, NULL_OPERANDS_1, NO_REGISTERS_ALL,
            0, 0,
            PSW_OF | DUF, 0 },
        { TYPE_NONE, CLASS_RCL, "rclS", COMPLEX,
            Ev, Ib, NULL_OPERANDS_1, NO_REGISTERS_ALL,
            0, 0,
            PSW_OF | PSW_CF | DUF, PSW_CF },
        { TYPE_NONE, CLASS_RCR, "rcrS", COMPLEX,
            Ev, Ib, NULL_OPERANDS_1, NO_REGISTERS_ALL,
            0, 0,
            PSW_OF | PSW_CF | DUF, PSW_CF },
        { TYPE_NONE, CLASS_SHL, "shlS", 1,
            Ev, Ib, NULL_OPERANDS_1, NO_REGISTERS_ALL,
            0, 0,
            PSW_CF | PSW_OF | PSW_SF | PSW_ZF | PSW_PF | PSW_AF | DUF, PSW_CF },
        { TYPE_NONE, CLASS_SHR, "shrS", 1,
            Ev, Ib, NULL_OPERANDS_1, NO_REGISTERS_ALL,
            0, 0,
            PSW_CF | PSW_OF | PSW_SF | PSW_ZF | PSW_PF | PSW_AF | DUF, PSW_CF },
        { TYPE_ERROR, CLASS_ERROR, "(bad)" },
        { TYPE_NONE, CLASS_SAR, "sarS", 1,
            Ev, Ib, NULL_OPERANDS_1, NO_REGISTERS_ALL,
            0, 0,
            PSW_CF | PSW_OF | PSW_SF | PSW_ZF | PSW_PF | PSW_AF | DUF, PSW_CF },
    },
    /* GRP2b_one */
    {
        { TYPE_NONE, CLASS_ROL, "rolb", 4,
            Eb, NULL_OPERANDS_2, NO_REGISTERS_ALL,
            0, 0,
            PSW_OF | DUF, 0 },
        { TYPE_NONE, CLASS_ROR, "rorb", 4,
            Eb, NULL_OPERANDS_2, NO_REGISTERS_ALL,
            0, 0,
            PSW_OF | DUF, 0 },
        { TYPE_NONE, CLASS_RCL, "rclb", COMPLEX,
            Eb, NULL_OPERANDS_2, NO_REGISTERS_ALL,
            0, 0,
            PSW_OF | DUF | PSW_CF, PSW_CF },
        { TYPE_NONE, CLASS_RCR, "rcrb", COMPLEX,
            Eb, NULL_OPERANDS_2, NO_REGISTERS_ALL,
            0, 0,
            PSW_OF | DUF | PSW_CF, PSW_CF },
        { TYPE_NONE, CLASS_SHLB, "shlb", 4,
            Eb, NULL_OPERANDS_2, NO_REGISTERS_ALL,
            0, 0,
            PSW_CF | PSW_OF | PSW_SF | PSW_ZF | PSW_PF | PSW_AF | DUF, PSW_CF },
        { TYPE_NONE, CLASS_SHR, "shrb", 4,
            Eb, NULL_OPERANDS_2, NO_REGISTERS_ALL,
            0, 0,
            PSW_CF | PSW_OF | PSW_SF | PSW_ZF | PSW_PF | PSW_AF | DUF, PSW_CF },
        { TYPE_ERROR, CLASS_ERROR, "(bad)" },
        { TYPE_NONE, CLASS_SAR, "sarb", 4,
            Eb, NULL_OPERANDS_2, NO_REGISTERS_ALL,
            0, 0,
            PSW_CF | PSW_OF | PSW_SF | PSW_ZF | PSW_PF | PSW_AF | DUF, PSW_CF },
    },
    /* GRP2S_one */
    {
        { TYPE_NONE, CLASS_ROL, "rolS", 4,
            Ev, NULL_OPERANDS_2, NO_REGISTERS_ALL,
            0, 0,
            PSW_OF | DUF, 0 },
        { TYPE_NONE, CLASS_ROR, "rorS", 4,
            Ev, NULL_OPERANDS_2, NO_REGISTERS_ALL,
            0, 0,
            PSW_OF | DUF, 0 },
        { TYPE_NONE, CLASS_RCL, "rclS", COMPLEX,
            Ev, NULL_OPERANDS_2, NO_REGISTERS_ALL,
            0, 0,
            PSW_OF | DUF | PSW_CF, PSW_CF },
        { TYPE_NONE, CLASS_RCR, "rcrS", COMPLEX,
            Ev, NULL_OPERANDS_2, NO_REGISTERS_ALL,
            0, 0,
            PSW_OF | DUF | PSW_CF, PSW_CF },
        { TYPE_NONE, CLASS_SHL, "shlS", 4,
            Ev, NULL_OPERANDS_2, NO_REGISTERS_ALL,
            0, 0,
            PSW_CF | PSW_OF | PSW_SF | PSW_ZF | PSW_PF | PSW_AF | DUF, PSW_CF },
        { TYPE_NONE, CLASS_SHR, "shrS", 4,
            Ev, NULL_OPERANDS_2, NO_REGISTERS_ALL,
            0, 0,
            PSW_CF | PSW_OF | PSW_SF | PSW_ZF | PSW_PF | PSW_AF | DUF, PSW_CF },
        { TYPE_ERROR, CLASS_ERROR, "(bad)" },
        { TYPE_NONE, CLASS_SAR, "sarS", 4,
            Ev, NULL_OPERANDS_2,
            NO_REGISTERS_ALL,
            0, 0,
            PSW_CF | PSW_OF | PSW_SF | PSW_ZF | PSW_PF | PSW_AF | DUF, PSW_CF },
    },
    /* GRP2b_cl */
    {
        { TYPE_NONE, CLASS_ROL, "rolb", 1,
            Eb, CL, NULL_OPERANDS_1, NO_REGISTERS_ALL,
            0, 0,
            PSW_OF | DUF, 0 },
        { TYPE_NONE, CLASS_ROR, "rorb", 1,
            Eb, CL, NULL_OPERANDS_1, NO_REGISTERS_ALL,
            0, 0,
            PSW_OF | DUF, 0 },
        { TYPE_NONE, CLASS_RCL, "rclb", COMPLEX,
            Eb, CL, NULL_OPERANDS_1,  NO_REGISTERS_ALL,
            0, 0,
            PSW_OF | PSW_CF | DUF, PSW_CF },
        { TYPE_NONE, CLASS_RCR, "rcrb", COMPLEX,
            Eb, CL, NULL_OPERANDS_1, NO_REGISTERS_ALL,
            0, 0,
            PSW_OF | PSW_CF | DUF, PSW_CF },
        { TYPE_NONE, CLASS_SHLB, "shlb", 1,
            Eb, CL, NULL_OPERANDS_1, NO_REGISTERS_ALL,
            0, 0,
            PSW_CF | PSW_OF | PSW_SF | PSW_ZF | PSW_PF | PSW_AF | DUF, PSW_CF },
        { TYPE_NONE, CLASS_SHR, "shrb", 1,
            Eb, CL, NULL_OPERANDS_1, NO_REGISTERS_ALL,
            0, 0,
            PSW_CF | PSW_OF | PSW_SF | PSW_ZF | PSW_PF | PSW_AF | DUF, PSW_CF },
        { TYPE_ERROR, CLASS_ERROR, "(bad)" },
        { TYPE_NONE, CLASS_SAR, "sarb", 1,
            Eb, CL, NULL_OPERANDS_1, NO_REGISTERS_ALL,
            0, 0,
            PSW_CF | PSW_OF | PSW_SF | PSW_ZF | PSW_PF | PSW_AF | DUF, PSW_CF}
    },
    /* GRP2S_cl */
    {
        { TYPE_NONE, CLASS_ROL, "rolS", 1,
            Ev, CL, CL, NO_REGISTERS_ALL,
            0, 0, 
            PSW_OF | DUF, 0 },
        { TYPE_NONE, CLASS_ROR, "rorS", 1,
            Ev, CL, CL, NO_REGISTERS_ALL,
            0, 0, 
            PSW_OF | DUF, 0 },
        { TYPE_NONE, CLASS_RCL, "rclS", COMPLEX,
            Ev, CL, NULL_OPERANDS_1, NO_REGISTERS_ALL,
            0, 0, 
            PSW_OF | PSW_CF | DUF, PSW_CF },
        { TYPE_NONE, CLASS_RCR, "rcrS", COMPLEX,
            Ev, CL, NULL_OPERANDS_1, NO_REGISTERS_ALL,
            0, 0, 
            PSW_OF | PSW_CF | DUF, PSW_CF },
        { TYPE_NONE, CLASS_SHL, "shlS", 1,
            Ev, CL, NULL_OPERANDS_1, NO_REGISTERS_ALL,
            0, 0, 
            PSW_CF | PSW_OF | PSW_SF | PSW_ZF | PSW_PF | PSW_AF | DUF, PSW_CF },
        { TYPE_NONE, CLASS_SHR, "shrS", 1,
            Ev, CL, NULL_OPERANDS_1, NO_REGISTERS_ALL,
            0, 0, 
            PSW_CF | PSW_OF | PSW_SF | PSW_ZF | PSW_PF | PSW_AF | DUF, PSW_CF },
        { TYPE_ERROR, CLASS_ERROR, "(bad)" },
        { TYPE_NONE, CLASS_SAR, "sarS", 1,
            Ev, CL, NULL_OPERANDS_1, NO_REGISTERS_ALL,
            0, 0, 
            PSW_CF | PSW_OF | PSW_SF | PSW_ZF | PSW_PF | PSW_AF | DUF, PSW_CF }
    },
    /* GRP3b */
    {
        { TYPE_NONE, CLASS_TEST, "testb", 1,
            NULL_OPERANDS_1, Eb, Ib, NO_REGISTERS_ALL,
            0, 0,
            PSW_OF | PSW_CF | PSW_SF | PSW_ZF | PSW_PF, 0 },
        { TYPE_ERROR, CLASS_ERROR, "(bad)", COMPLEX,
            Eb },
        { TYPE_NONE, CLASS_NOT, "notb", 1,
            Eb, NULL_OPERANDS_2, NO_REGISTERS_ALL,
            0, 0,
            DUF, 0 },
        { TYPE_NONE, CLASS_NEG, "negb", 1,
            Eb, NULL_OPERANDS_2, NO_REGISTERS_ALL,
            0, 0,
            PSW_CF | PSW_OF | PSW_AF | PSW_SF | PSW_ZF | PSW_PF | DUF, 0 },
        { TYPE_NONE, CLASS_MUL, "mulb", 4,
            NULL_OPERANDS_1, AL, Eb, ax_reg, no_reg, no_reg, no_reg,
            0, 0,
            PSW_CF | PSW_OF, 0 },
        { TYPE_NONE, CLASS_IMUL, "imulb", 4,
            AL, Eb, NULL_OPERANDS_1, ax_reg, no_reg, no_reg, no_reg,
            0, 0,
            PSW_CF | PSW_OF | DUF, 0 },
        { TYPE_NONE, CLASS_DIV, "divb", 3,
            AL, Eb, NULL_OPERANDS_1, al_reg, ah_reg, ax_reg, no_reg,
            0, 0,
            DUF, 0 },
        { TYPE_NONE, CLASS_IDIV, "idivb", 3,
            AL, Eb, NULL_OPERANDS_1, al_reg, ah_reg, ax_reg, no_reg,
            0, 0,
            DUF, 0 }
    },
    /* GRP3S */
    {
        { TYPE_NONE, CLASS_TEST, "testS", 1,
            NULL_OPERANDS_1, Ev, Iv, NO_REGISTERS_ALL,
            0, 0,
            PSW_CF | PSW_OF | PSW_PF | PSW_SF | PSW_ZF, 0 },
        { TYPE_ERROR, CLASS_ERROR, "(bad)" },
        { TYPE_NONE, CLASS_NOT, "notS", 1,
            Ev, NULL_OPERANDS_2, NO_REGISTERS_ALL,
            0, 0,
            DUF, 0 },
        { TYPE_NONE, CLASS_NEG, "negS", 1,
            Ev, NULL_OPERANDS_2, NO_REGISTERS_ALL,
            0, 0,
            PSW_CF | PSW_OF | PSW_AF | PSW_SF | PSW_ZF | PSW_PF | DUF, 0 },
        { TYPE_NONE, CLASS_MUL, "mulS", 4,
            NULL_OPERANDS_1, eAX, Ev, eDX_reg, eAX_reg, no_reg, no_reg,
            0, 0,
            PSW_CF | PSW_OF, 0 },
        { TYPE_NONE, CLASS_IMUL, "imulS", 1,
            eAX, Ev, NULL_OPERANDS_1, eDX_reg, no_reg, no_reg, no_reg,
            0, 0,
            PSW_CF | PSW_OF | DUF, 0 },
        { TYPE_NONE, CLASS_DIV, "divS", 4,
            eAX, Ev, NULL_OPERANDS_1, eDX_reg, eAX_reg, eDX_reg, eAX_reg,
            0, 0,
            DUF, 0 },
        { TYPE_NONE, CLASS_IDIV, "idivS", 4,
            eAX, Ev, NULL_OPERANDS_1, eDX_reg, eAX_reg, eDX_reg, eAX_reg,
            0, 0,
            DUF, 0 },
    },
    /* GRP4 */
    {
        { TYPE_NONE, CLASS_INC, "incb", 1,
            Eb, NULL_OPERANDS_2, NO_REGISTERS_ALL,
            0, 0,
            PSW_OF | PSW_SF | PSW_ZF | PSW_AF | PSW_PF | DUF, 0 },
        { TYPE_NONE, CLASS_DEC, "decb", 1,
            Eb, NULL_OPERANDS_2, NO_REGISTERS_ALL,
            0, 0,
            PSW_ZF | PSW_SF | PSW_AF | PSW_PF | PSW_OF | DUF, 0 },
        { TYPE_ERROR, CLASS_ERROR, "(bad)" },
        { TYPE_ERROR, CLASS_ERROR, "(bad)" },
        { TYPE_ERROR, CLASS_ERROR, "(bad)" },
        { TYPE_ERROR, CLASS_ERROR, "(bad)" },
        { TYPE_ERROR, CLASS_ERROR, "(bad)" },
        { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    },
    /* GRP5 */
    {
        { TYPE_NONE, CLASS_INCL, "incS", 1,
            Ev, NULL_OPERANDS_2, NO_REGISTERS_ALL,
            0, 0,
            PSW_OF | PSW_SF | PSW_ZF | PSW_AF | PSW_PF | DUF, 0 },
        { TYPE_NONE, CLASS_DEC, "decS", 1,
            Ev, NULL_OPERANDS_2, NO_REGISTERS_ALL,
            0, 0,
            PSW_OF | PSW_SF | PSW_ZF | PSW_AF | PSW_PF | DUF, 0 },
        { TYPE_UNCONDITIONAL, CLASS_CALL, "call", COMPLEX,
            NULL_OPERANDS_1, indirEv, NULL_OPERANDS_1, NO_REGISTERS_ALL,
            ADDRESS_SIZE, 0,
            PSW_ALL, 0 },
        { TYPE_UNCONDITIONAL, CLASS_LCALL, "lcall", COMPLEX,
            NULL_OPERANDS_1, indirEv, NULL_OPERANDS_1, NO_REGISTERS_ALL,
            ADDRESS_SIZE, 0,
            PSW_ALL, 0 },
        { TYPE_UNCONDITIONAL, CLASS_JUMP, "jmp", COMPLEX,
            NULL_OPERANDS_1, indirEv, NULL_OPERANDS_1, NO_REGISTERS_ALL,
            0, 0,
            0, 0 },
        { TYPE_UNCONDITIONAL, CLASS_LJUMP, "ljmp", COMPLEX,
            NULL_OPERANDS_1, indirEv, NULL_OPERANDS_1, NO_REGISTERS_ALL,
            0, 0,
            PSW_ALL, 0 },
        { TYPE_NONE, CLASS_PUSH, "pushS", 4,
            NULL_OPERANDS_1, Ev, NULL_OPERANDS_1, eSP_reg, no_reg, eSP_reg, no_reg,
            ADDRESS_SIZE, 0,
            0, 0 },
        { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    },
    /* GRP6 */
    {
        { TYPE_NONE, CLASS_SLDT, "sldt", 4,
            Ew, NULL_OPERANDS_2, NO_REGISTERS_ALL,
            0, 0,
            0, 0 },
        { TYPE_NONE, CLASS_STR, "str", 4,
            Ew, NULL_OPERANDS_2, NO_REGISTERS_ALL,
            0, 0,
            0, 0 },
        { TYPE_NONE, CLASS_LLDT, "lldt", COMPLEX,
            NULL_OPERANDS_1, Ew, NULL_OPERANDS_1, NO_REGISTERS_ALL,
            0, 0,
            0, 0 },
        { TYPE_NONE, CLASS_LTR, "ltr", COMPLEX,
            NULL_OPERANDS_1, Ew, NULL_OPERANDS_1, NO_REGISTERS_ALL,
            0, 0,
            0, 0 },
        { TYPE_NONE, CLASS_VERR, "verr", COMPLEX,
            NULL_OPERANDS_1, Ew, NULL_OPERANDS_1, NO_REGISTERS_ALL,
            0, 0,
            PSW_ZF, 0 },
        { TYPE_NONE, CLASS_VERW, "verw", COMPLEX,
            NULL_OPERANDS_1, Ew, NULL_OPERANDS_1, NO_REGISTERS_ALL,
            0, 0,
            PSW_ZF, 0 },
        { TYPE_ERROR, CLASS_ERROR, "(bad)" },
        { TYPE_ERROR, CLASS_ERROR, "(bad)" }
    },
    /* GRP7 */
    {
        { TYPE_NONE, CLASS_SGDT, "sgdt", 4,
            Ew, NULL_OPERANDS_2, NO_REGISTERS_ALL,
            0, 0,
            0, 0 },
        { TYPE_NONE, CLASS_SIDT, "sidt", COMPLEX,
            Ew, NULL_OPERANDS_2, NO_REGISTERS_ALL,
            0, 0,
            0, 0 },
        { TYPE_NONE, CLASS_LGDT, "lgdt", COMPLEX,
            NULL_OPERANDS_1, Ed, NULL_OPERANDS_1, NO_REGISTERS_ALL,
            0, 0,
            0, 0 },
        { TYPE_NONE, CLASS_LIDT, "lidt", COMPLEX,
            NULL_OPERANDS_1, Ed, NULL_OPERANDS_1, NO_REGISTERS_ALL,
            0, 0,
            0, 0 },
        { TYPE_NONE, CLASS_SMSW, "smsw", 4,
            Ew, NULL_OPERANDS_2, NO_REGISTERS_ALL,
            0, 0,
            0, 0 },
        { TYPE_ERROR, CLASS_ERROR, "(bad)" },
        { TYPE_NONE, CLASS_LMSW, "lmsw", COMPLEX,
            NULL_OPERANDS_1, Ew, NULL_OPERANDS_1, NO_REGISTERS_ALL,
            0, 0,
            0, 0 },
        { TYPE_NONE, CLASS_INVLPG, "invlpg", COMPLEX,
            NULL_OPERANDS_2, Ew, NO_REGISTERS_ALL,
            0, 0,
            0, 0 },
    },
    /* GRP8 */
    {
        { TYPE_ERROR, CLASS_ERROR, "(bad)" },
        { TYPE_ERROR, CLASS_ERROR, "(bad)" },
        { TYPE_ERROR, CLASS_ERROR, "(bad)" },
        { TYPE_ERROR, CLASS_ERROR, "(bad)" },
        { TYPE_NONE, CLASS_BT, "btS", COMPLEX,
            NULL_OPERANDS_1, Ev, Ib, NO_REGISTERS_ALL,
            0, 0,
            PSW_CF, 0 },
        { TYPE_NONE, CLASS_BTS, "btsS", 1,
            Ev, Ib, NULL_OPERANDS_1, NO_REGISTERS_ALL,
            0, 0,
            PSW_CF | DUF, 0 },
        { TYPE_NONE, CLASS_BTR, "btrS", 1,
            Ev, Ib, NULL_OPERANDS_1, NO_REGISTERS_ALL,
            0, 0,
            PSW_CF | DUF, 0 },
        { TYPE_NONE, CLASS_BTC, "btcS", 1,
            Ev, Ib, NULL_OPERANDS_1, NO_REGISTERS_ALL,
            0, 0,
            PSW_CF | DUF, 0 },
    },
    /* GRP9 */
    {
        { TYPE_ERROR, CLASS_ERROR, "(bad)" },
        { TYPE_NONE, CLASS_CMPXCHG8B, "cmpxchg8b", COMPLEX,
            Ev, NULL_OPERANDS_2, eAX_reg, no_reg, eAX_reg, no_reg,
            0, 0,
            PSW_ZF | DUF, 0 },
        { TYPE_ERROR, CLASS_ERROR, "(bad)" },
        { TYPE_ERROR, CLASS_ERROR, "(bad)" },
        { TYPE_ERROR, CLASS_ERROR, "(bad)" },
        { TYPE_ERROR, CLASS_ERROR, "(bad)" },
        { TYPE_ERROR, CLASS_ERROR, "(bad)" },
        { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    },
    /* GRP10 */
    {
        { TYPE_ERROR, CLASS_ERROR, "(bad)" },
        { TYPE_ERROR, CLASS_ERROR, "(bad)" },
        { TYPE_NONE, CLASS_PSRLW, "psrlw", COMPLEX,
            MS, Ib },
        { TYPE_ERROR, CLASS_ERROR, "(bad)" },
        { TYPE_NONE, CLASS_PSRAW, "psraw", COMPLEX,
            MS, Ib },
        { TYPE_ERROR, CLASS_ERROR, "(bad)" },
        { TYPE_NONE, CLASS_PSLLW, "psllw", COMPLEX,
            MS, Ib },
        { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    },
    /* GRP11 */
    {
        { TYPE_ERROR, CLASS_ERROR, "(bad)" },
        { TYPE_ERROR, CLASS_ERROR, "(bad)" },
        { TYPE_NONE, CLASS_PSRLD, "psrld", COMPLEX,
            MS, Ib },
        { TYPE_ERROR, CLASS_ERROR, "(bad)" },
        { TYPE_NONE, CLASS_PSRAD, "psrad", COMPLEX,
            MS, Ib },
        { TYPE_ERROR, CLASS_ERROR, "(bad)" },
        { TYPE_NONE, CLASS_PSLLD, "pslld", COMPLEX,
            MS, Ib },
        { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    },
    /* GRP12 */
    {
        { TYPE_ERROR, CLASS_ERROR, "(bad)" },
        { TYPE_ERROR, CLASS_ERROR, "(bad)" },
        { TYPE_NONE, CLASS_PSRLQ, "psrlq", COMPLEX,
            MS, Ib },
        { TYPE_ERROR, CLASS_ERROR, "(bad)" },
        { TYPE_ERROR, CLASS_ERROR, "(bad)" },
        { TYPE_ERROR, CLASS_ERROR, "(bad)" },
        { TYPE_NONE, CLASS_PSLLQ, "psllq", COMPLEX,
            MS, Ib },
        { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    }
};


X86op float_mem[] = {
    /* d8 */
    { TYPE_NONE, CLASS_FADDS, "fadds", 2,
        NULL_OPERANDS_1, Ev, NULL_OPERANDS_1, st0_reg, no_reg,	st0_reg, no_reg,
        0, 0,
        FPU_C1, 0 },
    { TYPE_NONE, CLASS_FMULS, "fmuls", 2,
        NULL_OPERANDS_1, Ev, NULL_OPERANDS_1, st0_reg, no_reg,	st0_reg, no_reg,
        0, 0,
        FPU_C1, 0 },
    { TYPE_NONE, CLASS_FCOMS, "fcoms", 2,
        NULL_OPERANDS_1, Ev, NULL_OPERANDS_1, no_reg, no_reg, st0_reg,	no_reg,
        0, 0,
        FPU_C0 | FPU_C2 | FPU_C3, 0 },
    { TYPE_NONE, CLASS_FCOMPS, "fcomps", 2,
        NULL_OPERANDS_1, Ev, NULL_OPERANDS_1, no_reg, no_reg, st0_reg,	no_reg,
        0, 0,
        FPU_C0 | FPU_C2 | FPU_C3, 0 },
    { TYPE_NONE, CLASS_FSUBS, "fsubs", 2,
        NULL_OPERANDS_1, Ev, NULL_OPERANDS_1, st0_reg, no_reg, st0_reg, no_reg,
        0, 0,
        FPU_C1, 0 },
    { TYPE_NONE, CLASS_FSUBRS, "fsubrs", 2,
        NULL_OPERANDS_1, Ev, NULL_OPERANDS_1, st0_reg, no_reg, st0_reg, no_reg,
        0, 0,
        FPU_C1, 0 },
    { TYPE_NONE, CLASS_FDIVS, "fdivs", 2,
        NULL_OPERANDS_1, Ev, NULL_OPERANDS_1, st0_reg, no_reg, st0_reg, no_reg,
        0, 0,
        FPU_C1, 0 },
    { TYPE_NONE, CLASS_FDIVRS, "fdivrs", 2,
        NULL_OPERANDS_1, Ev, NULL_OPERANDS_1, st0_reg, no_reg, st0_reg, no_reg,
        0, 0,
        FPU_C1, 0 },
    /*  d9 */
    { TYPE_NONE, CLASS_FLDS, "flds", 4,
        NULL_OPERANDS_1, Ev, NULL_OPERANDS_1, st0_reg, no_reg, no_reg,	no_reg,
        0, 0,
        FPU_C1, 0 },
    { TYPE_ERROR, CLASS_ERROR, "(bad)", COMPLEX,
        Ev },
    { TYPE_NONE, CLASS_FSTS, "fsts", 2,
        Ev, NULL_OPERANDS_2, no_reg, no_reg, st0_reg, no_reg,
        0, 0,
        FPU_C1, 0 },
    { TYPE_NONE, CLASS_FSTPS, "fstps", COMPLEX,
        Ev, NULL_OPERANDS_2, no_reg, no_reg, st0_reg, no_reg,
        0, 0,
        FPU_C1, 0 },
    { TYPE_NONE, CLASS_FLDENV, "fldenv", COMPLEX,
        NULL_OPERANDS_1, Ev, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
	FPU_C0 | FPU_C1 | FPU_C2 | FPU_C3, 0 },
    { TYPE_NONE, CLASS_FLDCW, "fldcw", 3,
        NULL_OPERANDS_1, Ev, NULL_OPERANDS_1, NO_REGISTERS_ALL,
        0, 0,
        0, 0 },
    { TYPE_NONE, CLASS_FNSTENV, "fNstenv", COMPLEX,
        Ev, NULL_OPERANDS_2, NO_REGISTERS_ALL,
        0, 0,
        0, 0 },
    { TYPE_NONE, CLASS_FNSTCW, "fNstcw", 3,
        Ev, NULL_OPERANDS_2, NO_REGISTERS_ALL,
        0, 0,
        0, 0 },
    /* da */
    { TYPE_NONE, CLASS_FIADDL, "fiaddl", COMPLEX,
        NULL_OPERANDS_1, Ev, NULL_OPERANDS_1, st0_reg, no_reg, st0_reg, no_reg,
        0, 0,
        FPU_C1, 0 },
    { TYPE_NONE, CLASS_FIMULL, "fimull", COMPLEX,
        NULL_OPERANDS_1, Ev, NULL_OPERANDS_1, st0_reg, no_reg, st0_reg, no_reg,
        0, 0,
        FPU_C1, 0 },
    { TYPE_NONE, CLASS_FICOML, "ficoml", COMPLEX,
        NULL_OPERANDS_1, Ev, NULL_OPERANDS_1, no_reg, no_reg, st0_reg,	no_reg,
        0, 0,
        FPU_C0 | FPU_C2 | FPU_C3, 0 },
    { TYPE_NONE, CLASS_FICOMPL, "ficompl", COMPLEX,
        NULL_OPERANDS_1, Ev, NULL_OPERANDS_1, no_reg, no_reg, st0_reg,	no_reg,
        0, 0,
        FPU_C0 | FPU_C2 | FPU_C3, 0 },
    { TYPE_NONE, CLASS_FISUBL, "fisubl", COMPLEX,
        NULL_OPERANDS_1, Ev, NULL_OPERANDS_1, st0_reg, no_reg, st0_reg, no_reg,
        0, 0,
        FPU_C1, 0 },
    { TYPE_NONE, CLASS_FISUBRL, "fisubrl", COMPLEX,
        NULL_OPERANDS_1, Ev, NULL_OPERANDS_1, st0_reg, no_reg, st0_reg, no_reg, 
        0, 0,
        FPU_C1, 0 },
    { TYPE_NONE, CLASS_FIDIVL, "fidivl", COMPLEX,
        NULL_OPERANDS_1, Ev, NULL_OPERANDS_1, st0_reg, no_reg, st0_reg, no_reg, 
        0, 0,
        FPU_C1, 0 },
    { TYPE_NONE, CLASS_FIDIVRL, "fidivrl", COMPLEX,
        NULL_OPERANDS_1, Ev, NULL_OPERANDS_1, st0_reg, no_reg, st0_reg, no_reg, 
        0, 0,
        FPU_C1, 0 },
    /* db */
    { TYPE_NONE, CLASS_FILDL, "fildl", COMPLEX,
        NULL_OPERANDS_1, Ev, NULL_OPERANDS_1, st0_reg, no_reg, no_reg,	no_reg, 
        0, 0,
        FPU_C1, 0 },
    { TYPE_ERROR, CLASS_ERROR, "(bad)", COMPLEX,
        Ev },
    { TYPE_NONE, CLASS_FISTL, "fistl", COMPLEX,
        Ev, NULL_OPERANDS_2, no_reg, no_reg, st0_reg, no_reg, 
        0, 0,
        FPU_C1, 0 },
    { TYPE_NONE, CLASS_FISTPL, "fistpl", COMPLEX,
        Ev, NULL_OPERANDS_2, no_reg, no_reg, st0_reg, no_reg, 
        0, 0,
        FPU_C1, 0 },
    { TYPE_ERROR, CLASS_ERROR, "(bad)", COMPLEX,
        Ev },
    { TYPE_NONE, CLASS_FLDT, "fldt", COMPLEX,
        NULL_OPERANDS_1, Ev, NULL_OPERANDS_1, st0_reg, no_reg, no_reg,	no_reg, 
        0, 0,
        FPU_C1, 0 },
    { TYPE_NONE, CLASS_ERROR, "(bad)", COMPLEX,
        Ev },
    { TYPE_NONE, CLASS_FSTPT, "fstpt", COMPLEX,
        Ev, NULL_OPERANDS_2, no_reg, no_reg, st0_reg, no_reg, 
        0, 0,
        FPU_C1, 0 },
    /* dc */
    { TYPE_NONE, CLASS_FADDL, "faddl", COMPLEX,
        NULL_OPERANDS_1, Ev, NULL_OPERANDS_1, st0_reg, no_reg, st0_reg, no_reg, 
        0, 0,
        FPU_C1, 0 },
    { TYPE_NONE, CLASS_FMULL, "fmull", COMPLEX,
        NULL_OPERANDS_1, Ev, NULL_OPERANDS_1, st0_reg, no_reg, st0_reg, no_reg, 
        0, 0,
        FPU_C1, 0 },
    { TYPE_NONE, CLASS_FCOML, "fcoml", COMPLEX,
        NULL_OPERANDS_1, Ev, NULL_OPERANDS_1, no_reg, no_reg, st0_reg,	no_reg, 
        0, 0,
        FPU_C0 | FPU_C2 | FPU_C3, 0 },
    { TYPE_NONE, CLASS_FCOMPL, "fcompl", COMPLEX,
        NULL_OPERANDS_1, Ev, NULL_OPERANDS_1, no_reg, no_reg, st0_reg,	no_reg, 
        0, 0,
        FPU_C0 | FPU_C2 | FPU_C3, 0 },
    { TYPE_NONE, CLASS_FSUBL, "fsubl", COMPLEX,
        NULL_OPERANDS_1, Ev, NULL_OPERANDS_1, st0_reg, no_reg, st0_reg, no_reg, 
        0, 0,
        FPU_C1, 0 },
    { TYPE_NONE, CLASS_FSUBRL, "fsubrl", COMPLEX,
        NULL_OPERANDS_1, Ev, NULL_OPERANDS_1, st0_reg, no_reg, st0_reg, no_reg, 
        0, 0,
        FPU_C1, 0 },
    { TYPE_NONE, CLASS_FDIVL, "fdivl", COMPLEX,
        NULL_OPERANDS_1, Ev, NULL_OPERANDS_1, st0_reg, no_reg, st0_reg, no_reg, 
        0, 0,
        FPU_C1, 0 },
    { TYPE_NONE, CLASS_FDIVRL, "fdivrl", COMPLEX,
        NULL_OPERANDS_1, Ev, NULL_OPERANDS_1, st0_reg, no_reg, st0_reg, no_reg, 
        0, 0,
        FPU_C1, 0 },
    /* dd */
    { TYPE_NONE, CLASS_FLDL, "fldl", COMPLEX,
        NULL_OPERANDS_1, Ev,  NULL_OPERANDS_1, st0_reg, no_reg, no_reg, no_reg, 
        0, 0,
        FPU_C1, 0 },
    { TYPE_NONE, CLASS_ERROR, "(bad)", COMPLEX,
        Ev },
    { TYPE_NONE, CLASS_FSTL, "fstl", COMPLEX,
        Ev, NULL_OPERANDS_2, no_reg, no_reg, st0_reg, no_reg, 
        0, 0,
        FPU_C1, 0 },
    { TYPE_NONE, CLASS_FSTPL, "fstpl", COMPLEX,
        Ev, NULL_OPERANDS_2, no_reg, no_reg, st0_reg, no_reg, 
        0, 0,
        FPU_C1, 0 },
    { TYPE_NONE, CLASS_FRSTOR, "frstor", COMPLEX,
        NULL_OPERANDS_1, Ev, NULL_OPERANDS_1, fpu_regs, no_reg, no_reg, no_reg, 
        0, 0,
        FPU_ALL, 0 },
    { TYPE_NONE, CLASS_ERROR, "(bad)", COMPLEX,
        Ev },
    { TYPE_NONE, CLASS_FNSAVE, "fNsave", COMPLEX,
        NULL_OPERANDS_1, Ev, NULL_OPERANDS_1, fpu_regs, no_reg, fpu_regs, no_reg,
        0, 0,
        FPU_ALL, 0 },
    { TYPE_NONE, CLASS_FNSTSW, "fNstsw", 3,
        Ev, NULL_OPERANDS_2, NO_REGISTERS_ALL, 
        0, 0,
        FPU_ALL, 0 },
    /* de */
    { TYPE_NONE, CLASS_FIADD, "fiadd", COMPLEX,
        NULL_OPERANDS_1, Ev, NULL_OPERANDS_1, st0_reg, no_reg, st0_reg, no_reg, 
        0, 0,
        FPU_C1, 0 },
    { TYPE_NONE, CLASS_FIMUL, "fimul", COMPLEX,
        NULL_OPERANDS_1, Ev, NULL_OPERANDS_1, st0_reg, no_reg, st0_reg, no_reg, 
        0, 0,
        FPU_C1, 0 },
    { TYPE_NONE, CLASS_FICOM, "ficom", COMPLEX,
        NULL_OPERANDS_1, Ev, NULL_OPERANDS_1, no_reg, no_reg, st0_reg,	no_reg, 
        0, 0,
        FPU_C0 | FPU_C2 | FPU_C3, 0 },
    { TYPE_NONE, CLASS_FICOMP, "ficomp", COMPLEX,
        NULL_OPERANDS_1, Ev, NULL_OPERANDS_1, no_reg, no_reg, st0_reg,	no_reg, 
        0, 0,
        FPU_C0 | FPU_C2 | FPU_C3, 0 },
    { TYPE_NONE, CLASS_FISUB, "fisub", COMPLEX,
        NULL_OPERANDS_1, Ev, NULL_OPERANDS_1, st0_reg, no_reg, st0_reg, no_reg, 
        0, 0,
        FPU_C1, 0 },
    { TYPE_NONE, CLASS_FISUBR, "fisubr", COMPLEX,
        NULL_OPERANDS_1, Ev, NULL_OPERANDS_1, st0_reg, no_reg, st0_reg, no_reg, 
        0, 0,
        FPU_C1, 0 },
    { TYPE_NONE, CLASS_FIDIV, "fidiv", COMPLEX,
        NULL_OPERANDS_1, Ev, NULL_OPERANDS_1, st0_reg, no_reg, st0_reg, no_reg, 
        0, 0,
        FPU_C1, 0 },
    { TYPE_NONE, CLASS_FIDIVR, "fidivr", COMPLEX,
        NULL_OPERANDS_1, Ev, NULL_OPERANDS_1, st0_reg, no_reg, st0_reg, no_reg, 
        0, 0,
        FPU_C1, 0 },
    /* df */
    { TYPE_NONE, CLASS_FILD, "fild", 4,
        NULL_OPERANDS_1, Ev, NULL_OPERANDS_1, st0_reg, no_reg, no_reg,	no_reg, 
        0, 0,
        FPU_C1, 0 },
    { TYPE_NONE, CLASS_ERROR, "(bad)", COMPLEX,
        Ev },
    { TYPE_NONE, CLASS_FIST, "fist", 4,
        Ev, NULL_OPERANDS_2, no_reg, no_reg, st0_reg, no_reg, 
        0, 0,
        FPU_C1, 0 },
    { TYPE_NONE, CLASS_FISTP, "fistp", 4,
        Ev, NULL_OPERANDS_2, no_reg, no_reg, st0_reg, no_reg, 
        0, 0,
        FPU_C1, 0 },
    { TYPE_NONE, CLASS_FBLD, "fbld", COMPLEX,
        NULL_OPERANDS_1, Ev, NULL_OPERANDS_1, st0_reg, no_reg, no_reg,	no_reg, 
        0, 0,
        FPU_C1, 0 },
    { TYPE_NONE, CLASS_FILDLL, "fildll", COMPLEX,
        NULL_OPERANDS_1, Ev, NULL_OPERANDS_1, st0_reg, no_reg, no_reg,	no_reg, 
        0, 0,
        FPU_C1, 0 },
    { TYPE_NONE, CLASS_FBSTP, "fbstp", COMPLEX,
        Ev, NULL_OPERANDS_2, no_reg, no_reg, st0_reg, no_reg, 
        0, 0,
        FPU_C1, 0 },
    { TYPE_NONE, CLASS_FISTPLL, "fistpll", COMPLEX,
        Ev, NULL_OPERANDS_2, no_reg, no_reg, st0_reg, no_reg, 
        0, 0,
        FPU_C1, 0 }
};

X86op float_reg[][8] = {
    /* d8 */
    {
        { TYPE_NONE, CLASS_FADD, "fadd", 2,
            ST, STi, NULL_OPERANDS_1, no_reg, no_reg, st0_reg, no_reg,
            0, 0,
            FPU_C1 | DUF, 0 },
        { TYPE_NONE, CLASS_FMUL, "fmul", 2,
            ST, STi, NULL_OPERANDS_1, no_reg, no_reg, st0_reg, no_reg, 0,
            0, FPU_C1, 0 },
        { TYPE_NONE, CLASS_FCOM, "fcom", 1,
            NULL_OPERANDS_1, STi, NULL_OPERANDS_1, no_reg, no_reg, st0_reg, no_reg, 
            0, 0,
            FPU_C0 | FPU_C2 | FPU_C3, 0 },
        { TYPE_NONE, CLASS_FCOMP, "fcomp", 1,
            NULL_OPERANDS_1, STi, NULL_OPERANDS_1, no_reg, no_reg, st0_reg, no_reg, 
            0, 0,
            FPU_C0 | FPU_C2 | FPU_C3, 0 },
        { TYPE_NONE, CLASS_FSUB, "fsub", 2,
            ST, STi, NULL_OPERANDS_1, no_reg, no_reg, st0_reg, no_reg,
            0, 0,
            FPU_C1 | DUF, 0 },
        { TYPE_NONE, CLASS_FSUBR, "fsubr", 2,
            ST, STi, NULL_OPERANDS_1, no_reg, no_reg, st0_reg, no_reg,
            0, 0,
            FPU_C1 | DUF, 0 },
        { TYPE_NONE, CLASS_FDIV, "fdiv", 2,
            ST, STi, NULL_OPERANDS_1, no_reg, no_reg, st0_reg, no_reg,
            0, 0,
            FPU_C1 | DUF, 0 },
        { TYPE_NONE, CLASS_FDIVR, "fdivr", 2,
            ST, STi, NULL_OPERANDS_1, no_reg, no_reg, st0_reg, no_reg,		
            0, 0,
            FPU_C1 | DUF, 0 },
    },
    /* d9 */
    {
        { TYPE_NONE, CLASS_FLD, "fld", 1,
            NULL_OPERANDS_1, STi, NULL_OPERANDS_1, st0_reg, no_reg, no_reg, no_reg, 
            0, 0,
            FPU_C1, 0 },
        { TYPE_NONE, CLASS_FXCH, "fxch", 1,
            STi, NULL_OPERANDS_2, st0_reg, no_reg, st0_reg, no_reg, 
            0, 0,
            FPU_C1 | DUF, 0 },
        { FGRPd9_2 },
        { TYPE_ERROR, CLASS_ERROR, "(bad)" },
        { FGRPd9_4 },
        { FGRPd9_5 },
        { FGRPd9_6 },
        { FGRPd9_7 },
    },
    /* da */
    {
        { TYPE_NONE, CLASS_FCMOVB, "fcmovb", 2,
            ST, STi, NULL_OPERANDS_1, NO_REGISTERS_ALL, 
            0, 0,
            FPU_C1, PSW_CF },
        { TYPE_NONE, CLASS_FCMOVE, "fcmove", 2,
            ST, STi, NULL_OPERANDS_1, NO_REGISTERS_ALL, 
            0, 0,
            FPU_C1, PSW_ZF },
        { TYPE_NONE, CLASS_FCMOVBE, "fcmovbe", 2,
            ST, STi, NULL_OPERANDS_1, NO_REGISTERS_ALL, 
            0, 0,
            FPU_C1, PSW_CF | PSW_ZF },
        { TYPE_NONE, CLASS_FCMOVU, "fcmovu", 2,
            ST, STi, NULL_OPERANDS_1, NO_REGISTERS_ALL, 
            0, 0,
            FPU_C1, PSW_PF },
        { TYPE_ERROR, CLASS_ERROR, "(bad)" },
        { FGRPda_5 },
        { TYPE_ERROR, CLASS_ERROR, "(bad)" },
        { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    },
    /* db */
    {
        { TYPE_NONE, CLASS_FCMOVNB, "fcmovnb", 2,
            ST, STi, NULL_OPERANDS_1, NO_REGISTERS_ALL, 
            0, 0,
            FPU_C1, PSW_CF },
        { TYPE_NONE, CLASS_FCMOVNE, "fcmovne", 2,
            ST, STi, NULL_OPERANDS_1, NO_REGISTERS_ALL, 
            0, 0,
            FPU_C1, PSW_ZF },
        { TYPE_NONE, CLASS_FCMOVNBE, "fcmovnbe", 2,
            ST, STi, NULL_OPERANDS_1, NO_REGISTERS_ALL, 
            0, 0,
            FPU_C1, PSW_CF | PSW_ZF },
        { TYPE_NONE, CLASS_FCMOVNU, "fcmovnu", 2,
            ST, STi, NULL_OPERANDS_1, NO_REGISTERS_ALL, 
            0, 0,
            FPU_C1, PSW_PF },
        { FGRPdb_4 },
        { TYPE_NONE, CLASS_FUCOMI, "fucomi", 1,
            NULL_OPERANDS_1, ST, STi, NO_REGISTERS_ALL, 
            0, 0,
            FPU_C1 | PSW_ZF | PSW_CF | PSW_PF, 0 },
        { TYPE_NONE, CLASS_FCOMI, "fcomi", 1,
            NULL_OPERANDS_1, ST, STi, NO_REGISTERS_ALL, 
            0, 0,
            FPU_C1 | PSW_ZF | PSW_CF | PSW_PF, 0 },
        { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    },
    /* dc */
    {
        { TYPE_NONE, CLASS_FADD, "fadd", 2,
            STi, ST, NULL_OPERANDS_1, no_reg, no_reg, no_reg, no_reg,
            0, 0,
            FPU_C1 | DUF, 0 },
        { TYPE_NONE, CLASS_FMUL, "fmul", 2,
            STi, ST, NULL_OPERANDS_1, no_reg, no_reg, no_reg, no_reg,
            0, 0,
            FPU_C1 | DUF, 0 },
        { TYPE_ERROR, CLASS_ERROR, "(bad)" },
        { TYPE_ERROR, CLASS_ERROR, "(bad)" },
        { TYPE_NONE, CLASS_FSUB, "fsub", 2,
            STi, ST, NULL_OPERANDS_1, NO_REGISTERS_ALL, 
            0, 0,
            FPU_C1 | DUF, 0 },
        { TYPE_NONE, CLASS_FSUBR, "fsubr", 2,
            STi, ST, NULL_OPERANDS_1, NO_REGISTERS_ALL, 
            0, 0,
            FPU_C1 | DUF, 0 },
        { TYPE_NONE, CLASS_FDIV, "fdiv", 2,
            STi, ST, NULL_OPERANDS_1, no_reg, no_reg, no_reg, no_reg, 
            0, 0,
            FPU_C1 | DUF, 0 },
        { TYPE_NONE, CLASS_FDIVR, "fdivr", 2,
            STi, ST, NULL_OPERANDS_1, no_reg, no_reg, no_reg, no_reg, 
            0, 0,
            FPU_C1 | DUF, 0 },
    },
    /* dd */
    {
        { TYPE_NONE, CLASS_FFREE, "ffree", 1,
            STi, NULL_OPERANDS_2, NO_REGISTERS_ALL, 
            0, 0,
            0, 0 },
        { TYPE_ERROR, CLASS_ERROR, "(bad)" },
        { TYPE_NONE, CLASS_FST, "fst", 1,
            STi, NULL_OPERANDS_2, no_reg, no_reg, st0_reg, no_reg, 
            0, 0,
            FPU_C1, 0 },
        { TYPE_NONE, CLASS_FSTP, "fstp", 1,
            STi, NULL_OPERANDS_2, no_reg, no_reg, st0_reg, no_reg, 
            0, 0,
            FPU_C1, 0 },
        { TYPE_NONE, CLASS_FUCOM, "fucom", 1,
            STi, NULL_OPERANDS_2, no_reg, no_reg, st0_reg, no_reg, 
            0, 0,
            FPU_C0 | FPU_C1 | FPU_C2 | FPU_C3, 0 },
        { TYPE_NONE, CLASS_FUCOMP, "fucomp", 1,
            STi, NULL_OPERANDS_2, no_reg, no_reg, st0_reg, no_reg, 
            0, 0,
            FPU_C0 | FPU_C1 | FPU_C2 | FPU_C3, 0 },
        { TYPE_ERROR, CLASS_ERROR, "(bad)" },	// fstsw
        { TYPE_ERROR, CLASS_ERROR, "(bad)" },	// fstsw
    },
    /* de */
    {
        { TYPE_NONE, CLASS_FADDP, "faddp", 1,
            STi, ST, NULL_OPERANDS_1, no_reg, no_reg, st0_reg, no_reg,
            0, 0,
            FPU_C1 | DUF, 0 },
        { TYPE_NONE, CLASS_FMULP, "fmulp", 1,
            STi, ST, NULL_OPERANDS_1, no_reg, no_reg, st0_reg, no_reg,
            0, 0,
            FPU_C1 | DUF, 0 },
        { TYPE_ERROR, CLASS_ERROR, "(bad)" },
        { FGRPde_3 },
        { TYPE_NONE, CLASS_FSUBP, "fsubp", 1,
            STi, ST, NULL_OPERANDS_1, NO_REGISTERS_ALL, 
            0, 0,
            FPU_C1 | DUF, 0 },
        { TYPE_NONE, CLASS_FSUBRP, "fsubrp", 1,
            STi, ST, NULL_OPERANDS_1, NO_REGISTERS_ALL, 
            0, 0,
            FPU_C1 | DUF, 0 },
        { TYPE_NONE, CLASS_FDIVP, "fdivp", 1,
            STi, ST, NULL_OPERANDS_1, no_reg, no_reg, no_reg, no_reg, 
            0, 0,
            FPU_C1 | DUF, 0 },
        { TYPE_NONE, CLASS_FDIVRP, "fdivrp", 1,
            STi, ST, NULL_OPERANDS_1, NO_REGISTERS_ALL, 
            0, 0,
            FPU_C1 | DUF, 0 },
    },
    /* df */
    {
        { TYPE_ERROR, CLASS_ERROR, "(bad)" },
        { TYPE_ERROR, CLASS_ERROR, "(bad)" },
        { TYPE_ERROR, CLASS_ERROR, "(bad)" },
        { TYPE_ERROR, CLASS_ERROR, "(bad)" },
        { FGRPdf_4 },
        { TYPE_NONE, CLASS_FUCOMIP, "fucomip", 1,
            NULL_OPERANDS_1, ST, STi, NO_REGISTERS_ALL, 
            0, 0,
            FPU_C1 | PSW_ZF | PSW_CF | PSW_PF, 0 },
        { TYPE_NONE, CLASS_FCOMIP, "fcomip", 1,
            NULL_OPERANDS_1, ST, STi, NO_REGISTERS_ALL, 
            0, 0,
            FPU_C1 | PSW_ZF | PSW_CF | PSW_PF, 0 },
        { TYPE_ERROR, CLASS_ERROR, "(bad)" },
    },
};

X86op fgrps[][8] = {
    /* d9_2  0 */
    {
        { TYPE_NONE, CLASS_FNOP, "fnop", 1,
            NULL_OPERANDS_3, NO_REGISTERS_ALL, 
            0, 0,
            0, 0 },
        { TYPE_NONE, CLASS_ERROR, "(bad)" },
        { TYPE_NONE, CLASS_ERROR, "(bad)" },
        { TYPE_NONE, CLASS_ERROR, "(bad)" },
        { TYPE_NONE, CLASS_ERROR, "(bad)" },
        { TYPE_NONE, CLASS_ERROR, "(bad)" },
        { TYPE_NONE, CLASS_ERROR, "(bad)" },
        { TYPE_NONE, CLASS_ERROR, "(bad)" }
    },

    /* d9_4  1 */
    {
        { TYPE_NONE, CLASS_FCHS, "fchs", 3,
            NULL_OPERANDS_3, st0_reg, no_reg, st0_reg, no_reg,
            0, 0,
            FPU_C1, 0 },
        { TYPE_NONE, CLASS_FABS, "fabs", 1,
            NULL_OPERANDS_3, st0_reg, no_reg, st0_reg, no_reg,
            0, 0,
            FPU_C1, 0 },
        { TYPE_NONE, CLASS_ERROR, "(bad)" },
        { TYPE_NONE, CLASS_ERROR, "(bad)" },
        { TYPE_NONE, CLASS_FTST, "ftst", 1,
            NULL_OPERANDS_3, no_reg, no_reg, st0_reg, no_reg,
            0, 0,
            FPU_C0 | FPU_C2 | FPU_C3, 0 },
        { TYPE_NONE, CLASS_FXAM, "fxam", 1,
            NULL_OPERANDS_3, no_reg, no_reg, st0_reg, no_reg,
            0, 0,
            FPU_C0 | FPU_C1 | FPU_C2 | FPU_C3, 0 },
        { TYPE_NONE, CLASS_ERROR, "(bad)" },
        { TYPE_NONE, CLASS_ERROR, "(bad)" }
    },

    /* d9_5  2 */
    {
        { TYPE_NONE, CLASS_FLD1, "fld1", 2,
            NULL_OPERANDS_3, st0_reg, no_reg, no_reg, no_reg, 
            0, 0,
            FPU_C1, 0 },
        { TYPE_NONE, CLASS_FLDL2T, "fldl2t", 2,
            NULL_OPERANDS_3, st0_reg, no_reg, no_reg, no_reg, 
            0, 0,
            FPU_C1, 0 },
        { TYPE_NONE, CLASS_FLDL2E, "fldl2e", 2,
            NULL_OPERANDS_3, st0_reg, no_reg, no_reg, no_reg, 
            0, 0,
            FPU_C1, 0 },
        { TYPE_NONE, CLASS_FLDPI, "fldpi", 2,
            NULL_OPERANDS_3, st0_reg, no_reg, no_reg, no_reg, 
            0, 0,
            FPU_C1, 0 },
        { TYPE_NONE, CLASS_FLDLG2, "fldlg2", 2,
            NULL_OPERANDS_3, st0_reg, no_reg, no_reg, no_reg, 
            0, 0,
            FPU_C1, 0 },
        { TYPE_NONE, CLASS_FLDLN2, "fldln2", 2,
            NULL_OPERANDS_3, st0_reg, no_reg, no_reg, no_reg, 
            0, 0,
            FPU_C1, 0 },
        { TYPE_NONE, CLASS_FLDZ, "fldz", 1,
            NULL_OPERANDS_3, st0_reg, no_reg, no_reg, no_reg, 
            0, 0,
            FPU_C1, 0 },
        { TYPE_NONE, CLASS_ERROR, "(bad)" }
    },

    /* d9_6  3 */
    {
        { TYPE_NONE, CLASS_F2XM1, "f2xm1", COMPLEX,
            NULL_OPERANDS_3, st0_reg, no_reg, st0_reg, no_reg, 
            0, 0,
            FPU_C1, 0 },
        { TYPE_NONE, CLASS_FYL2X, "fyl2x", COMPLEX,
            NULL_OPERANDS_3, st1_reg, no_reg, st1_reg, st0_reg, 
            0, 0,
            FPU_C1, 0 },
        { TYPE_NONE, CLASS_FPTAN, "fptan", COMPLEX,
            NULL_OPERANDS_3, st0_reg, no_reg, st0_reg, no_reg, 
            0, 0,
            FPU_C1 | FPU_C2, 0 },
        { TYPE_NONE, CLASS_FPATAN, "fpatan", COMPLEX,
            NULL_OPERANDS_3, st1_reg, no_reg, st1_reg, st0_reg, 
            0, 0,
            FPU_C1, 0 },
        { TYPE_NONE, CLASS_FXTRACT, "fxtract", COMPLEX,
            NULL_OPERANDS_3, st0_reg, no_reg, st0_reg, no_reg, 
            0, 0,
            FPU_C1, 0 },
        { TYPE_NONE, CLASS_FPREM1, "fprem1", COMPLEX,
            NULL_OPERANDS_3, st0_reg, no_reg, st0_reg, st1_reg, 
            0, 0,
            FPU_C0 | FPU_C1 | FPU_C2 | FPU_C3, 0 },
        { TYPE_NONE, CLASS_FDECSTP, "fdecstp", 1,
            NULL_OPERANDS_3, NO_REGISTERS_ALL, 
            0, 0,
            FPU_C1, 0 },
        { TYPE_NONE, CLASS_FINCSTP, "fincstp", 1,
            NULL_OPERANDS_3, NO_REGISTERS_ALL, 
            0, 0,
            FPU_C1, 0 }
    },

    /* d9_7  4 */
    {
        { TYPE_NONE, CLASS_FPREM, "fprem", COMPLEX,
            NULL_OPERANDS_3, st0_reg, no_reg, st0_reg, st1_reg, 
            0, 0,
            FPU_C0 | FPU_C1 | FPU_C2 | FPU_C3, 0 },
        { TYPE_NONE, CLASS_FYL2XP1, "fyl2xp1", COMPLEX,
            NULL_OPERANDS_3, st1_reg, no_reg, st1_reg, st0_reg, 
            0, 0,
            FPU_C1, 0 },
        { TYPE_NONE, CLASS_FSQRT, "fsqrt", 1,
            NULL_OPERANDS_3, st0_reg, no_reg, st0_reg, no_reg, 
            0, 0,
            FPU_C1, 0 },
        { TYPE_NONE, CLASS_FSINCOS, "fsincos", COMPLEX,
            NULL_OPERANDS_3, st0_reg, no_reg, st0_reg, no_reg, 
            0, 0,
            FPU_C1 | FPU_C2, 0 },
        { TYPE_NONE, CLASS_FRNDINT, "frndint", COMPLEX,
            NULL_OPERANDS_3, st0_reg, no_reg, st0_reg, no_reg, 
            0, 0,
            FPU_C1, 0 },
        { TYPE_NONE, CLASS_FSCALE, "fscale", COMPLEX,
            NULL_OPERANDS_3, st0_reg, no_reg, st0_reg, st1_reg, 
            0, 0,
            FPU_C1, 0 },
        { TYPE_NONE, CLASS_FSIN, "fsin", COMPLEX,
            NULL_OPERANDS_3, st0_reg, no_reg, st0_reg, no_reg, 
            0, 0,
            FPU_C1 | FPU_C2, 0 },
        { TYPE_NONE, CLASS_FCOS, "fcos", COMPLEX,
            NULL_OPERANDS_3, st0_reg, no_reg, st0_reg, no_reg, 
            0, 0,
            FPU_C1 | FPU_C2, 0 }
    },

    /* da_5  5 */
    {
        { TYPE_NONE, CLASS_ERROR, "(bad)" },
        { TYPE_NONE, CLASS_FUCOMPP, "fucompp", 2,
            NULL_OPERANDS_3, no_reg, no_reg, st0_reg, st1_reg, 
            0, 0,
            FPU_C0 | FPU_C1 | FPU_C2 | FPU_C3, 0 },
        { TYPE_NONE, CLASS_ERROR, "(bad)" },
        { TYPE_NONE, CLASS_ERROR, "(bad)" },
        { TYPE_NONE, CLASS_ERROR, "(bad)" },
        { TYPE_NONE, CLASS_ERROR, "(bad)" },
        { TYPE_NONE, CLASS_ERROR, "(bad)" },
        { TYPE_NONE, CLASS_ERROR, "(bad)" }
    },

    /* db_4  6 */
    {
        { TYPE_NONE, CLASS_ERROR, "(bad)" },
        { TYPE_NONE, CLASS_ERROR, "(bad)" },
        { TYPE_NONE, CLASS_FNCLEX, "fNclex", 3,
            NULL_OPERANDS_3, NO_REGISTERS_ALL, 
            0, 0,
            0, 0 },
        { TYPE_NONE, CLASS_FNINIT, "fNinit", COMPLEX,
            NULL_OPERANDS_3, NO_REGISTERS_ALL, 
            0, 0,
            FPU_C0 | FPU_C1 | FPU_C2 | FPU_C3, 0 },
        { TYPE_NONE, CLASS_ERROR, "(bad)" },
        { TYPE_NONE, CLASS_ERROR, "(bad)" },
        { TYPE_NONE, CLASS_ERROR, "(bad)" },
        { TYPE_NONE, CLASS_ERROR, "(bad)" }
    },

    /* de_3  7 */
    {
        { TYPE_NONE, CLASS_ERROR, "(bad)" },
        { TYPE_NONE, CLASS_FCOMPP, "fcompp", 2,
            NULL_OPERANDS_3, no_reg, no_reg, st1_reg, st0_reg, 
            0, 0,
            FPU_C0 | FPU_C2 | FPU_C3, 0 },
        { TYPE_NONE, CLASS_ERROR, "(bad)" },
        { TYPE_NONE, CLASS_ERROR, "(bad)" },
        { TYPE_NONE, CLASS_ERROR, "(bad)" },
        { TYPE_NONE, CLASS_ERROR, "(bad)" },
        { TYPE_NONE, CLASS_ERROR, "(bad)" },
        { TYPE_NONE, CLASS_ERROR, "(bad)" }
    },

    /* df_4  8 */
    {
        { TYPE_NONE, CLASS_FNSTSW, "fNstsw", 3,
            NULL_OPERANDS_3, ax_reg, no_reg, no_reg, no_reg, 
            0, 0,
            0, FPU_ALL },
        { TYPE_NONE, CLASS_ERROR, "(bad)" },
        { TYPE_NONE, CLASS_ERROR, "(bad)" },
        { TYPE_NONE, CLASS_ERROR, "(bad)" },
        { TYPE_NONE, CLASS_ERROR, "(bad)" },
        { TYPE_NONE, CLASS_ERROR, "(bad)" },
        { TYPE_NONE, CLASS_ERROR, "(bad)" },
        { TYPE_NONE, CLASS_ERROR, "(bad)" }

    }
};
