/*
 *  x86-icreate.h
 *
 *  Creates and returns instructions for the x86 instruction set
 */
#ifndef X86_ICREATE_H
#define X86_ICREATE_H

extern Instruction *xInstrCreateNOP();		// NOP
extern Instruction *xInstrCreateJMP();		// JMP
extern Instruction *xInstrCreateSave();		// PUSHF
extern Instruction *xInstrCreateRestore();	// POPF
extern Instruction *_InstrCreate32BitMemINC(ADDRESS addr);  // INC  (addr)
extern Instruction *_InstrCreate32BitMemAdd(ADDRESS addr);  // ADD  X -> (addr)
extern Instruction *_InstrCreate32BitMemAdc(ADDRESS addr);  // ADD  X -> (addr)
extern Instruction *_InstrCreate32BitMovRegToMem(int reg);
extern Instruction *_InstrCreate32BitMovMemToReg(int reg);
extern Instruction *_InstrCreate32BitAddToReg(int reg, int value);
extern Instruction *xInstrCreateAddImmToIndirectReg(int imm_value, int reg);
extern Instruction *xInstrCreateCALLDisp();	  // CALL
//Igor, next 1
extern Instruction *xInstrCreateCallNearMemBase(int reg);
extern Instruction *xInstrCreateSPInc();	  // add IMMEDIATE, %esp
extern Instruction *xInstrCreateSPDec();	  // sub IMMEDIATE, %esp
extern Instruction *xInstrCreateBPAdd();	  // add IMMEDIATE, %ebp
extern Instruction *xInstrCreateMovBPToSP();	  // mov %esp <- %ebp
extern Instruction *_InstrCreate1ByteDoNothing(); // NOP
extern Instruction *_InstrCreate2ByteDoNothing(); // mov %esi,%esi
extern Instruction *_InstrCreate3ByteDoNothing(); // lea 0x0(%esi), %esi
extern Instruction *_InstrCreate4ByteDoNothing(); // lea 0x0(%esi,1),%esi b_mode
extern Instruction *_InstrCreate7ByteDoNothing(); // lea 0x0(%esi,1),%esi v_mode
extern Instruction *xInstrCreateCmovRegToReg();	  // cmov %esi <- %esi
extern Instruction *xInstrCreateCmovMemToReg();	  // cmov %esi <- mem
extern Instruction *xInstrCreateMovImmToReg(int reg, int value); // mov reg<-IMM
extern Instruction *xInstrCreatePushAll();
extern Instruction *xInstrCreatePopAll();

extern Instruction *xInstrCreatePush(int reg);		// push %<reg>
extern Instruction *xInstrCreatePushImm(int value);	// push $0x<value>
extern Instruction *xInstrCreatePushMem(int address);	// pushl 0x<addr>
extern Instruction *xInstrCreatePushIndMem(int reg);	// pushl (%<reg>)

extern Instruction *xInstrCreatePushMemBase(int reg);   // pushl (%<reg>)

extern Instruction *xInstrCreatePushIndMemDisp(int reg,int value); // pushl v(r)
//Igor, next 2
extern Instruction *xInstrCreatePushMemDisp8Base(INT8 disp8, int reg);
extern Instruction *xInstrCreatePushMemDisp32Base(INT32 disp32, int reg);

extern Instruction *xInstrCreatePushIndMemBaseIndex(
    int base, int index, int scale);
    // pushl (base, index, scale) or pushl (base, scale) if no index // Push_07

extern Instruction *_InstrCreatePushIndMemBaseDisp32(int base, int disp32);
    // pushl disp32(base) // Push_10

extern Instruction *xInstrCreatePop(int reg);

extern Instruction *xInstrCreateSafeStackAdd();

/* XXX hack, fix this */ //replaced by Push_09
extern Instruction *xInstrCreatePushStack(int reg);

/* XXX hack, fix this */
//extern Instruction *xInstrCreateCmpStack(int reg, int value);
extern Instruction *xInstrCreateCmpStackToValue(int offset, int value);
extern Instruction *xInstrCreateCmpRegToValue(int reg, int value);
//Igor, next 12
extern Instruction *xInstrCreateCmpImm8WithReg(INT8 imm8, int reg);
extern Instruction *xInstrCreateCmpImm32WithReg(INT32 imm32, int reg);
extern Instruction *xInstrCreateCmpImm8WithMemAbs(INT8 imm8, int offset);
extern Instruction *xInstrCreateCmpImm32WithMemAbs(INT32 imm32, int offset);
extern Instruction *xInstrCreateCmpImm8WithMemBase(INT8 imm8, int reg);
extern Instruction *xInstrCreateCmpImm32WithMemBase(INT32 imm32, int reg);
extern Instruction *xInstrCreateCmpImm8WithMemDisp8Base(INT8 imm8,
    INT8 disp8, int reg);
extern Instruction *xInstrCreateCmpImm32WithMemDisp8Base(INT32 imm32,
    INT8 disp8, int reg);
extern Instruction *xInstrCreateCmpImm8WithMemDisp32Base(INT8 imm8,
    INT32 disp32, int reg);
extern Instruction *xInstrCreateCmpImm32WithMemDisp32Base(INT32 imm32,
    INT32 disp32, int reg);
extern Instruction *xInstrCreateCmpImm8WithMemBaseIndex(INT8 imm8, int base,
    int index, int scale);
extern Instruction *xInstrCreateCmpImm32WithMemBaseIndex(INT32 imm32, int base,
    int index, int scale);
//Igor, prev 12
extern Instruction *xInstrCreateJne();
extern Instruction *xInstrCreateJe();

extern Instruction *xInstrCreateCondMov(Instruction *jcc_ins, bool dir);

extern Instruction *_InstrCreate32BitLea();
extern Instruction *_InstrCreate32BitAnd(int reg, int value);
extern Instruction *_InstrCreate32BitShlImm(UINT32 reg, int times);
extern Instruction *_InstrCreate32BitShr(int reg, int value);
extern Instruction *xInstrCreate32BitShlCL(UINT32 reg);
extern Instruction *xInstrCreate32BitShrCL(UINT32 reg);
//Igor, next 8
extern Instruction *xInstrCreateAddRegToReg(int src_reg, int dest_reg);
extern Instruction *xInstrCreateAddImm8ToReg(INT8 imm8, int dest_reg);
extern Instruction *xInstrCreateAddImm32ToReg(INT32 imm32, int dest_reg);
extern Instruction *xInstrCreateSubImm8FromReg(INT8 imm8, int dest_reg);
extern Instruction *xInstrCreateSubImm32FromReg(INT32 imm32, int dest_reg);
extern Instruction *xInstrCreateAndImmWithReg(int imm, int dest_reg);
extern Instruction *xInstrCreateOrRegWithMemBaseIndex(int src, int base,
    int index, UINT8 scale);
extern Instruction *xInstrCreateOrMemBaseIndexWithReg(int base, int index,
    UINT8 scale, int dest);

extern Instruction *xInstrCreateXorRegWithMemDisp8Base(int src,
    INT8 dest_disp, int dest);
extern Instruction *xInstrCreateXorRegWithMemDisp32Base(int src,
    INT32 dest_disp, int dest);

/* movl off(s_reg), d_reg */ //same as xInstrCreateMovMemDisp8BaseToReg
extern Instruction *_InstrCreate32BitMovMemDispToReg(int s_reg, INT8 off,
    int d_reg); //to be replaced
extern Instruction *xInstrCreateMovRegToReg(int src_reg, int dest_reg);
//Igor, next 13
extern Instruction *xInstrCreateMovRegToMemBase(int src, int dest);
extern Instruction *xInstrCreateMovMemBaseToReg(int src, int dest);
extern Instruction *xInstrCreateMovRegToMemDisp8Base(int src, INT8 dest_disp,
    int dest);
extern Instruction *xInstrCreateMovRegToMemDisp32Base(int src, INT32 dest_disp,
    int dest);
extern Instruction *xInstrCreateMovMemDisp8BaseToReg(INT8 src_disp, int src,
    int dest);
extern Instruction *xInstrCreateMovMemDisp32BaseToReg(INT32 src_disp, int src,
    int dest);
extern Instruction *xInstrCreateMovRegToMemBaseIndex(int src, int base,
    int index, UINT8 scale);
extern Instruction *xInstrCreateMovMemBaseIndexToReg(int base, int index,
    UINT8 scale, int dest);
extern Instruction *xInstrCreateMovImmToMemBase(int imm, int dest_reg);
extern Instruction *xInstrCreateMovImmToMemDisp8Base(int imm, INT8 disp,
    int dest);
extern Instruction *xInstrCreateMovImmToMemDisp32Base(int imm, INT32 disp,
    int dest);
extern Instruction *xInstrCreateMovImmToMemBaseIndex(int imm, int base,
    int index, int scale);
extern Instruction *xInstrCreateMovzReg8ToReg32(UINT32 src_reg,
    UINT32 dest_reg);
//Igor, prev 13
extern Instruction *xInstrCreateMovMemToReg(int scale,
    UINT32 index_reg, UINT32 base_reg, UINT32 dest_reg);
extern Instruction *xInstrCreateSubImmFromReg(int imm_value, UINT32 reg);
extern Instruction *xInstrCreateMovAbsoluteMemToReg(ADDRESS memloc, int reg);
extern Instruction *_InstrCreate32BitMovImmToMem(ADDRESS mem_loc, int imm);
extern Instruction *_InstrCreate32BitAddRegToMem(ADDRESS mem_loc, int reg);
extern Instruction *_InstrCreate32BitSubRegFromMem(ADDRESS mem_loc, int reg);
extern Instruction *_InstrCreate32BitAddImmToMem(ADDRESS mem_loc, int imm);
extern Instruction *_InstrCreate32BitAddMemToReg(ADDRESS mem_loc, UINT32 reg);
extern Instruction *xInstrCreateSubMemFromReg(ADDRESS mem_loc, UINT32 reg);
extern Instruction *xInstrCreateSubImmFromMem(int imm, ADDRESS mem_loc);
extern Instruction *_InstrCreate16BitAddRegToMem(ADDRESS mem_loc, int reg);
extern Instruction *_InstrCreate16BitSubRegFromMem(ADDRESS mem_loc, int reg);
extern Instruction *xInstrCreateLeave();
extern Instruction *xInstrCreateReturn();
extern Instruction *_InstrCreate32BitAddImmToReg(int imm, int reg);
extern Instruction *xInstrCreateSubRegFromReg(UINT32 src_reg, UINT32 dest_reg);
extern Instruction *xInstrCreateIMULRegReg(UINT32 src_reg, UINT32 dest_reg);
//extern Instruction *xInstrCreateIMULImmReg(
//    int imm, UINT32 src_reg, UINT32 dest_reg);	// Igor

extern Instruction *xInstrCreateInc(UINT32 reg);
extern Instruction *xInstrCreateDec(UINT32 reg);
extern Instruction *xInstrCreateCmpRegToReg(UINT32 reg1, UINT32 reg2);
//Igor, next 23
extern Instruction *xInstrCreateShortJb();
extern Instruction *xInstrCreateShortJae();
extern Instruction *xInstrCreateShortJe();
extern Instruction *xInstrCreateShortJne();
extern Instruction *xInstrCreateShortJbe();
extern Instruction *xInstrCreateShortJa();
extern Instruction *xInstrCreateShortJs();
extern Instruction *xInstrCreateShortJns();
extern Instruction *xInstrCreateShortJl();
extern Instruction *xInstrCreateShortJge();
extern Instruction *xInstrCreateShortJle();
extern Instruction *xInstrCreateShortJg();
extern Instruction *xInstrCreateShortJecxz();
extern Instruction *xInstrCreateNearJb();
extern Instruction *xInstrCreateNearJae();
extern Instruction *xInstrCreateNearJbe();
extern Instruction *xInstrCreateNearJa();
extern Instruction *xInstrCreateNearJs();
extern Instruction *xInstrCreateNearJns();
extern Instruction *xInstrCreateNearJl();
extern Instruction *xInstrCreateNearJge();
extern Instruction *xInstrCreateNearJle();
extern Instruction *xInstrCreateNearJg();

extern Instruction *xInstrCreateHalt();
extern Instruction *xInstrCreateSetCarry();
extern Instruction *xInstrCreateClearCarry();
extern Instruction *xInstrCreateJumpOnNoCarry();
extern Instruction *xInstrCreateCallIndirect();
extern Instruction *xInstrCreateJMPIndirectAbsMemAddr();
extern Instruction *_InstrCreate32BitPopToMem();
extern Instruction *xInstrCreateDiv(int divisor); //this is IDiv - signed
extern Instruction *xInstrCreateSyscall();
//Igor, next 15
extern Instruction *xInstrCreateTestRegWithReg(UINT32 reg1, UINT32 reg2);
extern Instruction *xInstrCreateTestImmWithReg(int imm, UINT32 reg);
extern Instruction *xInstrCreateLea(UINT32 src, int offset, UINT32 dest);
extern Instruction *xInstrCreateSeteReg(UINT32 reg);

extern Instruction *xInstrCreate8bitDivByReg(int divisor);
extern Instruction *xInstrCreate32bitDivByReg(int divisor);
extern Instruction *xInstrCreate8bitIDivByReg(int divisor);
extern Instruction *xInstrCreate32bitIDivByReg(int divisor);
extern Instruction *xInstrCreate8bitMulByReg(int multiplier);
extern Instruction *xInstrCreate32bitMulByReg(int multiplier);
extern Instruction *xInstrCreate8bitIMulByReg(int multiplier); 
extern Instruction *xInstrCreate32bitIMulByReg(int multiplier);

extern Instruction *xInstrCreateIMulImm8ByReg(INT8 imm8, int reg);
extern Instruction *xInstrCreateIMulImm32ByReg(INT32 imm32, int reg);

extern Instruction *xInstrCreateJmpNearAbsDisp32Index(INT32 disp32, int index,
    int scale);
//Igor, prev 15
#endif				// X86_ICREATE_H
