/*
 *  x86-op.h
 *
 *  Operand query routines
 */
#ifndef X86_OP_H
#define X86_OP_H

#include "x86-table-defs.h"

/* Removes all the bytemode flags in the operand 'op' */
void OperandClearBytemodeFlags(Operand *op);

/* Returns true if the operand uses a register */
bool xOperandUsesRegister(Operand *op);

/* Returns the number of bytes the operand will take up */
int getOperandBytes(Operand *op, int dflag);

/* Returns true if the operand is a general purpose register */
bool xOperandIsGPRegister(Operand *op);

/*
 *  Returns the bytemode of the operand. Either v_mode (double-word),
 *  w_mode (word), b_mode (byte_mode), 0 (no mode) or d_mode (double-word).
 *  Indeed, there are likely two double-word bytemodes.
 *  v_mode can also be a single word if an instruction prefix
 *  is given (but it never is).
 */
int GetByteMode(Operand *op, int dflag);

/*
 *  Looks at the operands and attempts to guess (intelligently)
 *  at what op routines might be used. Eventually this should replace
 *  GetRoutineFromFlags, as setting flags isn't a very good way to do this
 */
op_rtn GetOpRoutine(Instruction *instr, Operand *op, int bytemode1,
    int bytemode2);

/* Returns true if the give operand uses a scale-index-base byte */
bool OperandHasSIB(Operand *op);

/* Returns true if the operand access the stack using EBP */
bool xOperandAccessesStackFromFramePointer(Operand *op);

/* Returns true if the operand accesses the stack
 * using EBP along with some other register in SIB mode */
bool xOperandAccessesStackFromFramePointerWithSIB(Operand *op);

/* Returns true if the operand access the stack using ESP */
bool xOperandAccessesStackFromStackPointer(Operand *op);

/*
 *  Returns true if the operand is an immediate value (constant).  Relocatable
 *  immediate values are not considered constants here, so they return false.
 */
bool xOperandIsImmediate(Operand *op);

/* Returns true if the operand is an address */
bool xOperandIsAddress(Operand *op);

/* Returns true if 'op' uses the contents of ESP */
bool xOperandUsesContentsOfStackPointer(Operand *op);

/* Returns true if 'op' uses the contents of EBP */
bool xOperandUsesContentsOfFramePointer(Operand *op);

/* Returns true if 'op' is ESP */
bool xOperandIsStackPointer(Operand *op);

/* Returns true if 'op' is EBP */
bool xOperandIsFramePointer(Operand *op);

/* Returns true if the operand is an absolute memory location */
bool xOperandIsAbsoluteMemoryLocation(Operand *op);

/*
 *  Returns true if the operand ultimately is a memory location, e.g:
 *  (%eax), (0x80450100), (eax, ebx, 4), 0x80481812
 */
bool xOperandIsMemory(Operand *op);

/*
 *  Returns true if the operand is a memory address that may be
 *  the runtime stack, but we can't determine if it is
 */
bool xOperandWritesToUnknownStackSlot(Operand *op);

/* Sets the operand to use the largest bytemode */
void xOperandUseLargestBytemode(Operand *op);

#endif				// X86_OP_H
