/*
 *  x86-prefix.h
 *
 *  This header contains prefix codes that can come before the opcode
 *  in a 386 instruction.  X86_PREFIX* are the actual codes, and PREFIX
 *  correspond to the bit that is set in the instruction field's prefix area.
 *  See the Intel Developer's Manual Volume 2, Chapter 2, entitled
 *  "Instruction Format" for more information about the specifics of prefixes.
 */
#ifndef X86_PREFIX_H
#define X86_PREFIX_H

/*
 *  Lock and Repeat Prefixes
 */
/* Definitions for the REPE/REPZ prefix */
#define X86_PREFIX_REPZ  0xf3
#define PREFIX_REPZ       0x1

/* Definitions for the REPNE/REPNZ/REP prefix */
#define X86_PREFIX_REPNZ 0xf2
#define PREFIX_REPNZ      0x2

/* Definitions for the LOCK prefix */
#define X86_PREFIX_LOCK  0xf0
#define PREFIX_LOCK       0x4

/*
 *  Segment override prefixes
 */
/* Definitions for the CS prefix */
#define X86_PREFIX_CS 0x2e
#define PREFIX_CS      0x8

/* Definitions for the SS prefix */
#define X86_PREFIX_SS 0x36
#define PREFIX_SS      0x10

/* Definitions for the DS prefix */
#define X86_PREFIX_DS 0x3e
#define PREFIX_DS      0x20

/* Definitions for the ES prefix */
#define X86_PREFIX_ES 0x26
#define PREFIX_ES      0x40

/* Definitions for the FS prefix */
#define X86_PREFIX_FS 0x64
#define PREFIX_FS      0x80

/* Definitions for the GS prefix */
#define X86_PREFIX_GS 0x65
#define PREFIX_GS      0x100

/*
 *  Operand Size Override prefixes
 */
#define X86_PREFIX_OPERAND 0x66
#define PREFIX_OPERAND      0x200

/*
 *  Address Size Override prefixes
 */
#define X86_PREFIX_ADDRESS 0x67
#define PREFIX_ADDRESS      0x400

/*
 *  Miscellaneous prefixes
 */
/* Definitions for the FWAIT prefix */
#define X86_PREFIX_FWAIT 0x9b
#define PREFIX_FWAIT     0x800

#endif				// X86_PREFIX_H
